On peut subdiviser les types numériques en deux sous-catégories : les nombres entiers, et les nombres décimaux.


# Nombres entiers

Les types de données qui acceptent des nombres entiers comme valeur sont désignés par le mot-clé `INT`, et ses déclinaisons `TINYINT`, `SMALLINT`, `MEDIUMINT` et `BIGINT`. La différence entre ces types est le nombre d'octets (donc la place en mémoire) réservés à la valeur du champ. Voici un tableau reprenant ces informations, ainsi que l'intervalle dans lequel la valeur peut être comprise pour chaque type.




+-----------+---------------+--------------------+-------------------+
|Type       |Nombre d'octets|Minimum             |Maximum            |
+===========+===============+====================+===================+
|`TINYINT`  |1              |-128                |127                |
+-----------+---------------+--------------------+-------------------+
|`SMALLINT` |2              |-32768              |32767              |
+-----------+---------------+--------------------+-------------------+
|`MEDIUMINT`|3              |-8388608            |8388607            |
+-----------+---------------+--------------------+-------------------+
|`INT`      |4              |-2147483648         |2147483647         |
+-----------+---------------+--------------------+-------------------+
|`BIGINT`   |8              |-9223372036854775808|9223372036854775807|
+-----------+---------------+--------------------+-------------------+





[[information]]
| Si vous essayez de stocker une valeur en dehors de l'intervalle permis par le type de votre champ, MySQL stockera la valeur la plus proche. Par exemple, si vous essayez de stocker 12457 dans un `TINYINT`, la valeur stockée sera 127 ; ce qui n'est pas exactement pareil, vous en conviendrez. Réfléchissez donc bien aux types de vos champs.


## L'attribut UNSIGNED

Vous pouvez également préciser que vos colonnes sont `UNSIGNED`, c'est-à-dire qu'on ne précise pas s'il s'agit d'une valeur positive ou négative (on aura donc toujours une valeur positive). Dans ce cas, la longueur de l'intervalle reste la même, mais les valeurs possibles sont décalées, le minimum valant 0. Pour les `TINYINT`, on pourra par exemple aller de 0 à 255.


## Limiter la taille d'affichage et l'attribut ZEROFILL

Il est possible de préciser le nombre de chiffres minimum **à l'affichage** d'une colonne de type `INT` (ou un de ses dérivés). Il suffit alors de préciser ce nombre entre parenthèses : `INT(x)`. Notez bien que cela ne change pas les capacités de stockage dans la colonne. Si vous déclarez un `INT(2)`, vous pourrez toujours y stocker 45282 par exemple. Simplement, si vous stockez un nombre avec un nombre de chiffres inférieur au nombre défini, le caractère par défaut sera ajouté à gauche du chiffre, pour qu'il prenne la bonne taille. Sans précision, le caractère par défaut est l'espace.


[[attention]]
| Soyez prudents cependant. Si vous stockez des nombres dépassant la taille d'affichage définie, il est possible que vous ayez des problèmes lors de l'utilisation de ces nombres, notamment pour des jointures (nous le verrons dans la deuxième partie).

Cette taille d'affichage est généralement utilisée en combinaison avec l'attribut `ZEROFILL`. Cet attribut ajoute des zéros à gauche du nombre lors de son affichage, il change donc le caractère par défaut par '0'. Donc, si vous déclarez une colonne comme étant


```sql
INT(4) ZEROFILL
```

Vous aurez l'affichage suivant :




+-------------+--------------+
|Nombre stocké|Nombre affiché|
+=============+==============+
|45           |0045          |
+-------------+--------------+
|4156         |4156          |
+-------------+--------------+
|785164       |785164        |
+-------------+--------------+





# Nombres décimaux

Cinq mots-clés permettent de stocker des nombres décimaux dans une colonne : `DECIMAL`, `NUMERIC`, `FLOAT`, `REAL` et `DOUBLE`.


## NUMERIC et DECIMAL

`NUMERIC` et `DECIMAL` sont équivalents et acceptent deux paramètres : la précision et l'échelle. 


- La précision définit le nombre de chiffres significatifs stockés, donc les 0 à gauche ne comptent pas. En effet 0024 est équivalent à 24. Il n'y a donc que deux chiffres significatifs dans 0024.
- L'échelle définit le nombre de chiffres après la virgule.


Dans un champ `DECIMAL(5,3)`, on peut donc stocker des nombres de 5 chiffres significatifs maximum, dont 3 chiffres sont après la virgule. Par exemple : 12.354, -54.258, 89.2 ou -56.
`DECIMAL(4)` équivaut à écrire `DECIMAL(4, 0)`.


[[information]]
| En SQL pur, on ne peut pas stocker dans un champ `DECIMAL(5,3)` un nombre supérieur à 99.999, puisque le nombre ne peut avoir que deux chiffres avant la virgule (5 chiffres en tout, dont 3 après la virgule, 5-3 = 2 avant). Cependant, MySQL permet en réalité de stocker des nombres allant jusqu'à 999.999. En effet, dans le cas de nombres positifs, MySQL utilise l'octet qui sert à stocker le signe ||-|| pour stocker un chiffre supplémentaire.

Comme pour les nombres entiers, si l'on entre un nombre qui n'est pas dans l'intervalle supporté par la colonne, MySQL le remplacera par le plus proche supporté. Donc si la colonne est définie comme un `DECIMAL(5,3)` et que le nombre est trop loin dans les positifs (1012,43 par exemple), 999.999 sera stocké, et -99.999 si le nombre est trop loin dans les négatifs.
S'il y a trop de chiffres après la virgule, MySQL arrondira à l'échelle définie.


## FLOAT, DOUBLE et REAL

Le mot-clé `FLOAT` peut s'utiliser sans paramètre, auquel cas quatre octets sont utilisés pour stocker les valeurs de la colonne. Il est cependant possible de spécifier une précision et une échelle, de la même manière que pour `DECIMAL` et `NUMERIC`.

Quant à `REAL` et `DOUBLE`, ils ne supportent pas de paramètres. `DOUBLE` est normalement plus précis que `REAL` (stockage dans 8 octets contre stockage dans 4 octets), mais ce n'est pas le cas avec MySQL qui utilise 8 octets dans les deux cas. Je vous conseille donc d'utiliser `DOUBLE` pour éviter les surprises en cas de changement de SGBDR.


## Valeurs exactes vs. valeurs approchées

Les nombres stockés en tant que `NUMERIC` ou `DECIMAL` sont stockés sous forme de chaînes de caractères. Par conséquent, c'est la valeur exacte qui est stockée. Par contre, les types `FLOAT`, `DOUBLE` et `REAL` sont stockés sous forme de nombres, et c'est une valeur approchée qui est stockée. 

Cela signifie que si vous stockez par exemple 56,6789 dans une colonne de type `FLOAT`, en réalité, MySQL stockera une valeur qui se rapproche de 56,6789 (par exemple, 56,678900000000000001). Cela peut poser problème pour des comparaison notamment (56,678900000000000001 n'étant pas égal à 56,6789). S'il est nécessaire de conserver la précision exacte de vos données (l'exemple type est celui des données bancaires), il est donc conseillé d'utiliser un type numérique à valeur exacte (`NUMERIC` ou `DECIMAL` donc).


[[attention]]
| La documentation **anglaise** de MySQL donne des exemples de problèmes rencontrés avec les valeurs approchées. N'hésitez pas à y faire un tour si vous pensez pouvoir être concernés par ce problème, ou si vous êtes simplement curieux.