La clause `WHERE` ("où" en anglais) permet de restreindre les résultats selon des critères de recherche. On peut par exemple vouloir ne sélectionner que les chiens :


```sql
SELECT * 
FROM Animal 
WHERE espece='chien';
```


[[information]]
| Comme 'chien' est une chaîne de caractères, je dois bien sûr l'entourer de guillemets.


# Les opérateurs de comparaison

Les opérateurs de comparaison sont les symboles que l'ont utilise pour définir les critères de recherche (le = dans notre exemple précédent). Huit opérateurs simples peuvent être utilisés.




+---------+--------------------------------+
|Opérateur|Signification                   |
+=========+================================+
|=        |égal                            |
+---------+--------------------------------+
|<        |inférieur                       |
+---------+--------------------------------+
|<=       |inférieur ou égal               |
+---------+--------------------------------+
|\>       |supérieur                       |
+---------+--------------------------------+
|\>=      |supérieur ou égal               |
+---------+--------------------------------+
|<> ou != |différent                       |
+---------+--------------------------------+
|<=>      |égal (valable pour `NULL` aussi)|
+---------+--------------------------------+




**Exemples** :

```sql
SELECT * 
FROM Animal 
WHERE date_naissance < '2008-01-01'; -- Animaux nés avant 2008

SELECT * 
FROM Animal 
WHERE espece <> 'chat'; -- Tous les animaux sauf les chats
```


# Combinaisons de critères

Tout ça c'est bien beau, mais comment faire si on veut les chats et les chiens par exemple ? Faut-il faire deux requêtes ?
Non bien sûr, il suffit de combiner les critères. Pour cela, il faut des opérateurs logiques, qui sont au nombre de quatre :




+---------+-------+-------------+
|Opérateur|Symbole|Signification|
+=========+=======+=============+
|`AND`    |&&     |ET           |
+---------+-------+-------------+
|`OR`     |||     |OU           |
+---------+-------+-------------+
|`XOR`    |       |OU exclusif  |
+---------+-------+-------------+
|`NOT`    |!      |NON          |
+---------+-------+-------------+




Voici quelques exemples, sûrement plus efficaces qu'un long discours.


## AND

Je veux sélectionner toutes les chattes. Je veux donc sélectionner les animaux qui sont à la fois des chats ET des femelles. J'utilise l'opérateur `AND` :


```sql
SELECT * 
FROM Animal 
WHERE espece='chat' 
    AND sexe='F';
-- OU
SELECT * 
FROM Animal 
WHERE espece='chat' 
    && sexe='F';
```


## OR

Sélection des tortues et des perroquets. Je désire donc obtenir les animaux qui sont des tortues OU des perroquets :


```sql
SELECT * 
FROM Animal 
WHERE espece='tortue' 
    OR espece='perroquet';
-- OU
SELECT * 
FROM Animal 
WHERE espece='tortue' 
    || espece='perroquet';
```


[[information]]
| Je vous conseille d'utiliser plutôt `OR` que `||`, car dans la majorité des SGBDR (et dans la norme SQL), l'opérateur `||` sert à la concaténation. C'est-à-dire à rassembler plusieurs chaînes de caractères en une seule. Il vaut donc mieux prendre l'habitude d'utiliser `OR`, au cas où vous changeriez un jour de SGBDR (ou tout simplement parce que c'est une bonne habitude).


## NOT

Sélection de tous les animaux femelles sauf les chiennes.


```sql
SELECT * 
FROM Animal 
WHERE sexe='F' 
    AND NOT espece='chien';
-- OU
SELECT * 
FROM Animal 
WHERE sexe='F' 
    AND ! espece='chien';
```


## XOR

Sélection des animaux qui sont soit des mâles, soit des perroquets (mais pas les deux) :


```sql
SELECT * 
FROM Animal 
WHERE sexe='M' 
    XOR espece='perroquet';
```

Et voilà pour les opérateurs logiques. Rien de bien compliqué, et pourtant, c'est souvent source d'erreur. Pourquoi ? Tout simplement parce que tant que vous n'utilisez qu'un seul opérateur logique, tout va très bien. Mais on a souvent besoin de combiner plus de deux critères, et c'est là que ça se corse.


# Sélection complexe

Lorsque vous utilisez plusieurs critères, et que vous devez donc combiner plusieurs opérateurs logiques, il est extrêmement important de bien structurer la requête. En particulier, il faut placer des parenthèses au bon endroit. En effet, cela n'a pas de sens de mettre plusieurs opérateurs logiques différents sur un même niveau. 

Petit exemple simple :  
Critères : rouge `AND` vert `OR` bleu

Qu'accepte-t-on ? 


- Ce qui est rouge et vert, et ce qui est bleu ?
- Ou ce qui est rouge et, soit vert soit bleu ?


Dans le premier cas, [rouge, vert] et [bleu] seraient acceptés. Dans le deuxième, c'est [rouge, vert] et [rouge, bleu] qui seront acceptés, et non [bleu].

En fait, le premier cas correspond à  (rouge `AND` vert) `OR` bleu, et le deuxième cas à  rouge `AND` (vert `OR` bleu).

Avec des parenthèses, pas moyen de se tromper sur ce qu'on désire sélectionner !


## Exercice/Exemple

Alors, imaginons une requête bien tordue…

Je voudrais les animaux qui sont, soit nés après 2009, soit des chats mâles ou femelles, mais dans le cas des femelles, elles doivent être nées avant juin 2007.

Je vous conseille d'essayer d'écrire cette requête tout seuls. Si vous n'y arrivez pas, voici une petite aide : l'astuce, c'est de penser en niveaux. Je vais donc découper ma requête.

Je cherche :


- les animaux nés après 2009 ;
- les chats mâles et femelles (uniquement nées avant juin 2007 pour les femelles).


C'est mon premier niveau. L'opérateur logique sera `OR` puisqu'il faut que les animaux répondent à un seul des deux critères pour être sélectionnés.

On continue à découper. Le premier critère ne peut plus être subdivisé, contrairement au deuxième. Je cherche :


- les animaux nés après 2009 ;
- les chats :
    - mâles ;
    - et femelles nées avant juin 2007.


Et voilà, vous avez bien défini les différents niveaux, il n'y a plus qu'à écrire la requête avec les bons opérateurs logiques !


[[secret]]
| ```sql
| SELECT * 
| FROM Animal 
| WHERE date_naissance > '2009-12-31'
|     OR
|     ( espece='chat'
|          AND
|         ( sexe='M'
|             OR
|             ( sexe='F' AND date_naissance < '2007-06-01' )
|         )
|     );
| ```


# Le cas de NULL

Vous vous souvenez sans doute de la liste des opérateurs de comparaison que je vous ai présentée (sinon, retournez au début de la partie sur la clause `WHERE`). Vous avez probablement été un peu étonnés de voir dans cette liste l'opérateur <=> : égal (valable aussi pour `NULL`). D'autant plus que j'ai fait un peu semblant de rien et ne vous ai pas donné d'explication sur cette mystérieuse précision "aussi valable pour `NULL`" :-° . Mais je vais me rattraper maintenant !
En fait, c'est très simple, le marqueur `NULL` (qui représente donc "pas de valeur") est un peu particulier. En effet, vous ne pouvez pas tester directement `colonne = NULL`. Essayons donc :

```sql
SELECT * 
FROM Animal 
WHERE nom = NULL; -- sélection des animaux sans nom

SELECT * 
FROM Animal  
WHERE commentaires <> NULL; -- sélection des animaux pour lesquels un commentaire existe
```


Comme vous pouvez vous en douter après ma petite introduction, ces deux requêtes ne renvoient pas les résultats que l'on pourrait espérer. En fait, elles ne renvoient aucun résultat. C'est donc ici qu'intervient notre opérateur de comparaison un peu spécial <=> qui permet de reconnaître `NULL`. Une autre possibilité est d'utiliser les mots-clés `IS NULL`, et si l'on veut exclure les `NULL` : `IS NOT NULL`. Nous pouvons donc réécrire nos requêtes, correctement cette fois-ci :


```sql
SELECT * 
FROM Animal 
WHERE nom <=> NULL; -- sélection des animaux sans nom
-- OU
SELECT * 
FROM Animal 
WHERE nom IS NULL;

SELECT * 
FROM Animal 
WHERE commentaires IS NOT NULL; -- sélection des animaux pour lesquels un commentaire existe
```

Cette fois-ci, ça fonctionne parfaitement !




+--+------+----+-------------------+----+------------+
|id|espece|sexe|date_naissance     |nom |commentaires|
+==+======+====+===================+====+============+
|4 |tortue|F   |2009-08-03 05:12:00|NULL|NULL        |
+--+------+----+-------------------+----+------------+
|9 |tortue|NULL|2010-08-23 05:18:00|NULL|NULL        |
+--+------+----+-------------------+----+------------+







+--+------+----+-------------------+-------+--------------------------------+
|id|espece|sexe|date_naissance     |nom    |commentaires                    |
+==+======+====+===================+=======+================================+
|1 |chien |M   |2010-04-05 13:43:00|Rox    |Mordille beaucoup               |
+--+------+----+-------------------+-------+--------------------------------+
|5 |chat  |NULL|2010-10-03 16:44:00|Choupi |Né sans oreille gauche          |
+--+------+----+-------------------+-------+--------------------------------+
|6 |tortue|F   |2009-06-13 08:17:00|Bobosse|Carapace bizarre                |
+--+------+----+-------------------+-------+--------------------------------+
|35|chat  |M   |2006-05-19 16:56:00|Raccou |Pas de queue depuis la naissance|
+--+------+----+-------------------+-------+--------------------------------+
|52|tortue|F   |2006-03-15 14:26:00|Redbul |Insomniaque                     |
+--+------+----+-------------------+-------+--------------------------------+
|55|tortue|M   |2008-03-15 18:45:00|Relou  |Surpoids                        |
+--+------+----+-------------------+-------+--------------------------------+