Les index sont représentés par le mot-clé `INDEX` (surprise !  o_O ) ou `KEY` et peuvent être créés de deux manières :


- soit directement lors de la création de la table ;
- soit en les ajoutant par la suite.



# Ajout des index lors de la création de la table

Ici aussi, deux possibilités : vous pouvez préciser dans la description de la colonne qu'il s'agit d'un index, ou lister les index par la suite.


## Index dans la description de la colonne


[[attention]]
| Seuls les index "classiques" et uniques peuvent être créés de cette manière.
 
Je rappelle que la description de la colonne se rapporte à l'endroit où vous indiquez le type de données, si la colonne peut contenir `NULL`, etc. Il est donc possible, à ce même endroit, de préciser si la colonne est un index.


```sql
CREATE TABLE nom_table (
    colonne1 INT KEY,                  -- Crée un index simple sur colonne1
    colonne2 VARCHAR(40) UNIQUE,       -- Crée un index unique sur colonne2
);
```

Quelques petites remarques ici :


- Avec cette syntaxe, **seul le mot `KEY` peut être utilisé** pour définir un index simple. Ailleurs, vous pourrez utiliser `KEY` ou `INDEX`.
- Pour définir un index `UNIQUE` de cette manière, on n'utilise que le mot-clé `UNIQUE`, sans le faire précéder de `INDEX` ou de `KEY` (comme ce sera le cas avec d'autres syntaxes).
- Il n'est **pas possible de définir des index composites** (sur plusieurs colonnes) de cette manière.
- Il n'est pas non plus possible de créer un index sur une partie de la colonne (les _x_ premiers caractères).



## Liste d'index

L'autre possibilité est d'ajouter les index à la suite des colonnes, en séparant chaque élément par une virgule :


```sql
CREATE TABLE nom_table (
    colonne1 description_colonne1,
    [colonne2 description_colonne2,
    colonne3 description_colonne3,
    ...,]
    [PRIMARY KEY (colonne_clé_primaire)],
    [INDEX [nom_index] (colonne1_index [, colonne2_index, ...]]
)
[ENGINE=moteur];
```

**Exemple** : si l'on avait voulu créer la table _Animal_ avec un index sur la date de naissance, et un autre sur les 10 premières lettres du nom, on aurait pu utiliser la commande suivante :


```sql
CREATE TABLE Animal (
    id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT,
    espece VARCHAR(40) NOT NULL,
    sexe CHAR(1),
    date_naissance DATETIME NOT NULL,
    nom VARCHAR(30),
    commentaires TEXT,
    PRIMARY KEY (id),
    INDEX ind_date_naissance (date_naissance),  -- index sur la date de naissance
    INDEX ind_nom (nom(10))                     -- index sur le nom (le chiffre entre parenthèses étant le nombre de caractères pris en compte)
)
ENGINE=INNODB;
```

Vous n'êtes pas obligés de préciser un nom pour votre index. Si vous ne le faites pas, MySQL en créera un automatiquement pour vous.
Je préfère nommer mes index moi-même plutôt que de laisser MySQL créer un nom par défaut, et respecter certaines conventions personnelles. Ainsi, mes index auront toujours le préfixe "ind" suivi du ou des nom(s) des colonnes concernées, le tout séparé par des "_". Il vous appartient de suivre vos propres conventions bien sûr. L'important étant de vous y retrouver.

Et pour ajouter des index `UNIQUE` ou `FULLTEXT`, c'est le même principe :


```sql
CREATE TABLE nom_table (
    colonne1 INT NOT NULL,   
    colonne2 VARCHAR(40), 
    colonne3 TEXT,
    UNIQUE [INDEX] ind_uni_col2 (colonne2),     -- Crée un index UNIQUE sur la colonne2, INDEX est facultatif
    FULLTEXT [INDEX] ind_full_col3 (colonne3)   -- Crée un index FULLTEXT sur la colonne3, INDEX est facultatif
)
ENGINE=MyISAM;
```

**Exemple** : création de la table _Animal_ comme précédemment, en ajoutant un index `UNIQUE` sur (_nom, espece_).


```sql
CREATE TABLE Animal (
    id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT,
    espece VARCHAR(40) NOT NULL,
    sexe CHAR(1),
    date_naissance DATETIME NOT NULL,
    nom VARCHAR(30),
    commentaires TEXT,
    PRIMARY KEY (id),
    INDEX ind_date_naissance (date_naissance),  
    INDEX ind_nom (nom(10)),                    
    UNIQUE INDEX ind_uni_nom_espece (nom, espece)  -- Index sur le nom et l'espece
)
ENGINE=INNODB;
```


[[information]]
| Avec cette syntaxe, `KEY` ne peut être utilisé que pour les index simples et `INDEX` pour tous les types d'index. Bon, je sais que c'est assez pénible à retenir, mais en gros, utilisez `INDEX` partout, sauf pour définir un index dans la description même de la colonne, et vous n'aurez pas de problème.


# Ajout des index après création de la table

En dehors du fait que parfois vous ne penserez pas à tout au moment de la création de votre table, il peut parfois être intéressant de créer les index après la table.
En effet, je vous ai dit que l'ajout d'index sur une table ralentissait l'exécution des requêtes d'écriture (insertion, suppression, modification de données). Par conséquent, si vous créez une table, que vous comptez remplir avec un grand nombre de données immédiatement, grâce à la commande `LOAD DATA INFILE` par exemple, il vaut bien mieux créer la table, la remplir, et ensuite seulement créer les index voulus sur cette table.

Il existe deux commandes permettant de créer des index sur une table existante : `ALTER TABLE`, que vous connaissez déjà un peu, et `CREATE INDEX`. Ces deux commandes sont équivalentes, utilisez celle qui vous parle le plus.


## Ajout avec ALTER TABLE


```sql
ALTER TABLE nom_table
ADD INDEX [nom_index] (colonne_index [, colonne2_index ...]); --Ajout d'un index simple

ALTER TABLE nom_table
ADD UNIQUE [nom_index] (colonne_index [, colonne2_index ...]); --Ajout d'un index UNIQUE

ALTER TABLE nom_table
ADD FULLTEXT [nom_index] (colonne_index [, colonne2_index ...]); --Ajout d'un index FULLTEXT
```


Contrairement à ce qui se passait pour l'ajout d'une colonne, il est ici obligatoire de préciser `INDEX` (ou `UNIQUE`, ou `FULLTEXT`) après `ADD`. 
Dans le cas d'un index multi-colonnes, il suffit comme d'habitude de toutes les indiquer entre parenthèses, séparées par des virgules.

Reprenons la table _Test_tuto_, utilisée pour tester `ALTER TABLE`, et ajoutons-lui un index sur la colonne _nom_ :

```sql
ALTER TABLE Test_tuto 
ADD INDEX ind_nom (nom);
```

Si vous affichez maintenant la description de votre table _Test_tuto_, vous verrez que dans la colonne "Key", il est indiqué **MUL** pour la colonne _nom_. L'index a donc bien été créé.


## Ajout avec CREATE INDEX

La syntaxe de `CREATE INDEX` est très simple :


```sql
CREATE INDEX nom_index
ON nom_table (colonne_index [, colonne2_index ...]);  -- Crée un index simple

CREATE UNIQUE INDEX nom_index
ON nom_table (colonne_index [, colonne2_index ...]);  -- Crée un index UNIQUE


CREATE FULLTEXT INDEX nom_index
ON nom_table (colonne_index [, colonne2_index ...]);  -- Crée un index FULLTEXT
```

**Exemple** : l'équivalent de la commande `ALTER TABLE` que nous avons utilisée pour ajouter un index sur la colonne _nom_ est donc :


```sql
CREATE INDEX ind_nom 
ON Test_tuto (nom);
```


# Complément pour la création d'un index UNIQUE - le cas des contraintes

Vous vous rappelez, j'espère, que les index `UNIQUE` sont ce qu'on appelle des contraintes.

Or, lorsque vous créez un index `UNIQUE`, vous pouvez explicitement créer une contrainte. C'est fait automatiquement bien sûr si vous ne le faites pas, mais ne soyez donc pas surpris de voir apparaître le mot `CONSTRAINT`, c'est à ça qu'il se réfère.

Pour pouvoir créer explicitement une contrainte lors de la création d'un index `UNIQUE`, vous devez créer cet index soit lors de la création de la table, en listant l'index (et la contrainte) à la suite des colonnes, soit après la création de la table, avec la commande `ALTER TABLE`.


```sql
CREATE TABLE nom_table (
    colonne1 INT NOT NULL,   
    colonne2 VARCHAR(40), 
    colonne3 TEXT,
    CONSTRAINT [symbole_contrainte] UNIQUE [INDEX] ind_uni_col2 (colonne2)
);

ALTER TABLE nom_table
ADD CONSTRAINT [symbole_contrainte] UNIQUE ind_uni_col2 (colonne2);
```

Il n'est pas obligatoire de donner un symbole (un nom en fait) à la contrainte. D'autant plus que dans le cas des index, vous pouvez donner un nom à l'index (ici : `ind_uni_col`).


# Suppression d'un index

Rien de bien compliqué :

```sql
ALTER TABLE nom_table 
DROP INDEX nom_index;
```

Notez qu'il n'existe pas de commande permettant de modifier un index. Le cas échéant, il vous faudra supprimer, puis recréer votre index avec vos modifications.