Les clés étrangères ont pour fonction principale la vérification de l'intégrité de votre base. Elles permettent de s'assurer que vous n'insérez pas de bêtises…

Reprenons l'exemple dans lequel on a une table _Client_ et une table _Commande_. Dans la table _Commande_, on a une colonne qui contient une référence au client. Ici, le client numéro 3, M. Nicolas Jacques, a donc passé une commande de trois tubes de colle, tandis que M^me^ Marie Malherbe (cliente numéro 2) a passé deux commandes, pour du papier et des ciseaux.

![Référence d'une table à une autre](/media/galleries/1255/160679c0-f295-470f-b030-18b89aa122b4.png)

C'est bien joli, mais que se passe-t-il si M. Hadrien Piroux passe une commande de 15 tubes de colle, et qu'à l'insertion dans la table _Commande_, votre doigt dérape et met 45 comme numéro de client ? C'est l'horreur ! Vous avez dans votre base de données une commande passée par un client inexistant, et vous passez votre après-midi du lendemain à vérifier tous vos bons de commande de la veille pour retrouver qui a commandé ces 15 tubes de colle.
Magnifique perte de temps !

Ce serait quand même sympathique si, à l'insertion d'une ligne dans la table _Commande_, un gentil petit lutin allait vérifier que le numéro de client indiqué correspond bien à quelque chose dans la table _Client_, non ?
Ce lutin, ou plutôt cette lutine, existe ! Elle s'appelle "clé étrangère".

Par conséquent, si vous créez une clé étrangère sur la colonne _client_ de la table _Commande_, en lui donnant comme référence la colonne _numero_ de la table _Client_, MySQL ne vous laissera plus jamais insérer un numéro de client inexistant dans la table _Commande_. Il s'agit bien d'une **contrainte** !

Avant d'entamer une danse de joie, parce que quand même le SQL c'est génial, restez concentrés cinq minutes, le temps de lire et retenir quelques points **importants**.


- Comme pour les index et les clés primaires, il est possible de créer des **clés étrangères composites**.
- Lorsque vous créez une clé étrangère sur une colonne (ou un groupe de colonnes) – la colonne _client_ de _Commande_ dans notre exemple –, **un index est automatiquement ajouté** sur celle-ci (ou sur le groupe).
- Par contre, la colonne (le groupe de colonnes) qui sert de référence - la colonne _numero_ de _Client_ - **doit** déjà posséder un index (où être clé primaire bien sûr).
- La colonne (ou le groupe de colonnes) sur laquelle (lequel) la clé est créée doit être **exactement** du même type que la colonne (le groupe de colonnes) qu'elle (il) référence. Cela implique qu'en cas de clé composite, il faut le même nombre de colonnes dans la clé et la référence. Donc, si _numero_ (dans _Client_) est un `INT UNSIGNED`, _client_ (dans _Commande_) doit être de type `INT UNSIGNED` aussi.
- Tous les moteurs de table ne permettent pas l'utilisation des clés étrangères. Par exemple, MyISAM ne le permet pas, contrairement à InnoDB.



# Création

Une clé étrangère est un peu plus complexe à créer qu'un index ou une clé primaire, puisqu'il faut deux éléments :


- la ou les colonnes sur laquelle (lesquelles) on crée la clé - on utilise `FOREIGN KEY` ;
- la ou les colonnes qui va (vont) servir de référence - on utilise `REFERENCES`.



## Lors de la création de la table

Du fait de la présence de deux paramètres, une clé étrangère ne peut que s'ajouter à la suite des colonnes, et pas directement dans la description d'une colonne. Par ailleurs, je vous conseille ici de créer explicitement une contrainte (grâce au mot-clé `CONSTRAINT`) et de lui donner un symbole. En effet, pour les index, on pouvait utiliser leur nom pour les identifier ; pour les clés primaires, le nom de la table suffisait puisqu'il n'y en a qu'une par table. Par contre, pour différencier facilement les clés étrangères d'une table, il est utile de leur donner un nom, à travers la contrainte associée. 

À nouveau, je respecte certaines conventions de nommage : mes clés étrangères ont des noms commençant par fk (pour `FOREIGN KEY`), suivi du nom de la colonne dans la table puis (si elle s'appelle différemment) du nom de la colonne de référence, le tout séparé par des ||_|| (_fk_client_numero_ par exemple).


```sql
CREATE TABLE [IF NOT EXISTS] Nom_table (
    colonne1 description_colonne1,
    [colonne2 description_colonne2,
    colonne3 description_colonne3,
    ...,]
    [ [CONSTRAINT [symbole_contrainte]]  FOREIGN KEY (colonne(s)_clé_étrangère) REFERENCES table_référence (colonne(s)_référence)]
)
[ENGINE=moteur];
```

Donc si on imagine les tables _Client_ et _Commande_, pour créer la table _Commande_ avec une clé étrangère ayant pour référence la colonne _numero_ de la table _Client_, on utilisera :


```sql
CREATE TABLE Commande (
    numero INT UNSIGNED PRIMARY KEY AUTO_INCREMENT,
    client INT UNSIGNED NOT NULL,
    produit VARCHAR(40),
    quantite SMALLINT DEFAULT 1,
    CONSTRAINT fk_client_numero          -- On donne un nom à notre clé
        FOREIGN KEY (client)             -- Colonne sur laquelle on crée la clé
        REFERENCES Client(numero)        -- Colonne de référence
)
ENGINE=InnoDB;                          -- MyISAM interdit, je le rappelle encore une fois !
```


## Après création de la table

Tout comme pour les clés primaires, pour créer une clé étrangère après création de la table, il faut utiliser `ALTER TABLE`.


```sql
ALTER TABLE Commande
ADD CONSTRAINT fk_client_numero FOREIGN KEY client REFERENCES Client(numero);
```


# Suppression d'une clé étrangère

Il peut y avoir plusieurs clés étrangères par table. Par conséquent, lors d'une suppression il faut identifier la clé à détruire. Cela se fait grâce au symbole de la contrainte.


```sql
ALTER TABLE nom_table
DROP FOREIGN KEY symbole_contrainte
```