Maintenant que vous connaissez les clés étrangères, nous allons en profiter pour modifier notre base et ajouter quelques tables afin de préparer le prochain chapitre, qui portera sur les jointures.

Jusqu'à maintenant, la seule information sur l'espèce des animaux de notre élevage était son nom courant. Nous voudrions maintenant stocker aussi son nom latin, ainsi qu'une petite description. Que faire ? Ajouter deux colonnes à notre table ? _nom_latin_espece_ et _description_espece_ ?

J'espère que vous n'avez envisagé cette possibilité qu'une demi-seconde, car il est assez évident que c'est une très mauvaise solution. En effet, ça obligerait à stocker la même description pour chaque chien, chaque chat, etc. Ainsi que le même nom latin. Nous le faisions déjà avec le nom courant, et ça aurait déjà pu poser problème. Imaginez que pour un animal vous fassiez une faute d'orthographe au nom de l'espèce, "chein" ou lieu de "chien" par exemple. L'animal en question n'apparaîtrait jamais lorsque vous feriez une recherche par espèce.

Il faudrait donc un système qui nous permette de ne pas répéter la même information plusieurs fois, et qui limite les erreurs que l'on pourrait faire.

La bonne solution est de créer une seconde table : la table _Espece_. Cette table aura 4 colonnes : le nom courant, le nom latin, une description, et un numéro d'identification (qui sera la clé primaire de cette table).


# La table _Espece_

Voici donc la commande que je vous propose d'exécuter pour créer la table _Espece_ :


```sql
CREATE TABLE Espece (
    id SMALLINT UNSIGNED AUTO_INCREMENT,
    nom_courant VARCHAR(40) NOT NULL,
    nom_latin VARCHAR(40) NOT NULL UNIQUE,
    description TEXT,
    PRIMARY KEY(id)
)
ENGINE=InnoDB;
```

On met un index `UNIQUE` sur la colonne _nom_latin_ pour être sûr que l'on ne rentrera pas deux fois la même espèce. Pourquoi sur le nom latin et pas sur le nom courant ? Tout simplement parce que le nom latin est beaucoup plus rigoureux et réglementé que le nom courant. On peut avoir plusieurs dénominations courantes pour une même espèce ; ce n'est pas le cas avec le nom latin.

Bien, remplissons donc cette table avec les espèces déjà présentes dans la base :


```sql
INSERT INTO Espece (nom_courant, nom_latin, description) VALUES
    ('Chien', 'Canis canis', 'Bestiole à quatre pattes qui aime les caresses et tire souvent la langue'),
    ('Chat', 'Felis silvestris', 'Bestiole à quatre pattes qui saute très haut et grimpe aux arbres'),
    ('Tortue d''Hermann', 'Testudo hermanni', 'Bestiole avec une carapace très dure'),
    ('Perroquet amazone', 'Alipiopsitta xanthops', 'Joli oiseau parleur vert et jaune');
```

Ce qui nous donne la table suivante :




+--+-----------------+---------------------+------------------------------------------------------------------------+
|id|nom_courant      |nom_latin            |description                                                             |
+==+=================+=====================+========================================================================+
|1 |Chien            |Canis canis          |Bestiole à quatre pattes qui aime les caresses et tire souvent la langue|
+--+-----------------+---------------------+------------------------------------------------------------------------+
|2 |Chat             |Felis silvestris     |Bestiole à quatre pattes qui saute très haut et grimpe aux arbres       |
+--+-----------------+---------------------+------------------------------------------------------------------------+
|3 |Tortue d'Hermann |Testudo hermanni     |Bestiole avec une carapace très dure                                    |
+--+-----------------+---------------------+------------------------------------------------------------------------+
|4 |Perroquet amazone|Alipiopsitta xanthops|Joli oiseau parleur vert et jaune                                       |
+--+-----------------+---------------------+------------------------------------------------------------------------+





Vous aurez remarqué que j'ai légèrement modifié les noms des espèces "perroquet" et "tortue". En effet, et j'espère que les biologistes pourront me pardonner, "perroquet" et "tortue" ne sont pas des espèces, mais des ordres. J'ai donc précisé un peu (si je donne juste l'ordre, c'est comme si je mettais "carnivore" au lieu de "chat" - ou "chien" d'ailleurs).

Bien, mais cela ne suffit pas ! Il faut également modifier notre table _Animal_. Nous allons ajouter une colonne _espece_id_, qui contiendra l'_id_ de l'espèce à laquelle appartient l'animal, et remplir cette colonne. Ensuite nous pourrons supprimer la colonne _espece_, qui n'aura plus de raison d'être.

La colonne _espece_id_ sera une clé étrangère ayant pour référence la colonne _id_ de la table _Espece_. Je rappelle donc que ça signifie qu'il ne sera pas possible d'insérer dans _espece_id_ un nombre qui n'existe pas dans la colonne _id_ de la table _Espece_.


# La table _Animal_

Je vous conseille d'essayer d'écrire vous-même les requêtes avant de regarder comment je fais.



[[secret]]
| ```sql
| -- Ajout d'une colonne espece_id
| ALTER TABLE Animal ADD COLUMN espece_id SMALLINT UNSIGNED; -- même type que la colonne id de Espece
| 
| -- Remplissage de espece_id
| UPDATE Animal SET espece_id = 1 WHERE espece = 'chien';
| UPDATE Animal SET espece_id = 2 WHERE espece = 'chat';
| UPDATE Animal SET espece_id = 3 WHERE espece = 'tortue';
| UPDATE Animal SET espece_id = 4 WHERE espece = 'perroquet';
| 
| -- Suppression de la colonne espece
| ALTER TABLE Animal DROP COLUMN espece;
| 
| -- Ajout de la clé étrangère
| ALTER TABLE Animal
| ADD CONSTRAINT fk_espece_id FOREIGN KEY (espece_id) REFERENCES Espece(id);
| ```

Pour tester l'efficacité de la clé étrangère, essayons d'ajouter un animal dont l'_espece_id_ est 5 (qui n'existe donc pas dans la table _Espece_) :


```sql
INSERT INTO Animal (nom, espece_id, date_naissance)
VALUES ('Caouette', 5, '2009-02-15 12:45:00');
```


```console
ERROR 1452 (23000): Cannot add or update a child row: a foreign key constraint fails (`elevage`.`animal`, CONSTRAINT `fk_espece_id` FOREIGN KEY (`espece_id`) REFERENCES `espece` (`id`))
```

Elle est pas belle la vie ?

J'en profite également pour ajouter une contrainte `NOT NULL` sur la colonne _espece_id_. Après tout, si _espece_ ne pouvait pas être `NULL`, pas de raison qu'_espece_id_ puisse l'être ! Ajoutons également l'index `UNIQUE` sur (_nom, espece_id_) dont on a déjà discuté.


[[secret]]
| ```sql
| ALTER TABLE Animal MODIFY espece_id SMALLINT UNSIGNED NOT NULL;
| 
| CREATE UNIQUE INDEX ind_uni_nom_espece_id ON Animal (nom, espece_id);
| ```

Notez qu'il n'était pas possible de mettre la contrainte `NOT NULL` à la création de la colonne, puisque tant que l'on n'avait pas rempli _espece_id_, elle contenait `NULL` pour toutes les lignes.

Voilà, nos deux tables sont maintenant prêtes. Mais avant de vous lâcher dans l'univers merveilleux des jointures et des sous-requêtes, nous allons encore compliquer un peu les choses. Parce que c'est bien joli pour un éleveur de pouvoir reconnaître un chien d'un chat, mais il serait de bon ton de reconnaître également un berger allemand d'un teckel, non ?

Par conséquent, nous allons encore ajouter deux choses à notre base. D'une part, nous allons ajouter une table _Race_, basée sur le même schéma que la table _Espece_. Il faudra également ajouter une colonne à la table _Animal_, qui contiendra l'_id_ de la race de l'animal. Contrairement à la colonne _espece_id_, celle-ci pourra être `NULL`. Il n'est pas impossible que nous accueillions des bâtards, ou que certaines espèces que nous élevons ne soient pas classées en plusieurs races différentes.

Ensuite, nous allons garder une trace du pedigree de nos animaux. Pour ce faire, il faut pouvoir connaître ses parents. Donc, nous ajouterons deux colonnes à la table _Animal_ : _pere_id_ et _mere_id_, qui contiendront respectivement l'_id_ du père et l'_id_ de la mère de l'animal.

Ce sont toutes des commandes que vous connaissez, donc je ne détaille pas plus.


```sql
-- --------------------------
-- CREATION DE  LA TABLE Race
-- --------------------------
CREATE TABLE Race (
    id SMALLINT UNSIGNED AUTO_INCREMENT,
    nom VARCHAR(40) NOT NULL,
    espece_id SMALLINT UNSIGNED NOT NULL,     -- pas de nom latin, mais une référence vers l'espèce
    description TEXT,
    PRIMARY KEY(id),
    CONSTRAINT fk_race_espece_id FOREIGN KEY (espece_id) REFERENCES Espece(id)  -- pour assurer l'intégrité de la référence
)
ENGINE = InnoDB;

-- -----------------------
-- REMPLISSAGE DE LA TABLE
-- -----------------------
INSERT INTO Race (nom, espece_id, description)
VALUES ('Berger allemand', 1, 'Chien sportif et élégant au pelage dense, noir-marron-fauve, noir ou gris.'),
('Berger blanc suisse', 1, 'Petit chien au corps compact, avec des pattes courtes mais bien proportionnées et au pelage tricolore ou bicolore.'),
('Boxer', 1, 'Chien de taille moyenne, au poil ras de couleur fauve ou bringé avec quelques marques blanches.'),
('Bleu russe', 2, 'Chat aux yeux verts et à la robe épaisse et argentée.'),
('Maine coon', 2, 'Chat de grande taille, à poils mi-longs.'),
('Singapura', 2, 'Chat de petite taille aux grands yeux en amandes.'),
('Sphynx', 2, 'Chat sans poils.');

-- ---------------------------------------------
-- AJOUT DE LA COLONNE race_id A LA TABLE Animal
-- ---------------------------------------------
ALTER TABLE Animal ADD COLUMN race_id SMALLINT UNSIGNED;

ALTER TABLE Animal
ADD CONSTRAINT fk_race_id FOREIGN KEY (race_id) REFERENCES Race(id);

-- -------------------------
-- REMPLISSAGE DE LA COLONNE
-- -------------------------
UPDATE Animal SET race_id = 1 WHERE id IN (1, 13, 20, 18, 22, 25, 26, 28);
UPDATE Animal SET race_id = 2 WHERE id IN (12, 14, 19, 7);
UPDATE Animal SET race_id = 3 WHERE id IN (23, 17, 21, 27);
UPDATE Animal SET race_id = 4 WHERE id IN (33, 35, 37, 41, 44, 31, 3);
UPDATE Animal SET race_id = 5 WHERE id IN (43, 40, 30, 32, 42, 34, 39, 8);
UPDATE Animal SET race_id = 6 WHERE id IN (29, 36, 38);

-- -------------------------------------------------------
-- AJOUT DES COLONNES mere_id ET pere_id A LA TABLE Animal
-- -------------------------------------------------------
ALTER TABLE Animal ADD COLUMN mere_id SMALLINT UNSIGNED;

ALTER TABLE Animal
ADD CONSTRAINT fk_mere_id FOREIGN KEY (mere_id) REFERENCES Animal(id);

ALTER TABLE Animal ADD COLUMN pere_id SMALLINT UNSIGNED;

ALTER TABLE Animal
ADD CONSTRAINT fk_pere_id FOREIGN KEY (pere_id) REFERENCES Animal(id);

-- -------------------------------------------
-- REMPLISSAGE DES COLONNES mere_id ET pere_id
-- -------------------------------------------
UPDATE Animal SET mere_id = 18, pere_id = 22 WHERE id = 1;
UPDATE Animal SET mere_id = 7, pere_id = 21 WHERE id = 10;
UPDATE Animal SET mere_id = 41, pere_id = 31 WHERE id = 3;
UPDATE Animal SET mere_id = 40, pere_id = 30 WHERE id = 2;
```

A priori, la seule chose qui pourrait vous avoir surpris dans ces requêtes ce sont les clés étrangères sur _mere_id_ et _pere_id_ qui référencent toutes deux une autre colonne de la même table. 

Bien, maintenant que nous avons trois tables et des données sympathiques à exploiter, nous allons passer aux choses sérieuses avec les jointures d'abord, puis les sous-requêtes.