Une sous-requête corrélée est une sous-requête qui fait référence à une colonne (ou une table) qui n'est pas définie dans sa clause `FROM`, mais bien ailleurs dans la requête dont elle fait partie.

Vu que ce n'est pas une définition extrêmement claire de prime abord, voici un exemple de requête avec une sous-requête corrélée :


```sql
SELECT colonne1 
FROM tableA
WHERE colonne2 IN (
    SELECT colonne3
    FROM tableB
    WHERE tableB.colonne4 = tableA.colonne5
    );
```


Si l'on prend la sous-requête toute seule, on ne pourra pas l'exécuter :


```sql
SELECT colonne3
FROM tableB
WHERE tableB.colonne4 = tableA.colonne5
```


En effet, seule la _tableB_ est sélectionnée dans la clause `FROM`, il n'y a pas de jointure avec la _tableA_, et pourtant on utilise la _tableA_ dans la condition.

Par contre, aucun problème pour l'utiliser comme sous-requête, puisque la clause `FROM` de la requête principale sélectionne la _tableA_. La sous-requête est donc **corrélée** à la requête principale.

**Attention** : si MySQL rencontre une table inconnue dans une sous-requête, elle va aller chercher dans les **niveaux supérieurs uniquement** si cette table existe. Donc, imaginons que l'on a une requête avec 3 niveaux : la requête principale (niveau 1), une ou plusieurs sous-requêtes (niveau 2) et une ou plusieurs sous-sous-requêtes, c'est-à-dire une sous-requête dans une sous-requête (niveau 3).


- Une sous-requête (niveau 2) peut être corrélée à la requête principale (niveau 1).
- Une sous-sous-requête (niveau 3) peut être corrélée à la sous-requête **dont elle dépend** (niveau 2), ou à la requête principale (niveau 1).
- Mais une sous-requête (niveau 2) ne peut pas être corrélée à une autre sous-requête (niveau 2).


Si l'on prend le schéma suivant, on peut donc remonter l'arbre, mais jamais descendre d'un cran pour trouver les tables nécessaires.

![Sous-requêtes corrélées](/media/galleries/1255/9709e91f-a658-411e-a0b4-8a11643c2540.png)


Peuvent être corrélées à :



- **A** : B, C, D, E, F, G et H
- **B** : aucune
- **C** : D, E, F, G et H
- **D** : F, G et H
- **E** : aucune






Le temps de vous expliquer le fonctionnement de `EXISTS` et `NOT EXISTS`, et nous verrons un exemple de sous-requête corrélée.


## Conditions avec EXISTS et NOT EXISTS

Les conditions `EXISTS` et `NOT EXISTS` s'utilisent de la manière suivante :


```sql
SELECT * FROM nom_table
WHERE [NOT] EXISTS (sous-requête)
```

Une condition avec `EXISTS` sera vraie (et donc la requête renverra quelque chose) si la sous-requête correspondante renvoie au moins une ligne.
Une condition avec `NOT EXISTS` sera vraie si la sous-requête correspondante ne renvoie aucune ligne.

**Exemple** : on sélectionne les races s'il existe un animal qui s'appelle Balou.


```sql
SELECT id, nom, espece_id FROM Race 
WHERE EXISTS (SELECT * FROM Animal WHERE nom = 'Balou');
```

Vu qu'il existe bien un animal du nom de Balou dans notre table _Animal_, la condition est vraie, on sélectionne donc toutes les races. Si l'on avait utilisé un nom qui n'existe pas, la requête n'aurait renvoyé aucun résultat.




+--+-------------------+---------+
|id|nom                |espece_id|
+==+===================+=========+
|1 |Berger allemand    |1        |
+--+-------------------+---------+
|2 |Berger blanc suisse|1        |
+--+-------------------+---------+
|3 |Boxer              |1        |
+--+-------------------+---------+
|4 |Bleu russe         |2        |
+--+-------------------+---------+
|5 |Maine coon         |2        |
+--+-------------------+---------+
|6 |Singapura          |2        |
+--+-------------------+---------+
|7 |Sphynx             |2        |
+--+-------------------+---------+
|8 |Nebelung           |2        |
+--+-------------------+---------+





Vous conviendrez cependant qu'une telle requête n'a pas beaucoup de sens, c'était juste pour vous faire comprendre le principe. En général, on utilise `WHERE [NOT] EXISTS` avec des sous-requêtes corrélées.

**Exemple** : je veux sélectionner toutes les races dont on ne possède aucun animal.


```sql
SELECT * FROM Race
WHERE NOT EXISTS (SELECT * FROM Animal WHERE Animal.race_id = Race.id);
```


La sous-requête est bien corrélée à la requête principale, puisqu'elle utilise la table _Race_, qui n'est pas sélectionnée dans la sous-requête.

En résultat, on a bien le Sphynx, puisqu'on n'en possède aucun.




+--+------+---------+----------------+
|id|nom   |espece_id|description     |
+==+======+=========+================+
|7 |Sphynx|2        |Chat sans poils.|
+--+------+---------+----------------+