# Utilisation des sous-requêtes


## Pour la sélection

Imaginez que pour une raison bizarre, vous vouliez que tous les perroquets aient en commentaire "Coco veut un gâteau !".

Si vous saviez que l'_id_ de l'espèce est 4, ce serait facile :


```sql
UPDATE Animal SET commentaires = 'Coco veut un gâteau' WHERE espece_id = 4;
```

Seulement voilà, vous ne savez évidemment pas que l'_id_ de cette espèce est 4. Sinon, ce n'est pas drôle ! Vous allez donc utiliser une magnifique sous-requête pour modifier ces bruyants volatiles !


```sql
UPDATE Animal SET commentaires = 'Coco veut un gâteau !' WHERE espece_id = 
    (SELECT id FROM Espece WHERE nom_courant LIKE 'Perroquet%');
```

Bien sûr, toutes les possibilités de conditions que l'on a vues pour la sélection sont encore valables pour les modifications. Après tout, une clause `WHERE` est une clause `WHERE` !


## Pour l'élément à modifier

Ce matin, un client demande à voir vos chats Bleu Russe, car il compte en offrir un à sa fille. Ni une ni deux, vous vérifiez dans la base de données, puis allez chercher Schtroumpfette, Filou, Caribou, Raccou, Callune, Feta et Cawette.
Et là, horreur et damnation ! À l'instant où ses yeux se posent sur Cawette, le client devient vert de rage. Il prend à peine le temps de vous expliquer, outré, que Cawette n'est pas un Bleu Russe mais bien un Nebelung, à cause de ses poils longs, puis s'en va chercher un éleveur plus compétent.

Bon… L'erreur est humaine, mais autant la réparer rapidement. Vous insérez donc une nouvelle race dans la table ad hoc.


```sql
INSERT INTO Race (nom, espece_id, description)
VALUES ('Nebelung', 2, 'Chat bleu russe, mais avec des poils longs...');
```

Une fois cela fait, il vous reste encore à modifier la race de Cawette. Pour cela, vous avez besoin de l'_id_ de la race Nebelung que vous venez d'ajouter.
Vous vous en doutez, il est tout à fait possible de le faire grâce à une sous-requête :


```sql
UPDATE Animal SET race_id = 
    (SELECT id FROM Race WHERE nom = 'Nebelung' AND espece_id = 2)
WHERE nom = 'Cawette';
```


Il est bien entendu indispensable que le résultat de la sous-requête soit une **valeur** !


## Limitation des sous-requêtes dans un UPDATE


Une limitation importante des sous-requêtes est qu'**on ne peut pas modifier un élément d'une table que l'on utilise dans une sous-requête**. 

**Exemple** : vous trouvez que Callune ressemble quand même fichtrement à Cawette, et ses poils sont aussi longs. Du coup, vous vous dites que vous auriez dû également modifier la race de Callune. Vous essayez donc la requête suivante :


```sql
UPDATE Animal SET race_id = 
    (SELECT race_id FROM Animal WHERE nom = 'Cawette' AND espece_id = 2)
WHERE nom = 'Callune';
```

Malheureusement :


```console
ERROR 1093 (HY000): You can't specify target table 'Animal' for update in FROM clause
```

La sous-requête utilise la table _Animal_, or vous cherchez à modifier le contenu de celle-ci. C'est impossible !

Il vous faudra donc utiliser la même requête que pour Cawette, en changeant simplement le nom (je ne vous fais pas l'affront de vous l'écrire).


# Modification avec jointure

Imaginons que vous vouliez que, pour les tortues et les perroquets, si un animal n'a pas de commentaire, on lui ajoute comme commentaire la description de l'espèce. Vous pourriez sélectionner les descriptions, les copier, retenir l'_id_ de l'espèce, et ensuite faire un `UPDATE` pour les tortues et un autre pour les perroquets.
Ou alors, vous pourriez simplement faire un `UPDATE` avec jointure !

Voici la syntaxe que vous devriez utiliser pour le faire avec une jointure :


```sql
UPDATE Animal                                                             -- Classique !
INNER JOIN Espece                                                         -- Jointure.
    ON Animal.espece_id = Espece.id                                        -- Condition de la jointure.
SET Animal.commentaires = Espece.description                               -- Ensuite, la modification voulue.
WHERE Animal.commentaires IS NULL                                         -- Seulement s'il n'y a pas encore de commentaire.
AND Espece.nom_courant IN ('Perroquet amazone', 'Tortue d''Hermann');      -- Et seulement pour les perroquets et les tortues.
```





- Vous pouvez bien sûr mettre ce que vous voulez comme modifications. Ici, j'ai utilisé la valeur de la colonne dans l'autre table, mais vous auriez pu mettre `Animal.commentaires = 'Tralala'`, et la jointure n'aurait alors servi qu'à sélectionner les tortues et les perroquets grâce au nom courant de l'espèce.
- Toutes les jointures sont possibles. Vous n'êtes pas limités aux jointures internes, ni à deux tables jointes.