La syntaxe d'`UNION` est simplissime : vous avez deux requêtes `SELECT` dont vous voulez additionner les résultats ; il vous suffit d'ajouter `UNION` entre ces deux requêtes.


```sql
SELECT ...
UNION
SELECT ...
```

Le nombre de requêtes qu'il est possible d'unir est illimité. Si vous avez cinquante requêtes de sélection, placez un `UNION` entre les cinquante requêtes.


```sql
SELECT ...
UNION
SELECT ...
UNION
SELECT ...
....
UNION
SELECT ...
```

Par exemple, vous pouvez obtenir les chiens et les tortues de la manière suivante :


```sql
SELECT Animal.* FROM Animal 
INNER JOIN Espece ON Animal.espece_id = Espece.id 
WHERE Espece.nom_courant = 'Chat'
UNION
SELECT Animal.* FROM Animal 
INNER JOIN Espece ON Animal.espece_id = Espece.id 
WHERE Espece.nom_courant = 'Tortue d''Hermann';
```


[[information]]
| Cette requête peut bien sûr être écrite sans `UNION`, en faisant tout simplement un seul `SELECT` avec `WHERE Espece.nom_courant = 'Chat' OR Espece.nom_courant = 'Tortue d''Hermann'`. Il ne s'agit ici que d'un exemple destiné à illustrer la syntaxe. Nous verrons plus loin des exemples de cas où `UNION` est indispensable.


# Les règles

Bien sûr, ce n'est pas si simple que ça, il faut respecter certaines contraintes.


## Nombre de colonnes

Il est absolument indispensable que toutes les requêtes unies renvoient le même nombre de colonnes.

Dans la requête que l'on a écrite ci-dessus, aucun problème puisque l'on sélectionne _Animal.*_ dans les deux requêtes. Mais il ne serait donc pas possible de sélectionner un nombre de colonnes différent dans chaque requête intermédiaire.

Par exemple, la requête ci-dessous renverra une erreur :


```sql
-- Pas le même nombre de colonnes --
------------------------------------

SELECT Animal.id, Animal.nom, Espece.nom_courant                    -- 3 colonnes sélectionnées
FROM Animal
INNER JOIN Espece ON Animal.espece_id = Espece.id
WHERE Espece.nom_courant = 'Chat'
UNION
SELECT Animal.id, Animal.nom, Espece.nom_courant, Animal.espece_id -- 4 colonnes sélectionnées
FROM Animal
INNER JOIN Espece ON Animal.espece_id = Espece.id
WHERE Espece.nom_courant = 'Tortue d''Hermann';
```


```console
ERROR 1222 (21000): The used SELECT statements have a different number of columns
```



## Type et ordre des colonnes
 
En ce qui concerne le type des colonnes, je pense vous avoir déjà signalé que MySQL est très (vraiment très) permissif. Par conséquent, si vous sélectionnez des colonnes de différents types, vous n'aurez pas d'erreurs, mais vous aurez des résultats un peu… spéciaux.  o_O 

Prenons la requête suivante :


```sql
SELECT Animal.id, Animal.nom, Espece.nom_courant    -- 3e colonne : nom_courant VARCHAR
FROM Animal
INNER JOIN Espece ON Animal.espece_id = Espece.id
WHERE Espece.nom_courant = 'Chat' AND Animal.nom LIKE 'C%'
UNION
SELECT Animal.id, Animal.nom, Espece.id             -- 3e colonne : id SMALLINT
FROM Animal
INNER JOIN Espece ON Animal.espece_id = Espece.id
WHERE Espece.nom_courant = 'Tortue d''Hermann' AND Animal.nom LIKE 'C%';
```

Vous aurez bien un résultat :




+--+--------+-----------+
|id|nom     |nom_courant|
+==+========+===========+
|5 |Choupi  |Chat       |
+--+--------+-----------+
|33|Caribou |Chat       |
+--+--------+-----------+
|34|Capou   |Chat       |
+--+--------+-----------+
|37|Callune |Chat       |
+--+--------+-----------+
|43|Cracotte|Chat       |
+--+--------+-----------+
|44|Cawette |Chat       |
+--+--------+-----------+
|50|Cheli   |3          |
+--+--------+-----------+
|51|Chicaca |3          |
+--+--------+-----------+





Mais avouez que ce n'est pas un résultat très cohérent. MySQL va simplement convertir tout en chaînes de caractères pour ne pas avoir de problème. Soyez donc très prudents !


[[information]]
| Vous pouvez constater en passant que les noms de colonnes utilisés dans les résultats sont ceux de la première requête effectuée. Vous pouvez bien sûr les renommer avec des alias.

Et enfin, pour l'ordre des colonnes, à nouveau vous n'aurez pas d'erreur tant que vous avez le même nombre de colonnes dans chaque requête, mais vous pouvez avoir des résultats bizarres. En effet, MySQL n'analyse pas les noms des colonnes pour trouver une quelconque correspondance d'une requête à l'autre ; tout se fait sur la base de la position de la colonne dans la requête.


```sql
SELECT Animal.id, Animal.nom, Espece.nom_courant FROM Animal
INNER JOIN Espece ON Animal.espece_id = Espece.id
WHERE Espece.nom_courant = 'Chat' AND Animal.nom LIKE 'C%'
UNION
SELECT Animal.nom, Animal.id, Espece.nom_courant FROM Animal   -- 1e et 2e colonnes inversées par rapport à la première requête
INNER JOIN Espece ON Animal.espece_id = Espece.id
WHERE Espece.nom_courant = 'Tortue d''Hermann' AND Animal.nom LIKE 'C%';
```




+-------+--------+----------------+
|id     |nom     |nom_courant     |
+=======+========+================+
|5      |Choupi  |Chat            |
+-------+--------+----------------+
|33     |Caribou |Chat            |
+-------+--------+----------------+
|34     |Capou   |Chat            |
+-------+--------+----------------+
|37     |Callune |Chat            |
+-------+--------+----------------+
|43     |Cracotte|Chat            |
+-------+--------+----------------+
|44     |Cawette |Chat            |
+-------+--------+----------------+
|Cheli  |50      |Tortue d'Hermann|
+-------+--------+----------------+
|Chicaca|51      |Tortue d'Hermann|
+-------+--------+----------------+