# LIMIT

Il est possible de restreindre les résultats avec `LIMIT` au niveau de la requête entière, ou au niveau des différentes requêtes unies. L'important est que ce soit clair. Par exemple, vous unissez deux requêtes et vous voulez limiter les résultats de la première. La requête suivante est parfaite pour ça :


```sql
SELECT id, nom, 'Race' AS table_origine FROM Race LIMIT 3
UNION
SELECT id, nom_latin, 'Espèce' AS table_origine FROM Espece;
```

Vous avez bien trois noms de races, suivis de toutes les espèces.




+--+---------------------+-------------+
|id|nom                  |table_origine|
+==+=====================+=============+
|1 |Berger allemand      |Race         |
+--+---------------------+-------------+
|2 |Berger blanc suisse  |Race         |
+--+---------------------+-------------+
|3 |Boxer                |Race         |
+--+---------------------+-------------+
|4 |Alipiopsitta xanthops|Espèce       |
+--+---------------------+-------------+
|1 |Canis canis          |Espèce       |
+--+---------------------+-------------+
|2 |Felis silvestris     |Espèce       |
+--+---------------------+-------------+
|3 |Testudo hermanni     |Espèce       |
+--+---------------------+-------------+






[[information]]
| En passant, voici un résultat qu'il n'est pas possible d'obtenir sans utiliser `UNION` !

Par contre, si vous voulez limiter les résultats de la seconde requête, comment faire ? Essayons la requête suivante.


```sql
SELECT id, nom, 'Race' AS table_origine FROM Race
UNION
SELECT id, nom_latin, 'Espèce' AS table_origine FROM Espece LIMIT 2;
```




+--+-------------------+-------------+
|id|nom                |table_origine|
+==+===================+=============+
|1 |Berger allemand    |Race         |
+--+-------------------+-------------+
|2 |Berger blanc suisse|Race         |
+--+-------------------+-------------+





Visiblement, ce n'est pas ce que nous voulions… En fait, `LIMIT` a été appliqué à l'ensemble des résultats, après `UNION`. Par conséquent, si l'on veut que `LIMIT` ne porte que sur la dernière requête, il faut le préciser. Pour ça, il suffit d'utiliser des parenthèses.


```sql
SELECT id, nom, 'Race' AS table_origine FROM Race
UNION
(SELECT id, nom_latin, 'Espèce' AS table_origine FROM Espece LIMIT 2);
```




+--+---------------------+-------------+
|id|nom                  |table_origine|
+==+=====================+=============+
|1 |Berger allemand      |Race         |
+--+---------------------+-------------+
|2 |Berger blanc suisse  |Race         |
+--+---------------------+-------------+
|3 |Boxer                |Race         |
+--+---------------------+-------------+
|4 |Bleu russe           |Race         |
+--+---------------------+-------------+
|5 |Maine coon           |Race         |
+--+---------------------+-------------+
|6 |Singapura            |Race         |
+--+---------------------+-------------+
|7 |Sphynx               |Race         |
+--+---------------------+-------------+
|8 |Nebelung             |Race         |
+--+---------------------+-------------+
|4 |Alipiopsitta xanthops|Espèce       |
+--+---------------------+-------------+
|1 |Canis canis          |Espèce       |
+--+---------------------+-------------+





Et voilà le travail !


# ORDER BY

Par contre, s'il est possible de trier le résultat final d'une requête avec `UNION`, on ne peut pas trier les résultats des requêtes intermédiaires. Par exemple, cette requête trie bien les résultats par ordre anti-alphabétique du nom :


```sql
SELECT id, nom, 'Race' AS table_origine FROM Race
UNION
SELECT id, nom_latin, 'Espèce' AS table_origine FROM Espece
ORDER BY nom DESC;
```


[[attention]]
| Il faut bien mettre ici `ORDER BY nom`, et surtout pas `ORDER BY Race.nom` ou `ORDER BY Espece.nom_latin`. En effet, l'`ORDER BY` agit sur l'ensemble de la requête, donc en quelque sorte, sur une table intermédiaire composée des résultats des deux requêtes unies. Cette table n'est pas nommée, et ne possède que deux colonnes : _id_ et _nom_ (définies par la première clause `SELECT` rencontrée).




+--+---------------------+-------------+
|id|nom                  |table_origine|
+==+=====================+=============+
|3 |Testudo hermanni     |Espèce       |
+--+---------------------+-------------+
|7 |Sphynx               |Race         |
+--+---------------------+-------------+
|6 |Singapura            |Race         |
+--+---------------------+-------------+
|8 |Nebelung             |Race         |
+--+---------------------+-------------+
|5 |Maine coon           |Race         |
+--+---------------------+-------------+
|2 |Felis silvestris     |Espèce       |
+--+---------------------+-------------+
|1 |Canis canis          |Espèce       |
+--+---------------------+-------------+
|3 |Boxer                |Race         |
+--+---------------------+-------------+
|4 |Bleu russe           |Race         |
+--+---------------------+-------------+
|2 |Berger blanc suisse  |Race         |
+--+---------------------+-------------+
|1 |Berger allemand      |Race         |
+--+---------------------+-------------+
|4 |Alipiopsitta xanthops|Espèce       |
+--+---------------------+-------------+





Vous pouvez bien sûr combiner `LIMIT` et `ORDER BY`.


```sql
(SELECT id, nom, 'Race' AS table_origine FROM Race LIMIT 6)
UNION
(SELECT id, nom_latin, 'Espèce' AS table_origine FROM Espece LIMIT 3)
ORDER BY nom LIMIT 5;
```




+--+---------------------+-------------+
|id|nom                  |table_origine|
+==+=====================+=============+
|4 |Alipiopsitta xanthops|Espèce       |
+--+---------------------+-------------+
|1 |Berger allemand      |Race         |
+--+---------------------+-------------+
|2 |Berger blanc suisse  |Race         |
+--+---------------------+-------------+
|4 |Bleu russe           |Race         |
+--+---------------------+-------------+
|3 |Boxer                |Race         |
+--+---------------------+-------------+






## Exception pour les tris sur les requêtes intermédiaires

Je vous ai dit qu'il n'était pas possible de trier les résultats d'une requête intermédiaire. En réalité, c'est plus subtil que ça.
Dans une requête intermédiaire, il est possible d'utiliser un `ORDER BY` mais uniquement combiné à un `LIMIT`. Cela permettra de restreindre les résultats voulus (les _X_ premiers dans l'ordre défini par l'`ORDER BY` par exemple).

Prenons la requête suivante :


```sql
(SELECT id, nom, 'Race' AS table_origine FROM Race LIMIT 6)
UNION
(SELECT id, nom_latin, 'Espèce' AS table_origine FROM Espece LIMIT 3);
```


Cela vous renvoie bien 6 races et 3 espèces, mais imaginons que vous ne vouliez pas n'importe quelles races, mais les 6 dernières par ordre alphabétique du nom. Dans ce cas-là, vous pouvez utiliser `ORDER BY` en combinaison avec `LIMIT` dans la requête intermédiaire :


```sql
(SELECT id, nom, 'Race' AS table_origine FROM Race ORDER BY nom DESC LIMIT 6)
UNION
(SELECT id, nom_latin, 'Espèce' AS table_origine FROM Espece LIMIT 3);
```




+--+---------------------+-------------+
|id|nom                  |table_origine|
+==+=====================+=============+
|7 |Sphynx               |Race         |
+--+---------------------+-------------+
|6 |Singapura            |Race         |
+--+---------------------+-------------+
|8 |Nebelung             |Race         |
+--+---------------------+-------------+
|5 |Maine coon           |Race         |
+--+---------------------+-------------+
|3 |Boxer                |Race         |
+--+---------------------+-------------+
|4 |Bleu russe           |Race         |
+--+---------------------+-------------+
|4 |Alipiopsitta xanthops|Espèce       |
+--+---------------------+-------------+
|1 |Canis canis          |Espèce       |
+--+---------------------+-------------+
|2 |Felis silvestris     |Espèce       |
+--+---------------------+-------------+