# Petits rappels


## La syntaxe

Voici comment on ajoute une clé étrangère à une table déjà existante :


```sql
ALTER TABLE nom_table
ADD [CONSTRAINT fk_col_ref]         -- On donne un nom à la clé (facultatif)
    FOREIGN KEY colonne             -- La colonne sur laquelle on ajoute la clé
    REFERENCES table_ref(col_ref);  -- La table et la colonne de référence
```


[[information]]
| Le principe expliqué ici est exactement le même si l'on crée la clé en même temps que la table. La commande `ALTER TABLE` est simplement plus courte, c'est la raison pour laquelle je l'utilise dans mes exemples plutôt que `CREATE TABLE`.


## Le principe

Dans notre table _Animal_, nous avons par exemple mis une clé étrangère sur la colonne _race_id_, référençant la colonne _id_ de la table _Race_. Cela implique que chaque fois qu'une valeur est insérée dans cette colonne (soit en ajoutant une ligne, soit en modifiant une ligne existante), MySQL va vérifier que cette valeur existe bien dans la colonne _id_ de la table _Race_.
Aucun animal ne pourra donc avoir un _race_id_ qui ne correspond à rien dans notre base.


# Suppression d'une référence

Que se passe-t-il si l'on supprime la race des Boxers ? Certains animaux référencent cette espèce dans leur colonne _race_id_. On risque donc d'avoir des données incohérentes. Or, éviter cela est précisément la raison d'être de notre clé étrangère.

Essayons : 


```sql
DELETE FROM Race WHERE nom = 'Boxer';
```


```console
ERROR 1451 (23000): Cannot delete or update a parent row: a foreign key constraint fails (`elevage`.`animal`, CONSTRAINT `fk_race_id` FOREIGN KEY (`race_id`) REFERENCES `race` (`id`))
```

Ouf ! Visiblement, MySQL vérifie la contrainte de clé étrangère lors d'une suppression aussi, et empêche de supprimer une ligne si elle contient une référence utilisée ailleurs dans la base (ici, l'_id_ de la ligne est donc utilisé par certaines lignes de la table _Animal_).

Mais ça veut donc dire que chaque fois qu'on veut supprimer des lignes de la table _Race_, il faut d'abord supprimer toutes les références à ces races. Dans notre base, ça va encore, il n'y a pas énormément de clés étrangères, mais imaginez si l'_id_ de _Race_ servait de référence à des clés étrangères dans ne serait-ce que cinq ou six tables. Pour supprimer une seule race, il faudrait faire jusqu'à six ou sept requêtes.

C'est donc ici qu'intervient notre option `ON DELETE`, qui permet de changer la manière dont la clé étrangère gère la suppression d'une référence.


## Syntaxe

Voici comment on ajoute cette option à la clé étrangère :


```sql
ALTER TABLE nom_table
ADD [CONSTRAINT fk_col_ref]         
    FOREIGN KEY (colonne)            
    REFERENCES table_ref(col_ref)
    ON DELETE {RESTRICT | NO ACTION | SET NULL | CASCADE};  -- <-- Nouvelle option !
```


Il y a donc quatre comportements possibles, que je vais vous détailler tout de suite (bien que leurs noms soient plutôt clairs) : `RESTRICT`, `NO ACTION`, `SET NULL` et `CASCADE`.


## RESTRICT ou NO ACTION

`RESTRICT` est le comportement par défaut. Si l'on essaye de supprimer une valeur référencée par une clé étrangère, l'action est avortée et on obtient une erreur. `NO ACTION` a exactement le même effet.


[[attention]]
| Cette équivalence de `RESTRICT` et `NO ACTION` est propre à MySQL. Dans d'autres SGBD, ces deux options n'auront pas le même effet (`RESTRICT` étant généralement plus strict que `NO ACTION`).


## SET NULL

Si on choisit `SET NULL`, alors tout simplement, `NULL` est substitué aux valeurs dont la référence est supprimée.
Pour reprendre notre exemple, en supprimant la race des Boxers, tous les animaux auxquels on a attribué cette race verront la valeur de leur _race_id_ passer à `NULL`.

D'ailleurs, ça me semble plutôt intéressant comme comportement dans cette situation ! Nous allons donc modifier notre clé étrangère _fk_race_id_. C'est-à-dire que nous allons supprimer la clé, puis la recréer avec le bon comportement :


```sql
ALTER TABLE Animal DROP FOREIGN KEY fk_race_id;

ALTER TABLE Animal
ADD CONSTRAINT fk_race_id FOREIGN KEY (race_id) REFERENCES Race(id) ON DELETE SET NULL;
```

Dorénavant, si vous supprimez une race, tous les animaux auxquels vous avez attribué cette race auparavant auront `NULL` comme _race_id_.

Vérifions en supprimant les Boxers, depuis le temps qu'on essaye !


```sql
-- Affichons d'abord tous les animaux, avec leur race --
-- -----------------------------------------------------
SELECT Animal.nom, Animal.race_id, Race.nom as race FROM Animal
LEFT JOIN Race ON Animal.race_id = Race.id
ORDER BY race;

-- Supprimons ensuite la race 'Boxer' --
-- -------------------------------------
DELETE FROM Race WHERE nom = 'Boxer';

-- Réaffichons les animaux --
-- --------------------------
SELECT Animal.nom, Animal.race_id, Race.nom as race FROM Animal
LEFT JOIN Race ON Animal.race_id = Race.id
ORDER BY race;
```

Les ex-boxers existent toujours dans la table _Animal_, mais ils n'appartiennent plus à aucune race.


## CASCADE

Ce dernier comportement est le plus risqué (et le plus violent !  :pirate: ). En effet, cela supprime purement et simplement toutes les lignes qui référençaient la valeur supprimée ! 
Donc, si on choisit ce comportement pour la clé étrangère sur la colonne _espece_id_ de la table _Animal_, vous supprimez l'espèce "Perroquet amazone" et POUF !, quatre lignes de votre table _Animal_ (les quatre perroquets) sont supprimées en même temps.
Il faut donc être bien sûr de ce que l'on fait si l'on choisit `ON DELETE CASCADE`. Il y a cependant de nombreuses situations dans lesquelles c'est utile. Prenez par exemple un forum sur un site internet. Vous avez une table _Sujet_, et une table _Message_, avec une colonne _sujet_id_. Avec `ON DELETE CASCADE`, il vous suffit de supprimer un sujet pour que tous les messages de ce sujet soient également supprimés. Plutôt pratique non ?


[[attention]]
| Je le répète : soyez bien sûrs de ce que vous faites ! Je décline toute responsabilité en cas de perte de données causée par un `ON DELETE CASCADE` inconsidérément utilisé !

*[SGBD]: Système de Gestion de Bases de Données