On peut également rencontrer des problèmes de cohérence des données en cas de modification. En effet, si l'on change par exemple l'_id_ de la race "Singapura", tous les animaux qui ont l'ancien _id_ dans leur colonne _race_id_ référenceront une ligne qui n'existe plus. Les modifications de références de clés étrangères sont donc soumises aux mêmes restrictions que la suppression.

**Exemple** : essayons de modifier l'_id_ de la race "Singapura".


```sql
UPDATE Race SET id = 3 WHERE nom = 'Singapura';
```


```console
ERROR 1451 (23000): Cannot delete or update a parent row: a foreign key constraint fails (`elevage`.`animal`, CONSTRAINT `fk_race_id` FOREIGN KEY (`race_id`) REFERENCES `race` (`id`) ON DELETE SET NULL)
```


L'option permettant de définir le comportement en cas de modification est donc `ON UPDATE {RESTRICT | NO ACTION | SET NULL | CASCADE}`. Les quatre comportements possibles sont exactement les mêmes que pour la suppression.


- `RESTRICT` et `NO ACTION` : empêche la modification si elle casse la contrainte (comportement par défaut).
- `SET NULL` : met `NULL` partout où la valeur modifiée était référencée.
- `CASCADE` : modifie également la valeur là où elle est référencée.



## Petite explication à propos de CASCADE

`CASCADE` signifie que l'événement est répété sur les tables qui référencent la valeur. Pensez à des "réactions en cascade". Ainsi, une suppression provoquera d'autres suppressions, tandis qu'une modification provoquera d'autres… modifications !  :soleil: 

Modifions par exemple la clé étrangère sur _Animal.race_id_, avant de modifier l'_id_ de la race "Singapura" (jetez d'abord un œil aux données des tables _Race_ et _Animal_, afin de voir les différences).


```sql
-- Suppression de la clé --
-- ------------------------
ALTER TABLE Animal DROP FOREIGN KEY fk_race_id;

-- Recréation de la clé avec les bonnes options --
-- -----------------------------------------------
ALTER TABLE Animal
ADD CONSTRAINT fk_race_id FOREIGN KEY (race_id) REFERENCES Race(id)
    ON DELETE SET NULL   -- N'oublions pas de remettre le ON DELETE !
    ON UPDATE CASCADE;   

-- Modification de l'id des Singapura --
-- -------------------------------------
UPDATE Race SET id = 3 WHERE nom = 'Singapura';
```

Les animaux notés comme étant des "Singapura" ont désormais leur _race_id_ à 3. Parfait !  :magicien: 


[[attention]]
| En règle générale (dans 99,99 % des cas), c'est une très très mauvaise idée de changer la valeur d'un _id_ (ou de votre clé primaire auto-incrémentée, quel que soit le nom que vous lui donnez). En effet, vous risquez des problèmes avec l'auto-incrément, si vous donnez une valeur non encore atteinte par auto-incrémentation par exemple. Soyez bien conscients de ce que vous faites. Je ne l'ai montré ici que pour illustrer le `ON UPDATE`, parce que toutes nos clés étrangères référencent des clés primaires. Mais ce n'est pas le cas partout. Une clé étrangère pourrait parfaitement référencer un simple index, dépourvu de toute auto-incrémentation, auquel cas vous pouvez vous amuser à en changer la valeur autant de fois que vous le voudrez.


Modifions une dernière fois cette clé étrangère pour remettre l'option `ON UPDATE` par défaut.


```sql
ALTER TABLE Animal DROP FOREIGN KEY fk_race_id;

ALTER TABLE Animal
ADD CONSTRAINT fk_race_id FOREIGN KEY (race_id) REFERENCES Race(id) ON DELETE SET NULL;
```