Lorsque vous voulez insérer une ligne dans une table, vous pouvez utiliser la commande bien connue `INSERT INTO`, ou vous pouvez utiliser `REPLACE INTO`. La différence entre ces deux requêtes est la façon qu'elles ont de gérer les contraintes d'unicité (ça tombe bien, c'est le sujet de ce chapitre !).

Dans le cas d'une insertion qui enfreint une contrainte d'unicité, `REPLACE` ne va ni renvoyer une erreur, ni ignorer l'insertion (comme `INSERT INTO [IGNORE]`). `REPLACE` va, purement, simplement et violemment, remplacer l'ancienne ligne par la nouvelle.


[[question]]
| Mais que veut dire "remplacer l'ancienne ligne par la nouvelle" ?

Prenons par exemple Spoutnik la tortue, qui se trouve dans notre table _Animal_. 


```sql
SELECT id, sexe, date_naissance, nom, espece_id FROM Animal WHERE nom = 'Spoutnik';
```




+--+----+-------------------+--------+---------+
|id|sexe|date_naissance     |nom     |espece_id|
+==+====+===================+========+=========+
|53|M   |2007-04-02 01:45:00|Spoutnik|3        |
+--+----+-------------------+--------+---------+





Étant donné que nous avons mis un index `UNIQUE` sur (_nom, espece_id_), il est absolument impossible d'avoir une autre tortue du nom de Spoutnik dans notre table. La requête suivante va donc échouer lamentablement.


```sql
INSERT INTO Animal (sexe, nom, date_naissance, espece_id)
VALUES ('F', 'Spoutnik', '2010-08-06 15:05:00', 3);
```


```console
ERROR 1062 (23000): Duplicate entry 'Spoutnik-3' for key 'ind_uni_nom_espece_id'
```


Par contre, si on utilise `REPLACE` au lieu de `INSERT` :


```sql
REPLACE INTO Animal (sexe, nom, date_naissance, espece_id)
VALUES ('F', 'Spoutnik', '2010-08-06 15:05:00', 3);
```


```console
Query OK, 2 rows affected (0.06 sec)
```


Pas d'erreur, mais vous pouvez voir que **deux** lignes ont été affectées par la commande. En effet, Spoutnik est mort, vive Spoutnik !


```sql
SELECT id, sexe, date_naissance, nom, espece_id FROM Animal WHERE nom = 'Spoutnik';
```




+--+----+-------------------+--------+---------+
|id|sexe|date_naissance     |nom     |espece_id|
+==+====+===================+========+=========+
|63|F   |2010-08-06 15:05:00|Spoutnik|3        |
+--+----+-------------------+--------+---------+





Comme vous voyez, nous n'avons toujours qu'une seule tortue du nom de Spoutnik, mais il ne s'agit plus du mâle né le 2 avril 2007 que nous avions précédemment, mais bien de la femelle née le 6 août 2010 que nous venons d'insérer avec `REPLACE INTO`. L'autre Spoutnik a été purement et simplement remplacé.

Attention cependant, quand je dis que l'ancien Spoutnik a été remplacé, j'utilise le terme "remplacé", car il s'agit de la traduction de `REPLACE`. Il s'agit cependant d'un abus de langage. En réalité, la ligne de **l'ancien Spoutnik a été supprimée**, et ensuite seulement, **le nouveau Spoutnik a été inséré**. C'est d'ailleurs pour cela que les deux Spoutnik n'ont **pas du tout le même _id_**.


# Remplacement de plusieurs lignes

Pourquoi ai-je bien précisé qu'il ne s'agissait pas vraiment d'un remplacement, mais d'une suppression suivie d'une insertion ? Parce que ce comportement a des conséquences qu'il ne faut pas négliger !

Prenons par exemple une table sur laquelle existent plusieurs contraintes d'unicité. C'est le cas d'_Animal_, puisqu'on a cet index `UNIQUE` (_nom, espece_id_), ainsi que la clé primaire (l'_id_ doit donc être unique aussi).

Nous allons insérer avec `REPLACE` une ligne qui viole les deux contraintes d'unicité : 


```sql
REPLACE INTO Animal (id, sexe, nom, date_naissance, espece_id) -- Je donne moi-même un id, qui existe déjà !
VALUES (32, 'M', 'Spoutnik', '2009-07-26 11:52:00', 3);        -- Et Spoutnik est mon souffre-douleur du jour.
```


```console
Query OK, 3 rows affected (0.05 sec)
```

Cette fois-ci, **trois** lignes ont été affectées. Trois !
Tout simplement, les deux lignes qui empêchaient l'insertion à cause des contraintes d'unicité ont été supprimées. La ligne qui avait l'_id_ 32, ainsi que l'ancien Spoutnik ont été supprimés. Le nouveau Spoutnik a ensuite été inséré.


[[attention]]
| Je l'ai fait ici pour vous donner un exemple, mais je rappelle que c'est une **très mauvaise idée** de donner soi-même un id lorsque la colonne est auto-incrémentée (ce qui sera presque toujours le cas).


# LOAD DATA INFILE

`REPLACE` est également disponible avec `LOAD DATA INFILE`. Le comportement est exactement le même.

Bien entendu, `IGNORE` et `REPLACE` ne peuvent pas être utilisés en même temps. C'est l'un **ou** l'autre.


## Syntaxe


```sql
LOAD DATA [LOCAL] INFILE 'nom_fichier' REPLACE   -- se place au même endroit que IGNORE
INTO TABLE nom_table
[FIELDS
    [TERMINATED BY '\t']
    [ENCLOSED BY '']
    [ESCAPED BY '\\' ]
]
[LINES 
    [STARTING BY '']    
    [TERMINATED BY '\n']
]
[IGNORE nombre LINES]
[(nom_colonne,...)];
```