Pour regrouper les lignes selon un critère, il faut utiliser `GROUP BY`, qui se place après l'éventuelle clause `WHERE` (sinon directement après `FROM`), suivi du nom de la colonne à utiliser comme critère de regroupement.


```sql
SELECT ...
FROM nom_table
[WHERE condition]
GROUP BY nom_colonne;
```


**Exemple 1** : comptons les lignes dans la table _Animal_, en regroupant sur le critère de l'espèce (donc avec la colonne _espece_id_).


```sql
SELECT COUNT(*) AS nb_animaux
FROM Animal
GROUP BY espece_id;
```




+----------+
|nb_animaux|
+==========+
|21        |
+----------+
|20        |
+----------+
|15        |
+----------+
|4         |
+----------+





**Exemple 2** : Même chose, mais on ne prend que les mâles cette fois-ci.


```sql
SELECT COUNT(*) AS nb_males
FROM Animal
WHERE sexe = 'M'
GROUP BY espece_id;
```




+--------+
|nb_males|
+========+
|10      |
+--------+
|9       |
+--------+
|4       |
+--------+
|3       |
+--------+





C'est déjà intéressant, mais nous n'allons pas en rester là. En effet, il serait quand même mieux de savoir à quelle espèce correspond quel nombre !


# Voir d'autres colonnes

Pour savoir à quoi correspond chaque nombre, il suffit d'afficher également le critère qui a permis de regrouper les lignes. Dans notre cas, _espece_id_.


```sql
SELECT espece_id, COUNT(*) AS nb_animaux
FROM Animal
GROUP BY espece_id;
```




+---------+----------+
|espece_id|nb_animaux|
+=========+==========+
|1        |21        |
+---------+----------+
|2        |20        |
+---------+----------+
|3        |15        |
+---------+----------+
|4        |4         |
+---------+----------+





C'est déjà mieux, mais l'idéal serait d'avoir le nom des espèces directement. Qu'à cela ne tienne, il suffit de faire une jointure ! Sans oublier de changer le critère et de mettre _nom_courant_ à la place de _espece_id_.


```sql
SELECT nom_courant, COUNT(*) AS nb_animaux
FROM Animal
INNER JOIN Espece ON Animal.espece_id = Espece.id
GROUP BY nom_courant;
```




+-----------------+----------+
|nom_courant      |nb_animaux|
+=================+==========+
|Chat             |20        |
+-----------------+----------+
|Chien            |21        |
+-----------------+----------+
|Perroquet amazone|4         |
+-----------------+----------+
|Tortue d'Hermann |15        |
+-----------------+----------+






# Colonnes sélectionnées


## La règle SQL

Lorsque l'on fait un groupement dans une requête, avec `GROUP BY`, on ne peut sélectionner que deux types d'éléments dans la clause `SELECT` :


- une ou des colonnes **ayant servi de critère** pour le regroupement ;
- **une fonction d'agrégation** (agissant sur n'importe quelle colonne).


Cette règle est d'ailleurs logique. Imaginez la requête suivante :


```sql
SELECT nom_courant, COUNT(*) AS nb_animaux, date_naissance
FROM Animal
INNER JOIN Espece ON Animal.espece_id = Espece.id
GROUP BY nom_courant;
```


Que vient faire la date de naissance dans cette histoire ? Et surtout, quelle date de naissance espère-t-on sélectionner ? **Chaque ligne représente une espèce** puisque l'on a regroupé les lignes sur la base de _Espece.nom_courant_. Donc _date_naissance_ n'a aucun sens par rapport aux groupes formés, une espèce n'ayant pas de date de naissance. Il en est de même pour les colonnes _sexe_ ou _commentaires_ par exemple.

Qu'en est-il des colonnes _Espece.id_, _Animal.espece_id_, ou _Espece.nom_latin_ ? En groupant sur le nom courant de l'espèce, ces différentes colonnes ont un sens, et pourraient donc être utiles. Il a cependant été décidé que **par sécurité, la sélection de colonnes n'étant pas dans les critères de groupement serait interdite**. Cela afin d'éviter les situations comme au-dessus, où les colonnes sélectionnées n'ont aucun sens par rapport au groupement fait. 
Par conséquent, si vous voulez afficher également l'id de l'espèce et son nom latin, il vous faudra grouper sur les trois colonnes : _Espece.nom_latin_, _Espece.nom_courant_ et _Espece.id_. Les groupes créés seront les mêmes qu'en groupant uniquement sur _Espece.nom_courant_, mais votre requête respectera les standards SQL.


```sql
SELECT Espece.id, nom_courant, nom_latin, COUNT(*) AS nb_animaux
FROM Animal
INNER JOIN Espece ON Animal.espece_id = Espece.id
GROUP BY nom_courant, Espece.id, nom_latin;
```




+--+-----------------+---------------------+----------+
|id|nom_courant      |nom_latin            |nb_animaux|
+==+=================+=====================+==========+
|2 |Chat             |Felis silvestris     |20        |
+--+-----------------+---------------------+----------+
|1 |Chien            |Canis canis          |21        |
+--+-----------------+---------------------+----------+
|4 |Perroquet amazone|Alipiopsitta xanthops|4         |
+--+-----------------+---------------------+----------+
|3 |Tortue d'Hermann |Testudo hermanni     |15        |
+--+-----------------+---------------------+----------+






## Le cas MySQL

On ne le répétera jamais assez : MySQL est un SGBD extrêmement permissif. Dans certains cas, c'est bien pratique, mais c'est toujours dangereux. 
Et notamment en ce qui concerne `GROUP BY`, MySQL ne sera pas perturbé pour un sou si vous sélectionnez une colonne qui n'est pas dans les critères de regroupement. Reprenons la requête qui sélectionne la colonne _date_naissance_ alors que le regroupement se fait sur la base de l'_espece_id_. J'insiste, cette requête ne respecte pas la norme SQL, et n'a aucun sens. La plupart des SGBD vous renverront une erreur si vous tentez de l'exécuter.


```sql
SELECT nom_courant, COUNT(*) AS nb_animaux, date_naissance
FROM Animal
INNER JOIN Espece ON Animal.espece_id = Espece.id
GROUP BY espece_id;
```

Pourtant, loin de rouspéter, MySQL donne le résultat suivant :




+-----------------+----------+-------------------+
|nom_courant      |nb_animaux|date_naissance     |
+=================+==========+===================+
|Chat             |20        |2010-03-24 02:23:00|
+-----------------+----------+-------------------+
|Chien            |21        |2010-04-05 13:43:00|
+-----------------+----------+-------------------+
|Perroquet amazone|4         |2007-03-04 19:36:00|
+-----------------+----------+-------------------+
|Tortue d'Hermann |15        |2009-08-03 05:12:00|
+-----------------+----------+-------------------+





MySQL a tout simplement pris n'importe quelle valeur parmi celles du groupe pour la date de naissance. D'ailleurs, il est tout à fait possible que vous ayez obtenu des valeurs différentes des miennes.
Soyez donc très prudents lorsque vous utilisez `GROUP BY`. Vous faites peut-être des requêtes qui n'ont aucun sens, et MySQL ne vous en avertira pas !


# Tri des données

Par défaut dans MySQL, les données sont **triées sur la base du critère de regroupement**. C'est la raison pour laquelle, dans la requête précédente, la colonne _nom_courant_ est triée par ordre alphabétique : c'est le premier critère de regroupement. MySQL permet d'utiliser les mots-clés `ASC` et `DESC` dans une clause `GROUP BY` pour choisir un tri ascendant (par défaut) ou descendant.


```sql
SELECT Espece.id, nom_courant, nom_latin, COUNT(*) AS nb_animaux
FROM Animal
INNER JOIN Espece ON Animal.espece_id = Espece.id
GROUP BY nom_courant DESC, Espece.id, nom_latin;  -- On trie par ordre anti-alphabétique sur le nom_courant
```




+--+-----------------+---------------------+----------+
|id|nom_courant      |nom_latin            |nb_animaux|
+==+=================+=====================+==========+
|3 |Tortue d'Hermann |Testudo hermanni     |15        |
+--+-----------------+---------------------+----------+
|4 |Perroquet amazone|Alipiopsitta xanthops|4         |
+--+-----------------+---------------------+----------+
|1 |Chien            |Canis canis          |21        |
+--+-----------------+---------------------+----------+
|2 |Chat             |Felis silvestris     |20        |
+--+-----------------+---------------------+----------+





Mais rien n'empêche d'utiliser une clause `ORDER BY` classique, après la clause `GROUP BY`. L'`ORDER BY` sera **prioritaire** sur l'ordre défini par la clause de regroupement.

Dans le même ordre d'idées, il n'est possible de faire un tri des données qu'à partir d'une colonne qui fait partie des critères de regroupement, ou à partir d'une fonction d'agrégation. Ça n'a pas plus de sens de trier les espèces par date de naissance que de sélectionner une date de naissance par espèce.

Vous pouvez par contre parfaitement écrire ceci :


```sql
SELECT Espece.id, nom_courant, nom_latin, COUNT(*) AS nb_animaux
FROM Animal
INNER JOIN Espece ON Animal.espece_id = Espece.id
GROUP BY nom_courant, Espece.id, nom_latin
ORDER BY nb_animaux;
```




+--+-----------------+---------------------+----------+
|id|nom_courant      |nom_latin            |nb_animaux|
+==+=================+=====================+==========+
|4 |Perroquet amazone|Alipiopsitta xanthops|4         |
+--+-----------------+---------------------+----------+
|3 |Tortue d'Hermann |Testudo hermanni     |15        |
+--+-----------------+---------------------+----------+
|2 |Chat             |Felis silvestris     |20        |
+--+-----------------+---------------------+----------+
|1 |Chien            |Canis canis          |21        |
+--+-----------------+---------------------+----------+





Notez que la norme SQL veut que l'on n'utilise pas d'expressions (fonction, opération mathématique,…) dans `GROUP BY` ou `ORDER BY`. C'est la raison pour laquelle j'ai mis `ORDER BY nb_animaux` et non pas `ORDER BY COUNT(*)`, bien qu'avec MySQL les deux fonctionnent. Pensez donc à utiliser des alias pour ces situations.


# Et les autres espèces ?

La requête suivante nous donne le nombre d'animaux qu'on possède pour chaque espèce **dont on possède au moins un animal**. Comment peut-on faire pour afficher également les autres espèces ?


```sql
SELECT Espece.nom_courant, COUNT(*) AS nb_animaux
FROM Animal
INNER JOIN Espece ON Animal.espece_id = Espece.id
GROUP BY nom_courant;
```


Essayons donc avec une jointure externe, puisqu'il faut tenir compte de toutes les espèces, même celles qui n'ont pas de correspondance dans la table _Animal_.


```sql
SELECT Espece.nom_courant, COUNT(*) AS nb_animaux
FROM Animal
RIGHT JOIN Espece ON Animal.espece_id = Espece.id -- RIGHT puisque la table Espece est à droite.
GROUP BY nom_courant;
```




+-----------------+----------+
|nom_courant      |nb_animaux|
+=================+==========+
|Chat             |20        |
+-----------------+----------+
|Chien            |21        |
+-----------------+----------+
|Perroquet amazone|4         |
+-----------------+----------+
|Rat brun         |1         |
+-----------------+----------+
|Tortue d'Hermann |15        |
+-----------------+----------+





Les rats bruns apparaissent bien. En revanche, ce n'est pas 1 qu'on attend, mais 0, puisqu'on n'a pas de rats dans notre élevage. Cela dit, ce résultat est logique : avec la jointure externe, on aura une ligne correspondant aux rats bruns, avec `NULL` dans toutes les colonnes de la table _Animal_. Donc ce qu'il faudrait, c'est avoir les cinq espèces, mais ne compter que lorsqu'il y a un animal correspondant. Pour ce faire, il suffit de faire `COUNT(Animal.espece_id)` par exemple.


```sql
SELECT Espece.nom_courant, COUNT(Animal.espece_id) AS nb_animaux
FROM Animal
RIGHT JOIN Espece ON Animal.espece_id = Espece.id
GROUP BY nom_courant;
```




+-----------------+----------+
|nom_courant      |nb_animaux|
+=================+==========+
|Chat             |20        |
+-----------------+----------+
|Chien            |21        |
+-----------------+----------+
|Perroquet amazone|4         |
+-----------------+----------+
|Rat brun         |0         |
+-----------------+----------+
|Tortue d'Hermann |15        |
+-----------------+----------+





C'est pas magique ça ?  :magicien:

*[SGBD]: Système de Gestion de Base de Données