Parlons maintenant de l'option `WITH ROLLUP` de `GROUP BY`. Cette option va afficher des lignes supplémentaires dans la table de résultats. Ces lignes représenteront des "super-groupes" (ou super-agrégats), donc des "groupes de groupes". Deux petits exemples, et vous aurez compris !


## Exemple avec un critère de regroupement


```sql
SELECT nom_courant, COUNT(*) as nb_animaux
FROM Animal
INNER JOIN Espece ON Espece.id = Animal.espece_id
GROUP BY nom_courant WITH ROLLUP;
```




+-----------------+----------+
|nom_courant      |nb_animaux|
+=================+==========+
|Chat             |20        |
+-----------------+----------+
|Chien            |21        |
+-----------------+----------+
|Perroquet amazone|4         |
+-----------------+----------+
|Tortue d'Hermann |15        |
+-----------------+----------+
|NULL             |60        |
+-----------------+----------+





Nous avons donc 20 chats, 21 chiens, 4 perroquets et 15 tortues. Et combien font 20 + 21 + 4 + 15 ? 60 ! Exactement. La ligne supplémentaire représente donc le regroupement de nos quatre groupes basé sur le critère `GROUP BY nom_courant`.


## Exemple avec deux critères de regroupement


```sql
SELECT nom_courant, sexe, COUNT(*) as nb_animaux
FROM Animal
INNER JOIN Espece ON Espece.id = Animal.espece_id
WHERE sexe IS NOT NULL                            
GROUP BY nom_courant, sexe WITH ROLLUP;
```




+-----------------+----+----------+
|nom_courant      |sexe|nb_animaux|
+=================+====+==========+
|Chat             |F   |9         |
+-----------------+----+----------+
|Chat             |M   |9         |
+-----------------+----+----------+
|Chat             |NULL|18        |
+-----------------+----+----------+
|Chien            |F   |11        |
+-----------------+----+----------+
|Chien            |M   |10        |
+-----------------+----+----------+
|Chien            |NULL|21        |
+-----------------+----+----------+
|Perroquet amazone|F   |1         |
+-----------------+----+----------+
|Perroquet amazone|M   |3         |
+-----------------+----+----------+
|Perroquet amazone|NULL|4         |
+-----------------+----+----------+
|Tortue d'Hermann |F   |10        |
+-----------------+----+----------+
|Tortue d'Hermann |M   |4         |
+-----------------+----+----------+
|Tortue d'Hermann |NULL|14        |
+-----------------+----+----------+
|NULL             |NULL|57        |
+-----------------+----+----------+





Les deux premières lignes correspondent aux nombres de chats mâles et femelles. Jusque-là, rien de nouveau. Par contre, la troisième ligne est une ligne insérée par `WITH ROLLUP`, et contient le nombre de chats (mâles et femelles). Nous avons fait des groupes en séparant les espèces et les sexes, et `WITH ROLLUP` a créé des "super-groupes" en regroupant les sexes mais gardant les espèces séparées.
Nous avons donc également le nombre de chiens à la sixième ligne, de perroquets à la neuvième, et de tortues à la douzième. Quant à la toute dernière ligne, c'est un "super-super-groupe" qui réunit tous les groupes ensemble.

C'est en utilisant `WITH ROLLUP` qu'on se rend compte que l'ordre des critères est vraiment important. En effet, voyons ce qui se passe si on échange les deux critères _nom_courant_ et _sexe_.


```sql
SELECT nom_courant, sexe, COUNT(*) as nb_animaux
FROM Animal
INNER JOIN Espece ON Espece.id = Animal.espece_id
WHERE sexe IS NOT NULL
GROUP BY sexe, nom_courant WITH ROLLUP;
```




+-----------------+----+----------+
|nom_courant      |sexe|nb_animaux|
+=================+====+==========+
|Chat             |F   |9         |
+-----------------+----+----------+
|Chien            |F   |11        |
+-----------------+----+----------+
|Perroquet amazone|F   |1         |
+-----------------+----+----------+
|Tortue d'Hermann |F   |10        |
+-----------------+----+----------+
|NULL             |F   |31        |
+-----------------+----+----------+
|Chat             |M   |9         |
+-----------------+----+----------+
|Chien            |M   |10        |
+-----------------+----+----------+
|Perroquet amazone|M   |3         |
+-----------------+----+----------+
|Tortue d'Hermann |M   |4         |
+-----------------+----+----------+
|NULL             |M   |26        |
+-----------------+----+----------+
|NULL             |NULL|57        |
+-----------------+----+----------+





Cette fois-ci, les super-groupes ne correspondent pas aux espèces, mais aux sexes, c'est-à-dire au premier critère. Le regroupement se fait bien dans l'ordre donné par les critères.


[[information]]
| J'ai ajouté la condition `WHERE sexe IS NOT NULL` pour des raisons de lisibilité uniquement, étant donné que sans cette condition, d'autres `NULL` seraient apparus, rendant plus compliquée l'explication des super-agrégats.



## NULL, c'est pas joli

Il est possible d'éviter d'avoir ces NULL dans les lignes des super-groupes. Pour cela, on peut utiliser la fonction COALESCE().
Cette fonction prend autant de paramètres que l'on veut, et renvoie le premier paramètre non `NULL`.

**Exemples** :


```sql
SELECT COALESCE(1, NULL, 3, 4); -- 1
SELECT COALESCE(NULL, 2);       -- 2
SELECT COALESCE(NULL, NULL, 3); -- 3
```


Voici comment l'utiliser dans le cas des super-agrégats.


```sql
SELECT COALESCE(nom_courant, 'Total'), COUNT(*) as nb_animaux
FROM Animal
INNER JOIN Espece ON Espece.id = Animal.espece_id
GROUP BY nom_courant WITH ROLLUP;
```




+------------------------------+----------+
|COALESCE(nom_courant, 'Total')|nb_animaux|
+==============================+==========+
|Chat                          |20        |
+------------------------------+----------+
|Chien                         |21        |
+------------------------------+----------+
|Perroquet amazone             |4         |
+------------------------------+----------+
|Tortue d'Hermann              |15        |
+------------------------------+----------+
|Total                         |60        |
+------------------------------+----------+





Tant qu'il s'agit de simples groupes, _nom_courant_ contient bien le nom de l'espèce. `COALESCE()` renvoie donc celui-ci. Par contre, quand il s'agit du super-groupe, la colonne _nom_courant_ du résultat contient `NULL`, et donc `COALESCE()` va renvoyer "Total".


[[attention]]
| Si vous utilisez `COALESCE()` dans ce genre de situation, il est impératif que vos critères de regroupement ne contiennent pas `NULL` (ou que vous éliminiez ces lignes-là). Sinon, vous aurez "Total" à des lignes qui ne sont pas des super-groupes.


**Exemple** : groupons sur le sexe, sans éliminer les lignes pour lesquelles le sexe n'est pas défini.


```sql
SELECT COALESCE(sexe, 'Total'), COUNT(*) as nb_animaux
FROM Animal
INNER JOIN Espece ON Espece.id = Animal.espece_id
GROUP BY sexe WITH ROLLUP;
```




+-----------------------+----------+
|COALESCE(sexe, 'Total')|nb_animaux|
+=======================+==========+
|Total                  |3         |
+-----------------------+----------+
|F                      |31        |
+-----------------------+----------+
|M                      |26        |
+-----------------------+----------+
|Total                  |60        |
+-----------------------+----------+