## Vocabulaire

Lorsque l'on valide les requêtes d'une transaction, on dit aussi que l'on **commite** les changements. À l'inverse, l'annulation des requêtes s'appelle un **rollback**.


## Comportement par défaut

Vous l'aurez compris, par défaut MySQL ne travaille pas avec les transactions. Chaque requête effectuée est directement **commitée** (validée). On ne peut pas revenir en arrière. On peut donc en fait considérer que chaque requête constitue une transaction, qui est automatiquement commitée. Par défaut, MySQL est donc en mode "**autocommit**".

Pour quitter ce mode, il suffit de lancer la requête suivante :


```sql
SET autocommit=0;
```


Une fois que vous n'êtes plus en autocommit, chaque modification de donnée devra être commitée pour prendre effet. Tant que vos modifications ne sont pas validées, vous pouvez à tout moment les annuler (faire un rollback).


# Valider/annuler les changements

Les commandes pour commiter et faire un rollback sont relativement faciles à retenir :


```sql
COMMIT;   -- pour valider les requêtes
ROLLBACK; -- pour annuler les requêtes
```



## Exemples de transactions en mode non-autocommit


Si ce n'est pas déjà fait, changez le mode par défaut de MySQL grâce à la commande que nous venons de voir.

**Première expérience** : annulation des requêtes.

Exécutez ces quelques requêtes :


```sql
INSERT INTO Animal (nom, espece_id, date_naissance, sexe) 
VALUES ('Baba', 5, '2012-02-13 15:45:00', 'F'); 
INSERT INTO Animal (nom, espece_id, date_naissance, sexe) 
VALUES ('Bibo', 5, '2012-02-13 15:48:00', 'M');
INSERT INTO Animal (nom, espece_id, date_naissance, sexe) 
VALUES ('Buba', 5, '2012-02-13 18:32:00', 'F'); -- Insertion de 3 rats bruns

UPDATE Espece
SET prix = 20
WHERE id = 5;  -- Les rats bruns coûtent maintenant 20 euros au lieu de 10
```


Faites maintenant un `SELECT` sur les tables _Espece_ et _Animal_. 


```sql
SELECT * 
FROM Animal
WHERE espece_id = 5;

SELECT * 
FROM Espece 
WHERE id = 5;
```


Les changements faits sont bien visibles. Les rats bruns valent maintenant 20 euros, et nos trois nouvelles bestioles ont bien été insérées. Cependant, un simple rollback va annuler ces changements.


```sql
ROLLBACK;
```


Nos rats coûtent à nouveau 10 euros et Baba, Bibo et Buba ont disparu.

**Deuxième expérience** : Interruption de la transaction.

Exécutez à nouveau les trois requêtes `INSERT` et la requête `UPDATE`.
Ensuite, quittez votre client MySQL (fermez simplement la fenêtre, ou tapez `quit` ou `exit`).

Reconnectez-vous et vérifiez vos données : les rats valent 10 euros, et Baba, Bibo et Buba n'existent pas. Les changements n'ont pas été commités, c'est comme s'il ne s'était rien passé !


[[attention]]
| Le mode autocommit est de nouveau activé ! Le fait de faire `SET autocommit = 0;` n'est valable que pour la session courante. Or, en ouvrant une nouvelle connexion, vous avez créé une nouvelle session. Désactivez donc à nouveau ce mode.


**Troisième expérience** : validation et annulation.

Exécutez la séquence de requêtes suivante :


```sql
INSERT INTO Animal (nom, espece_id, date_naissance, sexe) 
VALUES ('Baba', 5, '2012-02-13 15:45:00', 'F'); 
INSERT INTO Animal (nom, espece_id, date_naissance, sexe) 
VALUES ('Bibo', 5, '2012-02-13 15:48:00', 'M');
INSERT INTO Animal (nom, espece_id, date_naissance, sexe) 
VALUES ('Buba', 5, '2012-02-13 18:32:00', 'F'); -- Insertion de 3 rats bruns

COMMIT;

UPDATE Espece
SET prix = 20
WHERE id = 5;  -- Les rats valent 20 euros

ROLLBACK;
```


Si vous n'avez pas oublié de réactiver le mode non-autocommit, vous avez maintenant trois nouveaux rats bruns (les requêtes d'insertion ayant été validées), et ils ne valent toujours que 10 euros chacun (la modification de l'espèce ayant été annulée).

**Quatrième expérience** : visibilité des changements non-commités.

Exécutez la requête suivante :


```sql
UPDATE Animal 
SET commentaires = 'Queue coupée'
WHERE nom = 'Bibo' AND espece_id = 5;
```


Ensuite, tout en laissant ce client MySQL ouvert, ouvrez-en un deuxième. Connectez-vous comme d'habitude à la base de données _elevage_. Vous avez maintenant deux sessions ouvertes, connectées à votre base de données.
Sélectionnez les rats bruns.


```sql
SELECT id, sexe, nom, commentaires, espece_id, race_id 
FROM Animal
WHERE espece_id = 5;
```




+--+----+----+------------+---------+-------+
|id|sexe|nom |commentaires|espece_id|race_id|
+==+====+====+============+=========+=======+
|69|F   |Baba|NULL        |5        |NULL   |
+--+----+----+------------+---------+-------+
|70|M   |Bibo|NULL        |5        |NULL   |
+--+----+----+------------+---------+-------+
|71|F   |Buba|NULL        |5        |NULL   |
+--+----+----+------------+---------+-------+





Les commentaires de Bibo sont toujours vides. Les changements non-commités ne sont donc pas visibles à l'extérieur de la transaction qui les a faits. En particulier, une autre session n'a pas accès à ces changements.

Annulez la modification de Bibo dans la première session avec un `ROLLBACK`. Vous pouvez fermer la seconde session.


# Démarrer explicitement une transaction

En désactivant le mode autocommit, en réalité, on démarre une transaction. Et chaque fois que l'on fait un rollback ou un commit (ce qui met fin à la transaction), une nouvelle transaction est créée automatiquement, et ce tant que la session est ouverte.

Il est également possible de démarrer explicitement une transaction, auquel cas on peut laisser le mode autocommit activé, et décider au cas par cas des requêtes qui doivent être faites dans une transaction.

Repassons donc en mode autocommit :


```sql
SET autocommit=1;
```


Pour démarrer une transaction, il suffit de lancer la commande suivante :


```sql
START TRANSACTION;
```



[[information]]
| Avec MySQL, il est également possible de démarrer une transaction avec `BEGIN` ou `BEGIN WORK`. Cependant, il est conseillé d'utiliser plutôt `START TRANSACTION`, car il s'agit de la commande SQL standard.


Une fois la transaction ouverte, les requêtes devront être validées pour prendre effet. Attention au fait qu'un `COMMIT` ou un `ROLLBACK` met fin automatiquement à la transaction, donc les commandes suivantes seront à nouveau commitées automatiquement si une nouvelle transaction n'est pas ouverte.


## Exemples de transactions en mode autocommit


```sql
-- Insertion d'un nouveau rat brun, plus vieux
INSERT INTO Animal (nom, espece_id, date_naissance, sexe) 
VALUES ('Momy', 5, '2008-02-01 02:25:00', 'F');

-- Ouverture d'une transaction
START TRANSACTION;

-- La nouvelle rate est la mère de Buba et Baba
UPDATE Animal 
SET mere_id = LAST_INSERT_ID()
WHERE espece_id = 5
AND nom IN ('Baba', 'Buba');

-- On annule les requêtes de la transaction, ce qui termine celle-ci
ROLLBACK;

-- La nouvelle rate est la mère de Bibo
UPDATE Animal 
SET mere_id = LAST_INSERT_ID()
WHERE espece_id = 5
AND nom = 'Bibo';

-- Nouvelle transaction
START TRANSACTION;

-- Suppression de Buba
DELETE FROM Animal 
WHERE espece_id = 5
AND nom = 'Buba';

-- On valide les requêtes de la transaction, ce qui termine celle-ci
COMMIT;
```

Si vous avez bien suivi, vous devriez savoir les changements qui ont été faits.


[[secret]]
| - On a inséré Momy (insertion hors transaction)
| - Momy n'est pas la mère de Baba (modification dans une transaction dont les requêtes ont été annulées)
| - Momy est la mère de Bibo (modification hors transaction)
| - Buba a été supprimée (suppression dans une transaction dont les requêtes ont été commitées)
| 
| 
| 
| ```sql
| SELECT id, nom, espece_id, mere_id 
| FROM Animal
| WHERE espece_id = 5;
| ```
| 
| 
| 

+--+----+---------+-------+
|id|nom |espece_id|mere_id|
+==+====+=========+=======+
|69|Baba|5        |NULL   |
+--+----+---------+-------+
|70|Bibo|5        |72     |
+--+----+---------+-------+
|72|Momy|5        |NULL   |
+--+----+---------+-------+








# Jalon de transaction

Lorsque l'on travaille dans une transaction, et que l'on constate que certaines requêtes posent problème, on n'a pas toujours envie de faire un rollback depuis le début de la transaction, annulant toutes les requêtes alors qu'une partie aurait pu être validée.
**Il n'est pas possible de démarrer une transaction à l'intérieur d'une transaction**. Par contre, on peut poser des **jalons de transaction**. Il s'agit de **points de repère**, qui permettent d'annuler toutes les requêtes exécutées depuis ce jalon, et non toutes les requêtes de la transaction.


## Syntaxe

Trois nouvelles commandes suffisent pour pouvoir utiliser pleinement les jalons :


```sql
SAVEPOINT nom_jalon; -- Crée un jalon avec comme nom "nom_jalon"

ROLLBACK [WORK] TO [SAVEPOINT] nom_jalon; -- Annule les requêtes exécutées depuis le jalon "nom_jalon", WORK et SAVEPOINT ne sont pas obligatoires

RELEASE SAVEPOINT nom_jalon; -- Retire le jalon "nom_jalon" (sans annuler, ni valider les requêtes faites depuis)
```


**Exemple** : exécutez les requêtes suivantes.


```sql
START TRANSACTION;

INSERT INTO Animal (nom, espece_id, date_naissance, sexe) 
VALUES ('Popi', 5, '2007-03-11 12:45:00', 'M');

SAVEPOINT jalon1;

INSERT INTO Animal (nom, espece_id, date_naissance, sexe) 
VALUES ('Momo', 5, '2007-03-12 05:23:00', 'M');

ROLLBACK TO SAVEPOINT jalon1;

INSERT INTO Animal (nom, espece_id, date_naissance, sexe) 
VALUES ('Mimi', 5, '2007-03-12 22:03:00', 'F');

COMMIT;
```


On n'utilise qu'une seule transaction, on valide à la fin, et pourtant la seconde insertion n'a pas été faite au final, puisqu'elle a été annulée grâce au jalon. Seuls Popi et Mimi existent.


```sql
SELECT id, sexe, date_naissance, nom, espece_id, mere_id, pere_id
FROM Animal
WHERE espece_id = 5;
```




+--+----+-------------------+----+---------+-------+-------+
|id|sexe|date_naissance     |nom |espece_id|mere_id|pere_id|
+==+====+===================+====+=========+=======+=======+
|69|F   |2012-02-13 15:45:00|Baba|5        |NULL   |NULL   |
+--+----+-------------------+----+---------+-------+-------+
|70|M   |2012-02-13 15:48:00|Bibo|5        |72     |NULL   |
+--+----+-------------------+----+---------+-------+-------+
|72|F   |2008-02-01 02:25:00|Momy|5        |NULL   |NULL   |
+--+----+-------------------+----+---------+-------+-------+
|73|M   |2007-03-11 12:45:00|Popi|5        |NULL   |NULL   |
+--+----+-------------------+----+---------+-------+-------+
|75|F   |2007-03-12 22:03:00|Mimi|5        |NULL   |NULL   |
+--+----+-------------------+----+---------+-------+-------+