Les verrous de table sont **les seuls supportés par MyISAM**. Ils sont d'ailleurs principalement utilisés pour pallier en partie l'absence de transactions dans MyISAM.
Les tables InnoDB peuvent également utiliser ce type de verrou.

Pour verrouiller une table, il faut utiliser la commande `LOCK TABLES` :


```sql
LOCK TABLES nom_table [AS alias_table] [READ | WRITE] [, ...];
```




- En utilisant `READ`, un verrou de lecture sera posé ; c'est-à-dire que les autres sessions pourront toujours lire les données des tables verrouillées, mais ne pourront plus les modifier.
- En utilisant `WRITE`, un verrou d'écriture sera posé. Les autres sessions ne pourront plus ni lire ni modifier les données des tables verrouillées.


Pour déverrouiller les tables, on utilise `UNLOCK TABLES`. Cela déverrouille toutes les tables verrouillées. Il n'est pas possible de préciser les tables à déverrouiller. Tous les verrous de table d'une session sont relâchés en même temps.


## Session ayant obtenu le verrou


Lorsqu'une session acquiert un ou plusieurs verrous de table, cela a plusieurs conséquences pour cette session :


- elle ne peut plus accéder **qu'aux tables sur lesquelles elle a posé un verrou** ;
- elle ne peut accéder à ces tables qu'**en utilisant les noms qu'elle a donnés** lors du verrouillage (soit le nom de la table, soit le/les alias donné(s)) ;
- s'il s'agit d'un **verrou de lecture** (`READ`), elle peut **uniquement lire les données**, pas les modifier.

**Exemples** : on pose deux verrous, l'un `READ`, l'autre `WRITE`, l'un en donnant un alias au nom de la table, l'autre sans.

```sql
LOCK TABLES Espece READ,              -- On pose un verrou de lecture sur Espece
            Adoption AS adopt WRITE;  -- et un verrou d'écriture sur Adoption avec l'alias adopt
```


Voyons maintenant le résultat de ces différentes requêtes.

**1.** Sélection dans _Espece_, sans alias.


```sql
SELECT id, nom_courant FROM Espece;
```




+--+-----------------+
|id|nom_courant      |
+==+=================+
|1 |Chien            |
+--+-----------------+
|2 |Chat             |
+--+-----------------+
|3 |Tortue d'Hermann |
+--+-----------------+
|4 |Perroquet amazone|
+--+-----------------+
|5 |Rat brun         |
+--+-----------------+




Pas de problème, on a bien un verrou sur _Espece_, sans alias.

**2.** Sélection dans _Espece_, avec alias.


```sql
SELECT id, nom_courant 
FROM Espece AS table_espece;
```


```console
ERROR 1100 (HY000): Table 'table_espece' was not locked with LOCK TABLES
```

Par contre, si l'on essaye d'utiliser un alias, cela ne fonctionne pas. Le verrou est posé sur _Espece_, pas sur _Espece_ `AS` _table_espece_.

**3.** Modification dans _Espece_, sans alias.


```sql
UPDATE Espece
SET description = 'Petit piaf bruyant' 
WHERE id = 4;
```


```console
ERROR 1099 (HY000): Table 'Espece' was locked with a READ lock and can't be updated
```

Avec ou sans alias, impossible de modifier la table _Espece_, puisque le verrou que l'on possède dessus est un verrou de lecture.

**4.** Sélection dans _Adoption_, sans alias.


```sql
SELECT client_id, animal_id 
FROM Adoption;
```


```console
ERROR 1100 (HY000): Table 'Adoption' was not locked with LOCK TABLES
```

Cette fois, c'est le contraire, sans alias, ça ne passe pas.

**5.** Sélection dans _Adoption_, avec alias.


```sql
SELECT client_id, animal_id 
FROM Adoption AS adopt
WHERE client_id = 4;
```




+---------+---------+
|client_id|animal_id|
+=========+=========+
|4        |26       |
+---------+---------+
|4        |41       |
+---------+---------+





**6.** Modification dans _Adoption_, sans alias.


```sql
UPDATE Adoption
SET paye = 0 
WHERE client_id = 10 AND animal_id = 49;
```


```console
ERROR 1100 (HY000): Table 'Adoption' was not locked with LOCK TABLES
```

Idem pour la modification, l'alias est indispensable.

**7.** Modification dans _Adoption_, avec alias.


```sql
UPDATE Adoption AS adopt
SET paye = 0
WHERE client_id = 10 AND animal_id = 49;
```


```console
Query OK, 1 row affected (0.03 sec)
```


Il faut donc penser à acquérir tous les verrous nécessaires aux requêtes à exécuter. De plus, il faut les obtenir **en une seule requête** `LOCK TABLES`. En effet, `LOCK TABLES` commence par enlever tous les verrous de table de la session avant d'en acquérir de nouveaux.
Il est bien entendu possible de poser plusieurs verrous sur la même table en une seule requête afin de verrouiller son nom ainsi qu'un ou plusieurs alias.

**Exemples** : on pose un verrou de lecture sur _Adoption_, puis avec une seconde requête, on pose deux verrous de lecture sur la table _Espece_, l'un avec alias, l'autre sans.


```sql
UNLOCK TABLES; -- On relâche d'abord les deux verrous précédents

LOCK TABLES Adoption READ;
LOCK TABLES Espece READ, Espece AS table_espece READ;
```


Une fois ces deux requêtes effectuées, nous aurons donc bien deux verrous de lecture sur la table _Espece_ : un avec son nom, l'autre avec un alias. Par contre, le verrou sur _Adoption_ n'existera plus puisqu'il aura été relâché par l'exécution de la seconde requête `LOCK TABLES`.

**1.** Sélection dans _Espece_, sans alias.


```sql
SELECT id, nom_courant FROM Espece;
```




+--+-----------------+
|id|nom_courant      |
+==+=================+
|1 |Chien            |
+--+-----------------+
|2 |Chat             |
+--+-----------------+
|3 |Tortue d'Hermann |
+--+-----------------+
|4 |Perroquet amazone|
+--+-----------------+
|5 |Rat brun         |
+--+-----------------+





**2.** Sélection dans _Espece_, avec alias.


```sql
SELECT id, nom_courant FROM Espece AS table_espece;
```




+--+-----------------+
|id|nom_courant      |
+==+=================+
|1 |Chien            |
+--+-----------------+
|2 |Chat             |
+--+-----------------+
|3 |Tortue d'Hermann |
+--+-----------------+
|4 |Perroquet amazone|
+--+-----------------+
|5 |Rat brun         |
+--+-----------------+





Avec ou sans alias, on peut sélectionner les données de la table _Espece_, puisque l'on a un verrou sur _Espece_ et sur _Espece_ `AS` _table_espece_.

**3.** Sélection dans _Espece_, avec mauvais alias.


```sql
SELECT id, nom_courant FROM Espece AS table_esp;
```


```console
ERROR 1100 (HY000): Table 'table_esp' was not locked with LOCK TABLES
```


Bien entendu, cela ne fonctionne que pour l'alias que l'on a donné lors du verrouillage.

**4.** Sélection dans _Adoption_, sans alias.


```sql
SELECT * FROM Adoption;
```


```console
ERROR 1100 (HY000): Table 'Adoption' was not locked with LOCK TABLES
```


Le verrou sur _Adoption_ a été relâché lorsque l'on a posé les verrous sur _Espece_. On ne peut donc pas lire les données d'_Adoption_ (avec ou sans alias).


## Conséquences pour les autres sessions

Si une session a obtenu un **verrou de lecture** sur une table, les autres sessions :


- peuvent lire les données de la table ;
- peuvent également acquérir un verrou de lecture sur cette table ;
- ne peuvent pas modifier les données, ni acquérir un verrou d'écriture sur cette table.


Si par contre une session a obtenu un **verrou d'écriture**, les autres sessions ne peuvent absolument pas accéder à cette table tant que ce verrou existe.

**Exemples :** ouvrez un deuxième client MySQL et connectez-vous à votre base de données, afin d'avoir deux sessions ouvertes.

**1.** Sélection sur des tables verrouillées à partir d'une autre session.

Session 1 :

```sql
LOCK TABLES Client READ,        -- Verrou de lecture sur Client 
            Adoption WRITE;     -- Verrou d'écriture sur Adoption
```


Session 2 :

```sql
SELECT id, nom, prenom, ville, email 
FROM Client 
WHERE ville = 'Houtsiplou';
```




+--+-----------+--------+----------+----------------------+
|id|nom        |prenom  |ville     |email                 |
+==+===========+========+==========+======================+
|1 |Dupont     |Jean    |Houtsiplou|jean.dupont@email.com |
+--+-----------+--------+----------+----------------------+
|12|Broussaille|Virginie|Houtsiplou|vibrousaille@email.com|
+--+-----------+--------+----------+----------------------+





La sélection sur _Client_ se fait sans problème.

Session 2 :

```sql
SELECT * 
FROM Adoption 
WHERE client_id = 4;
```

Par contre, la sélection sur _Adoption_ ne passe pas. La session se bloque, jusqu'à ce que la session 1 déverrouille les tables avec `UNLOCK TABLES`.

**2.** Modification sur des tables verrouillées à partir d'une autre session.

Reverrouillez les tables avec la session 1 :

```sql
LOCK TABLES Client READ,       -- Verrou de lecture sur Client 
            Adoption WRITE;    -- Verrou d'écriture sur Adoption
```


Session 2 :

```sql
UPDATE Client
SET pays = 'Suisse'
WHERE id = 5;
```

La modification sur _Client_, contrairement à la sélection, est bloquée jusqu'au déverrouillage. Déverrouillez puis verrouillez à nouveau avec la session 1.

Session 2:

```sql
UPDATE Adoption
SET paye = 1
WHERE client_id = 3;
```

Bien entendu, la modification sur la table _Adoption_ attend également que les verrous soient relâchés par la session 1.

En ce qui concerne la pose de verrous de table par les autres sessions, faites vos propres tests, mais simplement : si une session peut lire les données d'une table, elle peut également poser un verrou de lecture. Si une session peut modifier les données d'une table, elle peut également poser un verrou d'écriture.


## Interaction avec les transactions

Si l'on utilise des tables MyISAM, il n'y a évidemment aucune précaution particulière à prendre par rapport aux transactions lorsqu'on utilise des verrous de table (les tables MyISAM étant non-transactionnelles). Par contre, si on utilise des tables InnoDB, il convient d'être prudent. En effet :


- `START TRANSACTION` ôte les verrous de table ;
- les commandes `LOCK TABLES` et `UNLOCK TABLES` provoquent une validation implicite si elles sont exécutées à l'intérieur d'une transaction.


Pour utiliser à la fois les transactions et les verrous de table, il faut renoncer à démarrer explicitement les transactions, et donc utiliser le mode non-autocommit. Lorsque l'on est dans ce mode, il est facile de contourner la validation implicite provoquée par `LOCK TABLES` et `UNLOCK TABLES` : il suffit d'appeler `LOCK TABLES` avant toute modification de données, et de commiter/annuler les modifications avant d'exécuter `UNLOCK TABLES`.

**Exemple** :

```sql
SET autocommit = 0;
LOCK TABLES Adoption WRITE; -- La validation implicite ne commite rien puisque aucun changement n'a été fait

UPDATE Adoption SET date_adoption = NOW() WHERE client_id = 9 AND animal_id = 54;
SELECT client_id, animal_id, date_adoption FROM Adoption WHERE client_id = 9;

ROLLBACK;
UNLOCK TABLES; -- On a annulé les changements juste avant donc la validation implicite n'a aucune conséquence
SELECT client_id, animal_id, date_adoption FROM Adoption WHERE client_id = 9;
SET autocommit = 1;
```