# Principe


Une requête préparée, c'est en fait un **modèle de requête** que l'on enregistre et auquel on donne un nom. On va ensuite pouvoir l'exécuter en l'appelant grâce à son nom, et en lui passant éventuellement un ou plusieurs paramètres.

Par exemple, si vous avez régulièrement besoin d'aller chercher des informations sur vos clients à partir de leur adresse email dans une session, plutôt que de faire :

```sql
SELECT * FROM Client WHERE email = 'truc@email.com';
SELECT * FROM Client WHERE email = 'machin@email.com';
SELECT * FROM Client WHERE email = 'bazar@email.com';
SELECT * FROM Client WHERE email = 'brol@email.com';
```

Vous pouvez préparer une requête modèle : 

```sql
SELECT * FROM Client WHERE email = ?
```

Où le ||?|| représente un paramètre. Ensuite, il suffit de l'appeler par son nom en lui passant `'truc@email.com'`, ou `'machin@email.com'`, selon les besoins du moment.


[[information]]
| Bien entendu, on parle d'un cas où l'on n'a qu'une seule adresse email à la fois. Typiquement, le cas où l'on s'occupe d'un client à la fois. Sinon, bien entendu, une simple clause `email IN ('truc@email.com', 'machin@email.com', 'bazar@email.com', 'brol@email.com')` suffirait.



## Portée

Tout comme les variables utilisateur, **une requête préparée n'existe que pour la session qui la crée**.


# Syntaxe

Voyons comment faire tout ça !


## Préparation d'une requête

Pour préparer une requête, il faut renseigner deux éléments :


- le nom qu'on donne à la requête préparée ;
- la requête elle-même, avec un ou plusieurs paramètres (représentés par un ||?||).


Voici la syntaxe à utiliser :

```sql
PREPARE nom_requete
FROM 'requete_preparable';
```


**Exemples**

```sql
-- Sans paramètre
PREPARE select_race
FROM 'SELECT * FROM Race';

-- Avec un paramètre
PREPARE select_client
FROM 'SELECT * FROM Client WHERE email = ?';

-- Avec deux paramètres
PREPARE select_adoption
FROM 'SELECT * FROM Adoption WHERE client_id = ? AND animal_id = ?';
```


Plusieurs choses importantes :


- Le nom de la requête préparée ne doit pas être entre guillemets. Par contre la requête à préparer, si. **La requête à préparer doit être passée comme une chaîne de caractères**.
- Que le paramètre soit un nombre (client_id = ?) ou une chaîne de caractères (email = ?), cela ne change rien. **On ne met pas de guillemets autour du ||?||** à l'intérieur de la requête à préparer.
- La chaîne de caractères contenant la requête à préparer ne peut contenir qu'**une seule requête** (et non plusieurs séparées par un ||;||).
- **Les paramètres ne peuvent représenter que des données, des valeurs**, pas des noms de tables ou de colonnes, ni des morceaux de commandes SQL.


Comme la requête à préparer est donnée sous forme de chaîne de caractères, il est également possible d'utiliser une variable utilisateur, dans laquelle on enregistre tout ou partie de la requête à préparer.

**Exemples**

```sql
SET @req = 'SELECT * FROM Race';
PREPARE select_race
FROM @req;

SET @colonne = 'nom';
SET @req_animal = CONCAT('SELECT ', @colonne, ' FROM Animal WHERE id = ?');
PREPARE select_col_animal
FROM @req_animal;
```


Par contre, il n'est pas possible de mettre directement la fonction `CONCAT()` dans la clause `FROM`.


[[information]]
| Si vous donnez à une requête préparée le nom d'une requête préparée déjà existante, cette dernière sera supprimée et remplacée par la nouvelle.


## Exécution d'une requête préparée

Pour exécuter une requête préparée, on utilise la commande suivante :

```sql
EXECUTE nom_requete [USING @parametre1, @parametre2, ...];
```

Si la requête préparée contient un ou plusieurs paramètres, on doit leur donner une valeur avec la clause `USING`, **en utilisant une variable utilisateur**. Il n'est pas possible de donner directement une valeur. Par ailleurs, il faut donner exactement autant de variables utilisateur qu'il y a de paramètres dans la requête.

**Exemples**

```sql
EXECUTE select_race;

SET @id = 3;
EXECUTE select_col_animal USING @id;

SET @client = 2;
EXECUTE select_adoption USING @client, @id;

SET @email = 'jean.dupont@email.com';
EXECUTE select_client USING @email;

SET @email = 'marie.boudur@email.com';
EXECUTE select_client USING @email;

SET @email = 'fleurtrachon@email.com';
EXECUTE select_client USING @email;

SET @email = 'jeanvp@email.com';
EXECUTE select_client USING @email;

SET @email = 'johanetpirlouit@email.com';
EXECUTE select_client USING @email;
```



## Suppression d'une requête préparée

Pour supprimer une requête préparée, on utilise `DEALLOCATE PREPARE`, suivi du nom de la requête préparée.

**Exemple**

```sql
DEALLOCATE PREPARE select_race;
```