# Usage

La syntaxe que nous venons de voir, avec `PREPARE`, `EXECUTE` et `DEALLOCATE` est en fait très rarement utilisée. 
En effet, vous le savez, MySQL est rarement utilisé seul. La plupart du temps, il est utilisé en conjonction avec un langage de programmation, comme Java, PHP, python, etc. Celui-ci permet de gérer un programme, un site web,… , et crée des requêtes SQL permettant de gérer la base de données de l'application.
Or, il existe des API (interfaces de programmation) pour plusieurs langages, qui permettent de faire des requêtes préparées sans exécuter vous-mêmes les commandes SQL `PREPARE`, `EXECUTE` et `DEALLOCATE`. Exemples : l'API C MySQL pour le langage C, MySQL Connector/J pour le Java, ou MySQL Connector/Net pour le .Net.

Voici un exemple de code C utilisant l'API MySQL pour préparer et exécuter une requête d'insertion :


```c
MYSQL_STMT	*req_prep;
MYSQL_BIND	param[3];

int		client = 2;
int		animal = 56;
MYSQL_TIME	date_reserv;

char		*req_texte = "INSERT INTO Adoption (client_id, animal_id, date_reservation) VALUES (?, ?, ?)";

// On prépare la requête
if (mysql_stmt_prepare(req_prep, req_texte, strlen(req_texte)) != 0)
{
	printf("Impossible de préparer la requête");
	exit(0);
}	

// On initialise un tableau (param, qui contiendra les paramètres) à 0
memset((void*) param, 0, sizeof(param));

// On définit le premier paramètre (pour client_id)
param[0].buffer_type = MYSQL_TYPE_INT;
param[0].buffer = (void*) &client;
param[0].is_unsigned = 0;
param[0].is_null = 0;

// On définit le deuxième paramètre (pour animal_id)
param[1].buffer_type = MYSQL_TYPE_INT;
param[1].buffer = (void*) &animal;
param[1].is_unsigned = 0;
param[1].is_null = 0;

// On définit le troisième paramètre (pour date_reservation)
param[2].buffer_type = MYSQL_TYPE_DATE;
param[2].buffer = (void*) &date_reserv;
param[2].is_null = 0;

// On lie les paramètres
if (mysql_stmt_bind_param(req_prep, param) != 0)
{
	printf("Impossible de lier les paramètres à la requête");
	exit(0);
}

// On définit la date
date_reserv.year = 2012;
date_reserv.month = 3;
date_reserv.day = 20;

// On exécute la requête
if (mysql_stmt_execute(req_prep) != 0)
{
	printf("Impossible d'exécuter la requête");
	exit(0);
}
```


Pour d'autres langages, des extensions ont été créées, qui sont en général elles-mêmes basées sur l'API C MySQL, comme par exemple PDO pour le PHP.

Exemple de code PHP utilisant l'extension PDO pour préparer et exécuter une requête de sélection :

```php
<?php
try
{
	$email = 'jean.dupont@email.com';
	
	// On se connecte
	$bdd = new PDO('mysql:host=localhost;dbname=elevage', 'sdz', '', array(PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION ));
	
	// On prépare la requête
	$requete = $bdd->prepare("SELECT * FROM Client WHERE email = :email");

	// On lie la variable $email définie au-dessus au paramètre :email de la requête préparée
	$requete->bindValue(':email', $email, PDO::PARAM_STR);

	//On exécute la requête
	$requete->execute();
	
	// On récupère le résultat
	if ($requete->fetch())
	{
		echo 'Le client existe !';
	}
} catch (Exception $e)
{
	die('Erreur : ' . $e->getMessage());
}
```



[[question]]
| Pourquoi nous avoir fait apprendre la syntaxe SQL si on ne s'en sert jamais ?

D'abord parce qu'il est toujours intéressant de savoir comment fonctionnent les requêtes préparées. 
Ensuite, parce qu'il pourrait arriver qu'il n'existe aucune API ni extension permettant de faire des requêtes préparées pour le langage dans lequel vous programmez, auquel cas, bien entendu, il faudrait construire vos requêtes préparées vous-mêmes. 
Ou vous pourriez tomber sur l'un des rares cas où il vous serait nécessaire de préparer une requête directement en SQL.
Enfin, vous aurez peut-être besoin de faire quelques tests impliquant des requêtes préparées directement dans MySQL.

Cependant, si une API ou une extension existe et répond à vos besoins, utilisez-la ! Elle sera généralement plus performante et plus sécurisée que ce que vous pourriez faire vous-mêmes.


# Utilité

Les requêtes préparées sont principalement utilisées pour deux raisons :


- protéger son application des injections SQL ;
- gagner en performance dans le cas d'une requête exécutée plusieurs fois par la même session.



## Empêcher les injections SQL

En général, quand on crée une application, l'utilisateur peut interagir avec celle-ci. L'utilisateur peut créer un membre sur un site web communautaire, un personnage sur un jeu vidéo, etc. Les actions de l'utilisateur vont donc avoir une incidence sur la base de données de l'application. Il va envoyer certaines informations, qui vont être traitées, puis une partie va être envoyée sous forme de requêtes à la base de données. 
Il existe un adage bien connu en programmation : "_Never trust user input_" traduit en français par "Ne jamais faire confiance aux données fournies par l'utilisateur".

Lorsque l'on traite des données qui viennent de l'extérieur, il est absolument impératif de toujours vérifier celles-ci, et de protéger les requêtes construites à partir de ces données. Ceci évite que l'utilisateur, volontairement ou non, fasse ce qu'on appelle une injection SQL et provoque un comportement inattendu et souvent indésirable, voire dangereux, pour les données.


[[attention]]
| Les injections SQL sont **un** type de failles exploitables par l'utilisateur. Il existe de nombreux autres types de failles.


Passons maintenant à la question qui doit vous brûler les lèvres. 


[[question]]
| Mais qu'est-ce qu'une injection SQL ?

On appelle injection SQL le fait qu'un utilisateur fournisse des données contenant des mots-clés SQL ou des caractères particuliers qui vont **détourner ou modifier le comportement des requêtes construites** sur la base de ces données.

Imaginons que vous créiez un site web avec un espace membre. 
Vos membres ont accès à une page "profil" grâce à laquelle ils peuvent gérer leurs informations, ou supprimer leur compte. Pour supprimer leur compte, ils ont simplement à appuyer sur un bouton qui envoie leur numéro d'id.

D'un côté on a donc une requête incomplète :

```sql
DELETE FROM Membre WHERE id =
```

De l'autre, l'id du client, par exemple _4_.

Avec votre langage de programmation web préféré, vous mettez les deux ensemble pour créer la requête complète :

```sql
DELETE FROM Membre WHERE id = 4;
```


Et maintenant, un méchant pirate s'amuse à modifier la donnée envoyée par le bouton "Supprimer compte" (oui oui, c'est faisable, et même plutôt facile).
À la suite du numéro d'_id_, il ajoute `OR 1 = 1`. Après construction de la requête, on obtient donc ceci :


```sql
DELETE FROM Membre WHERE id = 4 OR 1 = 1;
```

Et la seconde condition étant toujours remplie, c'est l'horreur : toute votre table _Membre_ est effacée !
Et voilà ! Vous avez été victimes d'une injection SQL.


[[question]]
| Comment une requête préparée peut-elle éviter les injections SQL ?

En utilisant les requêtes préparées, lorsque vous liez une valeur à un paramètre de la requête préparée grâce à la fonction correspondante dans le langage que vous utilisez, le type du paramètre attendu est également donné, explicitement ou implicitement. La plupart du temps, soit une erreur sera générée si la donnée de l'utilisateur ne correspond pas au type attendu, soit la donnée de l'utilisateur sera rendue inoffensive (par l'ajout de guillemets qui en feront une simple chaîne de caractères par exemple). 
Par ailleurs, lorsque MySQL injecte les valeurs dans la requête, les mots-clés SQL qui s'y trouveraient pour une raison où une autre ne seront pas interprétés (puisque les paramètres n'acceptent que des valeurs, et pas des morceaux de requêtes).


## Gagner en performance

Lorsque l'on exécute une simple requête (sans la préparer), voici les étapes qui sont effectuées :


1. La requête est envoyée vers le serveur.
1. La requête est compilée par le serveur (traduite du langage SQL compréhensible pour nous, pauvres humains limités, vers un "langage machine" dont se sert le serveur).
1. Le serveur crée un plan d'exécution de la requête (quelles tables utiliser ? quels index ? dans quel ordre ?…).
1. Le serveur exécute la requête.
1. Le résultat de la requête est renvoyé au client.


Voici maintenant les étapes lorsqu'il s'agit d'une requête préparée :

**Préparation** de la requête :


1. La requête est envoyée vers le serveur, avec un identifiant.
1. La requête est compilée par le serveur.
1. Le serveur crée un plan d'exécution de la requête.
1. La requête compilée et son plan d'exécution sont stockés en mémoire par le serveur.
1. Le serveur envoie vers le client une confirmation que la requête est prête (en se servant de l’identifiant de la requête).


**Exécution** de la requête :


1. L'identifiant de la requête à exécuter, ainsi que les paramètres à utiliser sont envoyés vers le serveur.
1. Le serveur exécute la requête demandée.
1. Le résultat de la requête est renvoyé au client.


On a donc plus d'étapes pour une requête préparée, que pour une requête non préparée (8 contre 5). Du moins, lorsque l'on exécute une seule fois la requête. Car si l'on exécute plusieurs fois une requête, la tendance s'inverse rapidement : dans le cas d'une requête non préparée, **toutes les étapes doivent être répétées à chaque fois** (sachant que la création du plan d'exécution est l'étape la plus longue), alors que pour une requête préparée, **seule les étapes d'exécution seront répétées**.

Il est donc intéressant, d'un point de vue des performances, de préparer une requête lorsqu'elle va **être exécutée plusieurs fois pendant la même session** (je vous rappelle que les requêtes préparées sont supprimées à la fin de la session).


[[information]]
| Le fait que la requête soit compilée lors de sa préparation et que le plan d'exécution soit calculé également lors de la préparation et non de l'exécution, explique pourquoi les paramètres peuvent uniquement être des valeurs, et non des parties de requêtes comme des noms de tables ou des mots-clés. En effet, il est impossible de créer le plan si l'on ne sait pas encore sur quelles tables on travaillera, ni quelles colonnes (et donc quels index) seront utilisées.

*[API]: Application Programming Interface