Voyons tout de suite la syntaxe à utiliser pour créer une procédure :


```sql
CREATE PROCEDURE nom_procedure ([parametre1 [, parametre2, ...]])
corps de la procédure;
```


Décodons tout ceci.



- `CREATE PROCEDURE` : sans surprise, il s'agit de la commande à exécuter pour créer une procédure. On fait suivre cette commande du nom que l'on veut donner à la nouvelle procédure.
- `([parametre1 [, parametre2, ...]])` : après le nom de la procédure viennent des parenthèses. **Celles-ci sont obligatoires** ! À l'intérieur de ces parenthèses, on définit les éventuels paramètres de la procédure. Ces paramètres sont des variables qui pourront être utilisées par la procédure.
- corps de la procédure :  c'est là que l'on met le **contenu** de la procédure, ce qui va être exécuté lorsqu'on lance la procédure. Cela peut être soit **une seule requête**, soit **un bloc d'instructions**.



[[information]]
| Les noms des procédures stockées ne sont pas sensibles à la casse.



# Procédure avec une seule requête

Voici une procédure toute simple, sans paramètres, qui va juste afficher toutes les races d'animaux.


```sql
CREATE PROCEDURE afficher_races_requete() -- pas de paramètres dans les parenthèses
SELECT id, nom, espece_id, prix FROM Race;
```



# Procédure avec un bloc d'instructions

Pour délimiter un bloc d'instructions (qui peut donc contenir plus d'une instruction), on utilise les mots `BEGIN` et `END`.


```sql
BEGIN
    -- Série d'instructions
END;
```


**Exemple** : reprenons la procédure précédente, mais en utilisant un bloc d'instructions.


```sql
CREATE PROCEDURE afficher_races_bloc() -- pas de paramètres dans les parenthèses
BEGIN
    SELECT id, nom, espece_id, prix FROM Race;
END;
```


Malheureusement…

```console
ERROR 1064 (42000): You have an error in your SQL syntax; check the manual that corresponds to your MySQL server version for the right syntax to use near '' at line 3
```



[[question]]
| Que s'est-il passé ? La syntaxe semble correcte…

Les mots-clés sont bons, il n'y a pas de paramètres mais on a bien mis les parenthèses, `BEGIN` et `END` sont tous les deux présents. Tout cela est correct, et pourtant, nous avons visiblement omis un détail.

Peut-être aurez-vous compris que le problème se situe au niveau du caractère ||;|| : en effet, un ||;|| termine une instruction SQL. Or, on a mis un ||;|| à la suite de `SELECT * FROM Race;`. Cela semble logique, mais pose problème puisque c'est le premier ||;|| rencontré par l'instruction `CREATE PROCEDURE`, qui naturellement pense devoir s'arrêter là. Ceci déclenche une erreur puisqu'en réalité, l'instruction `CREATE PROCEDURE` n'est pas terminée : le bloc d'instructions n'est pas complet !


[[question]]
| Comment faire pour écrire des instructions à l'intérieur d'une instruction alors ?

Il suffit de changer le délimiteur !


# Délimiteur


Ce qu'on appelle délimiteur, c'est tout simplement (par défaut), le caractère ||;||. C'est-à-dire le caractère qui permet de **délimiter les instructions**.
Or, il est tout à fait possible de définir le délimiteur manuellement, de manière à ce que ||;|| ne signifie plus qu'une instruction se termine. Auquel cas le caractère ||;|| pourra être utilisé à l'intérieur d'une instruction, et donc pourra être utilisé dans le corps d'une procédure stockée.

Pour changer le délimiteur, il suffit d'utiliser cette commande :


```sql
DELIMITER |
```


À partir de maintenant, vous devrez utiliser le caractère ||||| pour signaler la fin d'une instruction. ||;|| ne sera plus compris comme tel par votre session.

```sql
SELECT 'test'|
```




+----+
|test|
+====+
|test|
+----+






[[information]]
| `DELIMITER` n'agit que pour la **session courante**.


Vous pouvez utiliser le (ou les) caractère(s) de votre choix comme délimiteur. Bien entendu, il vaut mieux choisir quelque chose qui ne risque pas d'être utilisé dans une instruction. Bannissez donc les lettres, chiffres, ||@|| (qui servent pour les variables utilisateurs) et les ||\|| (qui servent à échapper les caractères spéciaux).

Les deux délimiteurs suivants sont les plus couramment utilisés :


```sql
DELIMITER //
DELIMITER |
```


Bien ! Ceci étant réglé, reprenons !


# Création d'une procédure stockée



```sql
DELIMITER |                           -- On change le délimiteur
CREATE PROCEDURE afficher_races()      -- toujours pas de paramètres, toujours des parenthèses
BEGIN
    SELECT id, nom, espece_id, prix
    FROM Race;                        -- Cette fois, le ; ne nous embêtera pas
END|                                  -- Et on termine bien sûr la commande CREATE PROCEDURE par notre nouveau délimiteur
```


Cette fois-ci, tout se passe bien. La procédure a été créée.


[[information]]
| Lorsqu'on utilisera la procédure, quel que soit le délimiteur défini par `DELIMITER`, les instructions à l'intérieur du corps de la procédure seront bien délimitées par ||;||. En effet, lors de la création d'une procédure, celle-ci est interprétée – on dit aussi "parsée" – par le serveur MySQL et le parseur des procédures stockées interprétera toujours ||;|| comme délimiteur. Il n'est pas influencé par la commande `DELIMITER`.


Les procédures stockées n'étant que très rarement composées d'une seule instruction, on utilise presque toujours un bloc d'instructions pour le corps de la procédure.


# Utilisation d'une procédure stockée

Pour appeler une procédure stockée, c'est-à-dire déclencher l'exécution du bloc d'instructions constituant le corps de la procédure, il faut utiliser le mot-clé `CALL`, suivi du nom de la procédure appelée, puis de parenthèses (avec éventuellement des paramètres).


```sql
CALL afficher_races()|   -- le délimiteur est toujours | !!!
```




+--+-------------------+---------+-------+
|id|nom                |espece_id|prix   |
+==+===================+=========+=======+
|1 |Berger allemand    |1        |485.00 |
+--+-------------------+---------+-------+
|2 |Berger blanc suisse|1        |935.00 |
+--+-------------------+---------+-------+
|3 |Singapura          |2        |985.00 |
+--+-------------------+---------+-------+
|4 |Bleu russe         |2        |835.00 |
+--+-------------------+---------+-------+
|5 |Maine coon         |2        |735.00 |
+--+-------------------+---------+-------+
|7 |Sphynx             |2        |1235.00|
+--+-------------------+---------+-------+
|8 |Nebelung           |2        |985.00 |
+--+-------------------+---------+-------+
|9 |Rottweiller        |1        |600.00 |
+--+-------------------+---------+-------+





Le bloc d'instructions a bien été exécuté (un simple `SELECT` dans ce cas).