Maintenant que l'on sait créer une procédure et l'appeler, intéressons-nous aux paramètres.


# Sens des paramètres

Un paramètre peut être de trois sens différents : entrant (`IN`), sortant (`OUT`), ou les deux (`INOUT`).


- `IN` : c'est un paramètre "entrant". C'est-à-dire qu'il s'agit d'un paramètre dont la valeur est fournie à la procédure stockée. Cette valeur sera utilisée pendant la procédure (pour un calcul ou une sélection par exemple).
- `OUT` : il s'agit d'un paramètre "sortant", dont la valeur va être établie au cours de la procédure et qui pourra ensuite être utilisé en dehors de cette procédure.
- `INOUT` : un tel paramètre sera utilisé pendant la procédure, verra éventuellement sa valeur modifiée par celle-ci, et sera ensuite utilisable en dehors.



# Syntaxe

Lorsque l'on crée une procédure avec un ou plusieurs paramètres, chaque paramètre est défini par trois éléments.


- Son sens : entrant, sortant, ou les deux. Si aucun sens n'est donné, il s'agira d'un paramètre `IN` par défaut.
- Son nom : indispensable pour le désigner à l'intérieur de la procédure.
- Son type : `INT`, `VARCHAR(10)`,…



# Exemples


## Procédure avec un seul paramètre entrant

Voici une procédure qui, selon l'_id_ de l'espèce qu'on lui passe en paramètre, affiche les différentes races existant pour cette espèce.


```sql
DELIMITER |                                                      -- Facultatif si votre délimiteur est toujours |
CREATE PROCEDURE afficher_race_selon_espece (IN p_espece_id INT)  -- Définition du paramètre p_espece_id
BEGIN
    SELECT id, nom, espece_id, prix 
    FROM Race
    WHERE espece_id = p_espece_id;                               -- Utilisation du paramètre
END |
DELIMITER ;                                                      -- On remet le délimiteur par défaut
```



[[information]]
| Notez que, suite à la création de la procédure, j'ai remis le délimiteur par défaut ||;||. Ce n'est absolument pas obligatoire, vous pouvez continuer à travailler avec ||||| si vous préférez.


Pour l'utiliser, il faut donc passer une valeur en paramètre de la procédure. Soit directement, soit par l'intermédiaire d'une variable utilisateur.


```sql
CALL afficher_race_selon_espece(1);
SET @espece_id := 2;
CALL afficher_race_selon_espece(@espece_id);
```




+--+-------------------+---------+------+
|id|nom                |espece_id|prix  |
+==+===================+=========+======+
|1 |Berger allemand    |1        |485.00|
+--+-------------------+---------+------+
|2 |Berger blanc suisse|1        |935.00|
+--+-------------------+---------+------+
|9 |Rottweiller        |1        |600.00|
+--+-------------------+---------+------+







+--+----------+---------+-------+
|id|nom       |espece_id|prix   |
+==+==========+=========+=======+
|3 |Singapura |2        |985.00 |
+--+----------+---------+-------+
|4 |Bleu russe|2        |835.00 |
+--+----------+---------+-------+
|5 |Maine coon|2        |735.00 |
+--+----------+---------+-------+
|7 |Sphynx    |2        |1235.00|
+--+----------+---------+-------+
|8 |Nebelung  |2        |985.00 |
+--+----------+---------+-------+





Le premier appel à la procédure affiche bien toutes les races de chiens, et le second, toutes les races de chats.


[[attention]]
| J'ai fait commencer le nom du paramètre par "p_". Ce n'est pas obligatoire, mais je vous conseille de le faire systématiquement pour vos paramètres afin de les distinguer facilement. Si vous ne le faites pas, soyez extrêmement prudents avec les noms que vous leur donnez. Par exemple, dans cette procédure, si on avait nommé le paramètre _espece_id_, cela aurait posé problème, puisque _espece_id_ est aussi le nom d'une colonne dans la table _Race_. Qui plus est, c'est le nom de la colonne dont on se sert dans la condition `WHERE`. En cas d’ambiguïté, MySQL interprète l'élément comme étant le paramètre, et non la colonne. On aurait donc eu `WHERE 1 = 1` par exemple, ce qui est toujours vrai.



## Procédure avec deux paramètres, un entrant et un sortant

Voici une procédure assez similaire à la précédente, si ce n'est qu'elle n'affiche pas les races existant pour une espèce, mais compte combien il y en a, puis stocke cette valeur dans un paramètre sortant.


```sql
DELIMITER |                                                      
CREATE PROCEDURE compter_races_selon_espece (p_espece_id INT, OUT p_nb_races INT)  
BEGIN
    SELECT COUNT(*) INTO p_nb_races 
    FROM Race
    WHERE espece_id = p_espece_id;                               
END |
DELIMITER ;
```


Aucun sens n'a été précisé pour _p_espece_id_, il est donc considéré comme un paramètre entrant.

`SELECT COUNT(*) INTO p_nb_races`. Voila qui est nouveau ! Comme vous l'avez sans doute deviné, le mot-clé `INTO` placé après la clause `SELECT` permet d'**assigner les valeurs sélectionnées** par ce `SELECT` à des variables, au lieu de simplement afficher les valeurs sélectionnées.
Dans le cas présent, la valeur du `COUNT(*)` est assignée à _p_nb_races_.

Pour pouvoir l'utiliser, il est nécessaire que le `SELECT` ne renvoie qu'**une seule ligne**, et il faut que le nombre de valeurs sélectionnées et le nombre de variables à assigner **soient égaux** :

**Exemple 1** : `SELECT ... INTO` correct avec deux valeurs


```sql
SELECT id, nom INTO @var1, @var2
FROM Animal 
WHERE id = 7;
SELECT @var1, @var2;
```




+-----+--------+
|@var1|@var2   |
+=====+========+
|7    |Caroline|
+-----+--------+





Le `SELECT ... INTO` n'a rien affiché, mais a assigné la valeur `7` à _@var1_, et la valeur `'Caroline'` à _@var2_, que nous avons ensuite affichées avec un autre `SELECT`.

**Exemple 2** : `SELECT ... INTO` incorrect, car le nombre de valeurs sélectionnées (deux) n'est pas le même que le nombre de variables à assigner (une).


```sql
SELECT id, nom INTO @var1
FROM Animal 
WHERE id = 7;
```


```console
ERROR 1222 (21000): The used SELECT statements have a different number of columns
```


**Exemple 3** : `SELECT ... INTO` incorrect, car il y a plusieurs lignes de résultats.


```sql
SELECT id, nom INTO @var1, @var2
FROM Animal 
WHERE espece_id = 5;
```


```console
ERROR 1172 (42000): Result consisted of more than one row
```


Revenons maintenant à notre nouvelle procédure _compter_races_selon_espece()_ et exécutons-la. Pour cela, il va falloir lui passer deux paramètres : _p_espece_id_ et _p_nb_races_. 
Le premier ne pose pas de problème, il faut simplement donner un nombre, soit directement soit par l'intermédiaire d'une variable, comme pour la procédure _afficher_race_selon_espece()_. 
Par contre, pour le second, il s'agit d'un paramètre sortant. Il ne faut donc pas donner une valeur, mais quelque chose dont la valeur sera déterminée par la procédure (grâce au `SELECT ... INTO`), et qu'on pourra utiliser ensuite : **une variable utilisateur** !


```sql
CALL compter_races_selon_espece (2, @nb_races_chats);
```


Et voilà ! La variable _@nb_races_chats_ contient maintenant le nombre de races de chats. Il suffit de l'afficher pour vérifier.


```sql
SELECT @nb_races_chats;
```




+---------------+
|@nb_races_chats|
+===============+
|5              |
+---------------+






## Procédure avec deux paramètres, un entrant et un entrant-sortant

Nous allons créer une procédure qui va servir à calculer le prix que doit payer un client. Pour cela, deux paramètres sont nécessaires : l'animal acheté (paramètre `IN`), et le prix à payer (paramètre `INOUT`).
La raison pour laquelle le prix est un paramètre à la fois entrant et sortant est qu'on veut pouvoir, avec cette procédure, calculer simplement un prix total dans le cas où un client achèterait plusieurs animaux.
Le principe est simple : si le client n'a encore acheté aucun animal, le prix est de 0. Pour chaque animal acheté, on appelle la procédure, qui ajoute au prix total le prix de l'animal en question.
Une fois n'est pas coutume, commençons par voir les requêtes qui nous serviront à tester la procédure. Cela devrait clarifier le principe. Je vous propose d'essayer ensuite d'écrire vous-mêmes la procédure correspondante avant de regarder à quoi elle ressemble.


```sql
SET @prix = 0;                   -- On initialise @prix à 0

CALL calculer_prix (13, @prix);  -- Achat de Rouquine
SELECT @prix AS prix_intermediaire;

CALL calculer_prix (24, @prix);  -- Achat de Cartouche
SELECT @prix AS prix_intermediaire;

CALL calculer_prix (42, @prix);  -- Achat de Bilba
SELECT @prix AS prix_intermediaire;

CALL calculer_prix (75, @prix);  -- Achat de Mimi
SELECT @prix AS total;
```


On passe donc chaque animal acheté tour à tour à la procédure, qui modifie le prix en conséquence. Voici quelques indices et rappels qui devraient vous aider à écrire vous-mêmes la procédure.


- Le prix n'est pas un nombre entier.
- Il est possible de faire des additions directement dans un `SELECT`.
- Pour déterminer le prix, il faut utiliser la fonction `COALESCE()`.


Réponse :


[[secret]]
| ```sql
| DELIMITER |
| 
| CREATE PROCEDURE calculer_prix (IN p_animal_id INT, INOUT p_prix DECIMAL(7,2))
| BEGIN
|     SELECT p_prix + COALESCE(Race.prix, Espece.prix) INTO p_prix
|     FROM Animal
|     INNER JOIN Espece ON Espece.id = Animal.espece_id
|     LEFT JOIN Race ON Race.id = Animal.race_id
|     WHERE Animal.id = p_animal_id;
| END |
| 
| DELIMITER ;
| ```


Et voici ce qu'affichera le code de test :




+------------------+
|prix_intermediaire|
+==================+
|485.00            |
+------------------+







+------------------+
|prix_intermediaire|
+==================+
|685.00            |
+------------------+







+------------------+
|prix_intermediaire|
+==================+
|1420.00           |
+------------------+







+-------+
|total  |
+=======+
|1430.00|
+-------+





Voilà qui devrait nous simplifier la vie. Et nous n'en sommes qu'au début des possibilités des procédures stockées !