# Blocs d'instructions

Nous avons vu qu'un bloc d'instructions était défini par les mots-clés `BEGIN` et `END`, entre lesquels on met les instructions qui composent le bloc (de zéro à autant d'instructions que l'on veut, séparées bien sûr d'un ||;||).

Il est possible d'imbriquer plusieurs blocs d'instructions. De même, à l'intérieur d'un bloc d'instructions, plusieurs blocs d'instructions peuvent se suivre. Ceux-ci permettent donc de **structurer les instructions** en plusieurs parties distinctes et sur plusieurs niveaux d'imbrication différents.


```sql
BEGIN
    SELECT 'Bloc d''instructions principal';
	
    BEGIN
        SELECT 'Bloc d''instructions 2, imbriqué dans le bloc principal';
		
        BEGIN
            SELECT 'Bloc d''instructions 3, imbriqué dans le bloc d''instructions 2';
        END;
    END;
	
    BEGIN
        SELECT 'Bloc d''instructions 4, imbriqué dans le bloc principal';
    END;

END;
```



[[information]]
| Cet exemple montre également l'importance de l'**indentation** pour avoir un code lisible. Ici, toutes les instructions d'un bloc sont au même niveau et décalées vers la droite par rapport à la déclaration du bloc. Cela permet de voir en un coup d’œil où commence et où se termine chaque bloc d'instructions.



# Variables locales

Nous connaissons déjà les variables utilisateur, qui sont des variables désignées par ||@||. J'ai également mentionné l'existence des variables système, qui sont des variables prédéfinies par MySQL.
Voyons maintenant les **variables locales**, qui peuvent être définies dans un bloc d'instructions.


## Déclaration d'une variable locale

La déclaration d'une variable locale se fait avec l'instruction `DECLARE` :


```sql
DECLARE nom_variable type_variable [DEFAULT valeur_defaut];
```


Cette instruction doit se trouver au tout début du bloc d'instructions dans lequel la variable locale sera utilisée (donc directement après le `BEGIN`).

On a donc une structure générale des blocs d'instructions qui se dégage :


```sql
BEGIN
    -- Déclarations (de variables locales par exemple)

    -- Instructions (dont éventuels blocs d'instructions imbriqués)
END;
```



[[information]]
| Tout comme pour les variables utilisateur, le nom des variables locales n'est **pas** sensible à la casse.


Si aucune valeur par défaut n'est précisée, la variable vaudra `NULL` tant que sa valeur n'est pas changée.
Pour changer la valeur d'une variable locale, on peut utiliser `SET` ou `SELECT ... INTO`.

**Exemple** : voici une procédure stockée qui donne la date d'aujourd'hui et de demain :

```sql
DELIMITER |
CREATE PROCEDURE aujourdhui_demain ()
BEGIN
    DECLARE v_date DATE DEFAULT CURRENT_DATE();               -- On déclare une variable locale et on lui met une valeur par défaut

    SELECT DATE_FORMAT(v_date, '%W %e %M %Y') AS Aujourdhui;

    SET v_date = v_date + INTERVAL 1 DAY;                     -- On change la valeur de la variable locale
    SELECT DATE_FORMAT(v_date, '%W %e %M %Y') AS Demain;
END|
DELIMITER ;
```


Testons-la :

```sql
SET lc_time_names = 'fr_FR';
CALL aujourdhui_demain();
```




+----------------+
|Aujourdhui      |
+================+
|mardi 1 mai 2012|
+----------------+







+-------------------+
|Demain             |
+===================+
|mercredi 2 mai 2012|
+-------------------+






[[attention]]
| Tout comme pour les paramètres, les variables locales peuvent poser problème si l'on ne fait pas attention au nom qu'on leur donne. En cas de conflit (avec un nom de colonne par exemple), comme pour les paramètres, le nom sera interprété comme désignant la variable locale en priorité. Par conséquent, toutes mes variables locales seront préfixées par "v_".



## Portée des variables locales dans un bloc d'instruction

Les variables locales n'existent que dans le bloc d'instructions dans lequel elles ont été déclarées. Dès que le mot-clé `END` est atteint, toutes les variables locales du bloc sont détruites.

**Exemple 1** :


```sql
DELIMITER |
CREATE PROCEDURE test_portee1()
BEGIN
    DECLARE v_test1 INT DEFAULT 1;

    BEGIN
        DECLARE v_test2 INT DEFAULT 2;

        SELECT 'Imbriqué' AS Bloc;
        SELECT v_test1, v_test2;
    END;
    SELECT 'Principal' AS Bloc;
    SELECT v_test1, v_test2;
    
END|
DELIMITER ;

CALL test_portee1();
```




+--------+
|Bloc    |
+========+
|Imbriqué|
+--------+







+-------+-------+
|v_test1|v_test2|
+=======+=======+
|1      |2      |
+-------+-------+







+---------+
|Bloc     |
+=========+
|Principal|
+---------+





```console
ERROR 1054 (42S22): Unknown column 'v_test2' in 'field list'
```


La variable locale _v_test2_ existe bien dans le bloc imbriqué, puisque c'est là qu'elle est définie, mais pas dans le bloc principal. _v_test1_ par contre existe dans le bloc principal (où elle est définie), mais aussi dans le bloc imbriqué. 

**Exemple 2** : 

```sql
DELIMITER |
CREATE PROCEDURE test_portee2()
BEGIN
    DECLARE v_test1 INT DEFAULT 1;

    BEGIN
        DECLARE v_test2 INT DEFAULT 2;

        SELECT 'Imbriqué 1' AS Bloc;
        SELECT v_test1, v_test2;
    END;
 
    BEGIN
        SELECT 'imbriqué 2' AS Bloc;
        SELECT v_test1, v_test2;    
    END;
    
    
END|
DELIMITER ;

CALL test_portee2();
```




+----------+
|Bloc      |
+==========+
|Imbriqué 1|
+----------+







+-------+-------+
|v_test1|v_test2|
+=======+=======+
|1      |2      |
+-------+-------+







+----------+
|Bloc      |
+==========+
|imbriqué 2|
+----------+





```console
ERROR 1054 (42S22): Unknown column 'v_test2' in 'field list'
```


À nouveau, _v_test1_, déclarée dans le bloc principal, existe dans les deux blocs imbriqués. Par contre, _v_test2_ n'existe que dans le bloc imbriqué dans lequel elle est déclarée.



[[attention]]
| Attention cependant à la subtilité suivante : si un bloc imbriqué déclare une variable locale ayant le même nom qu'une variable locale déclarée dans un bloc d'un niveau supérieur, il s'agira toujours de deux variables locales différentes, et seule la variable locale déclarée dans le bloc imbriqué sera visible dans ce même bloc.


**Exemple 3** :


```sql
DELIMITER |
CREATE PROCEDURE test_portee3()
BEGIN
    DECLARE v_test INT DEFAULT 1;

    SELECT v_test AS 'Bloc principal';

    BEGIN
        DECLARE v_test INT DEFAULT 0;

        SELECT v_test AS 'Bloc imbriqué';
        SET v_test = 2;
        SELECT v_test AS 'Bloc imbriqué après modification';
    END;

    SELECT v_test AS 'Bloc principal';
END |
DELIMITER ;

CALL test_portee3();
```




+--------------+
|Bloc principal|
+==============+
|1             |
+--------------+







+-------------+
|Bloc imbriqué|
+=============+
|0            |
+-------------+







+--------------------------------+
|Bloc imbriqué après modification|
+================================+
|2                               |
+--------------------------------+







+--------------+
|Bloc principal|
+==============+
|1             |
+--------------+





La variable locale _v_test_ est déclarée dans le bloc principal et dans le bloc imbriqué, avec deux valeurs différentes. Mais lorsqu'on revient dans le bloc principal après exécution du bloc d'instructions imbriqué, _v_test_ a toujours la valeur qu'elle avait avant l'exécution de ce bloc et sa deuxième déclaration. Il s'agit donc bien de **deux variables locales distinctes**.