Les structures conditionnelles permettent de déclencher une action ou une série d'instructions lorsqu'une condition préalable est remplie.

MySQL propose deux structures conditionnelles : `IF` et `CASE`.


# La structure IF

Voici la syntaxe de la structure `IF` :


```sql
IF condition THEN instructions
[ELSEIF autre_condition THEN instructions
[ELSEIF ...]]
[ELSE instructions]
END IF;
```



## Le cas le plus simple : si la condition est vraie, alors on exécute ces instructions

Voici la structure minimale d'un `IF` :


```sql
IF condition THEN
    instructions
END IF;
```

Soit on exécute les instructions (si la condition est vraie), soit on ne les exécute pas.

**Exemple** : la procédure suivante affiche `'J''ai déjà été adopté !'`, si c'est le cas, à partir de l'_id_ d'un animal :


```sql
DELIMITER |
CREATE PROCEDURE est_adopte(IN p_animal_id INT)
BEGIN
    DECLARE v_nb INT DEFAULT 0;           -- On crée une variable locale

    SELECT COUNT(*) INTO v_nb             -- On met le nombre de lignes correspondant à l'animal 
    FROM Adoption                         --  dans Adoption dans notre variable locale
    WHERE animal_id = p_animal_id;

    IF v_nb > 0 THEN                      -- On teste si v_nb est supérieur à 0 (donc si l'animal a été adopté)
        SELECT 'J''ai déjà été adopté !';
    END IF;                               -- Et on n'oublie surtout pas le END IF et le ; final
END |
DELIMITER ;

CALL est_adopte(3);
CALL est_adopte(28);
```


Seul le premier appel à la procédure va afficher `'J''ai déjà été adopté !'`, puisque l'animal 3 est présent dans la table _Adoption_, contrairement à l'animal 28.


## Deuxième cas : si ... alors, sinon ...

Grâce au mot-clé `ELSE`, on peut définir une série d'instructions à exécuter si la condition est fausse.


[[attention]]
| `ELSE` ne doit **pas** être suivi de `THEN`.


**Exemple** : la procédure suivante affiche `'Je suis né avant 2010'` ou `'Je suis né après 2010'`, selon la date de naissance de l'animal transmis en paramètre.


```sql
DELIMITER |
CREATE PROCEDURE avant_apres_2010(IN p_animal_id INT)
BEGIN
    DECLARE v_annee INT;

    SELECT YEAR(date_naissance) INTO v_annee
    FROM Animal
    WHERE id = p_animal_id;

    IF v_annee < 2010 THEN
        SELECT 'Je suis né avant 2010' AS naissance;
    ELSE                                               -- Pas de THEN
        SELECT 'Je suis né après 2010' AS naissance;
    END IF;                                            -- Toujours obligatoire

END |
DELIMITER ;

CALL avant_apres_2010(34);   -- Né le 20/04/2008
CALL avant_apres_2010(69);   -- Né le 13/02/2012
```



## Troisième et dernier cas : plusieurs conditions alternatives

Enfin, le mot-clé `ELSEIF... THEN` permet de vérifier d'autres conditions (en dehors de la condition du `IF`), chacune ayant une série d'instructions définies à exécuter en cas de véracité. Si plusieurs conditions sont vraies en même temps, seule la première rencontrée verra ses instructions exécutées.
On peut bien sûr toujours (mais ce n'est pas obligatoire) ajouter un `ELSE` pour le cas où aucune condition ne serait vérifiée.

**Exemple** : cette procédure affiche un message différent selon le sexe de l'animal passé en paramètre.


```sql
DELIMITER |
CREATE PROCEDURE message_sexe(IN p_animal_id INT)
BEGIN
    DECLARE v_sexe VARCHAR(10);

    SELECT sexe INTO v_sexe
    FROM Animal
    WHERE id = p_animal_id;

    IF (v_sexe = 'F') THEN                   -- Première possibilité
        SELECT 'Je suis une femelle !' AS sexe;
    ELSEIF (v_sexe = 'M') THEN                  -- Deuxième possibilité
        SELECT 'Je suis un mâle !' AS sexe;
    ELSE                                           -- Défaut
        SELECT 'Je suis en plein questionnement existentiel...' AS sexe;
    END IF;
END|
DELIMITER ;

CALL message_sexe(8);   -- Mâle
CALL message_sexe(6);   -- Femelle
CALL message_sexe(9);   -- Ni l'un ni l'autre
```


Il peut bien sûr y avoir autant de `ELSEIF... THEN` que l'on veut (mais un seul `ELSE`).


# La structure CASE

Deux syntaxes sont possibles pour utiliser `CASE`.


## Première syntaxe : conditions d'égalité


```sql
CASE valeur_a_comparer
    WHEN possibilite1 THEN instructions
    [WHEN possibilite2 THEN instructions] ...
    [ELSE instructions]
END CASE;
```


**Exemple** : on reprend la procédure _message_sexe()_, et on l'adapte pour utiliser `CASE`.


```sql
DELIMITER |
CREATE PROCEDURE message_sexe2(IN p_animal_id INT)
BEGIN
    DECLARE v_sexe VARCHAR(10);

    SELECT sexe INTO v_sexe
    FROM Animal
    WHERE id = p_animal_id;

    CASE v_sexe
        WHEN 'F' THEN                          -- Première possibilité
            SELECT 'Je suis une femelle !' AS sexe;
        WHEN 'M' THEN                             -- Deuxième possibilité
            SELECT 'Je suis un mâle !' AS sexe;
        ELSE                                         -- Défaut
            SELECT 'Je suis en plein questionnement existentiel...' AS sexe;
    END CASE;
END|
DELIMITER ;

CALL message_sexe2(8);   -- Mâle
CALL message_sexe2(6);   -- Femelle
CALL message_sexe2(9);   -- Ni l'un ni l'autre
```


On définit donc _v_sexe_ comme point de comparaison. Chaque `WHEN` donne alors un élément auquel _v_sexe_ doit être comparé. Les instructions exécutées seront celles du `WHEN` dont l'élément est égal à _v_sexe_. Le `ELSE` sera exécuté si aucun `WHEN` ne correspond.

Ici, on compare une variable locale (_v_sexe_) à des chaînes de caractères (`'F'` et `'M'`), mais on peut utiliser différents types d'éléments. Voici les principaux :


- des variables locales ;
- des variables utilisateur ;
- des valeurs constantes de tous types (`0`, `'chaîne'`, `5.67`, `'2012-03-23'`,…) ;
- des expressions (`2 + 4`, `NOW()`, `CONCAT(nom, ' ', prenom)`,…) ;
- …



[[attention]]
| Cette syntaxe ne permet pas de faire des comparaisons avec `NULL`, puisqu'elle utilise une comparaison de type `valeur1 = valeur2`. Or cette comparaison est inutilisable dans le cas de `NULL`. Il faudra donc utiliser la seconde syntaxe, avec le test `IS NULL`.



## Seconde syntaxe : toutes conditions

Cette seconde syntaxe ne compare pas un élément à différentes valeurs, mais utilise simplement des conditions classiques et permet donc de faire des comparaisons de type "plus grand que", "différent de", etc. (bien entendu, elle peut également être utilisée pour des égalités).


```sql
CASE
    WHEN condition THEN instructions
    [WHEN condition THEN instructions] ...
    [ELSE instructions]
END CASE
```


**Exemple** : on reprend la procédure _avant_apres_2010()_, qu'on réécrit avec `CASE`, et en donnant une possibilité en plus. De plus, on passe le message en paramètre `OUT` pour changer un peu.


```sql
DELIMITER |
CREATE PROCEDURE avant_apres_2010_case (IN p_animal_id INT, OUT p_message VARCHAR(100))
BEGIN
    DECLARE v_annee INT;

    SELECT YEAR(date_naissance) INTO v_annee
    FROM Animal
    WHERE id = p_animal_id;

    CASE
        WHEN v_annee < 2010 THEN
            SET p_message = 'Je suis né avant 2010.';
        WHEN v_annee = 2010 THEN
            SET p_message = 'Je suis né en 2010.';
        ELSE
            SET p_message = 'Je suis né après 2010.';   
    END CASE;
END |
DELIMITER ;

CALL avant_apres_2010_case(59, @message);   
SELECT @message;
CALL avant_apres_2010_case(62, @message);   
SELECT @message;
CALL avant_apres_2010_case(69, @message);
SELECT @message;
```



## Comportement particulier : aucune correspondance trouvée

En l'absence de clause `ELSE`, si aucune des conditions posées par les différentes clauses `WHEN` n'est remplie (quelle que soit la syntaxe utilisée), une erreur est déclenchée.

Par exemple, cette procédure affiche une salutation différente selon la terminaison du nom de l'animal passé en paramètre :


```sql
DELIMITER |
CREATE PROCEDURE salut_nom(IN p_animal_id INT)
BEGIN
    DECLARE v_terminaison CHAR(1);

    SELECT SUBSTRING(nom, -1, 1) INTO v_terminaison  -- Une position négative signifie qu'on recule au lieu d'avancer. 
    FROM Animal                                      --  -1 est donc la dernière lettre du nom.
    WHERE id = p_animal_id;

    CASE v_terminaison
        WHEN 'a' THEN
            SELECT 'Bonjour !' AS Salutations;
        WHEN 'o' THEN
            SELECT 'Salut !' AS Salutations;
        WHEN 'i' THEN
            SELECT 'Coucou !' AS Salutations;
    END CASE;

END|
DELIMITER ;

CALL salut_nom(69);  -- Baba
CALL salut_nom(5);   -- Choupi
CALL salut_nom(29);  -- Fiero
CALL salut_nom(54);  -- Bubulle
```




+-----------+
|Salutations|
+===========+
|Bonjour !  |
+-----------+







+-----------+
|Salutations|
+===========+
|Coucou !   |
+-----------+







+-----------+
|Salutations|
+===========+
|Salut !    |
+-----------+





```console
ERROR 1339 (20000): Case not found for CASE statement
```


L'appel de la procédure avec Bubulle présente un cas qui n'est pas couvert par les trois `WHEN`. Une erreur est donc déclenchée

Donc, si l'on n'est pas sûr d'avoir couvert tous les cas possibles, il faut toujours ajouter une clause `ELSE` pour éviter les erreurs. 
Si l'on veut qu'aucune instruction ne soit exécutée par le `ELSE`, il suffit simplement de mettre un bloc d'instructions vide (`BEGIN END;`).

**Exemple** : reprenons la procédure _salut_nom()_, et ajoutons-lui une clause `ELSE` vide : 


```sql
DROP PROCEDURE salut_nom;
DELIMITER |
CREATE PROCEDURE salut_nom(IN p_animal_id INT)
BEGIN
    DECLARE v_terminaison CHAR(1);

    SELECT SUBSTRING(nom, -1, 1) INTO v_terminaison
    FROM Animal
    WHERE id = p_animal_id;

    CASE v_terminaison
        WHEN 'a' THEN
            SELECT 'Bonjour !' AS Salutations;
        WHEN 'o' THEN
            SELECT 'Salut !' AS Salutations;
        WHEN 'i' THEN
            SELECT 'Coucou !' AS Salutations;
        ELSE
            BEGIN                              -- Bloc d'instructions vide
            END;
    END CASE;

END|
DELIMITER ;

CALL salut_nom(69);  -- Baba
CALL salut_nom(5);   -- Choupi
CALL salut_nom(29);  -- Fiero
CALL salut_nom(54);  -- Bubulle
```


Cette fois, pas d'erreur. Le dernier appel (avec Bubulle) n'affiche simplement rien.


[[erreur]]
| Il faut au minimum une instruction ou un bloc d'instructions par clause `WHEN` et par clause `ELSE`. Un bloc vide `BEGIN END;` est donc nécessaire si l'on ne veut rien exécuter.



# Utiliser une structure conditionnelle directement dans une requête

Jusqu'ici, on a vu l'usage des structures conditionnelles dans des procédures stockées. Il est cependant possible d'utiliser une structure `CASE` dans une simple requête.

Par exemple, écrivons une requête `SELECT` suivant le même principe que la procédure _message_sexe()_ :


```sql
SELECT id, nom, CASE
          WHEN sexe = 'M' THEN 'Je suis un mâle !'
          WHEN sexe = 'F' THEN 'Je suis une femelle !'
          ELSE 'Je suis en plein questionnement existentiel...'
       END AS message
FROM Animal
WHERE id IN (9, 8, 6);
```




+--+--------+----------------------------------------------+
|id|nom     |message                                       |
+==+========+==============================================+
|6 |Bobosse |Je suis une femelle !                         |
+--+--------+----------------------------------------------+
|8 |Bagherra|Je suis un mâle !                             |
+--+--------+----------------------------------------------+
|9 |NULL    |Je suis en plein questionnement existentiel...|
+--+--------+----------------------------------------------+





Quelques remarques :


- On peut utiliser les deux syntaxes de `CASE`.
- Il faut clôturer le `CASE` par `END`, et non par `END CASE` (et bien sûr ne pas mettre de ||;|| si la requête n'est pas finie).
- Ce n'est pas limité aux clauses `SELECT`, on peut tout à fait utiliser un `CASE` dans une clause `WHERE` par exemple.
- Ce n'est par conséquent pas non plus limité aux requêtes `SELECT`, on peut l'utiliser dans n'importe quelle requête.


Il n'est par contre pas possible d'utiliser une structure `IF` dans une requête. Cependant, il existe une **fonction** `IF()`, beaucoup plus limitée, dont la syntaxe est la suivante :

```sql
IF(condition, valeur_si_vrai, valeur_si_faux)
```


**Exemple** :


```sql
SELECT nom, IF(sexe = 'M', 'Je suis un mâle', 'Je ne suis pas un mâle') AS sexe
FROM Animal
WHERE espece_id = 5;
```




+----+----------------------+
|nom |sexe                  |
+====+======================+
|Baba|Je ne suis pas un mâle|
+----+----------------------+
|Bibo|Je suis un mâle       |
+----+----------------------+
|Momy|Je ne suis pas un mâle|
+----+----------------------+
|Popi|Je suis un mâle       |
+----+----------------------+
|Mimi|Je ne suis pas un mâle|
+----+----------------------+