Une boucle est une structure qui permet de répéter plusieurs fois une série d'instructions. Il existe trois types de boucles en MySQL : `WHILE`, `LOOP` et `REPEAT`.


# La boucle WHILE

La boucle `WHILE` permet de répéter une série d'instructions **tant que la condition donnée reste vraie**.


```sql
WHILE condition DO    -- Attention de ne pas oublier le DO, erreur classique
    instructions
END WHILE;
```


**Exemple** : la procédure suivante affiche les nombres entiers de 1 à _p_nombre_ (passé en paramètre).


```sql
DELIMITER |
CREATE PROCEDURE compter_jusque_while(IN p_nombre INT)
BEGIN
    DECLARE v_i INT DEFAULT 1;

    WHILE v_i <= p_nombre DO
        SELECT v_i AS nombre; 

        SET v_i = v_i + 1;    -- À ne surtout pas oublier, sinon la condition restera vraie
    END WHILE;
END |
DELIMITER ;

CALL compter_jusque_while(3);
```



[[attention]]
| Vérifiez que votre condition devient bien fausse après un certain nombre d'itérations de la boucle. Sinon, vous vous retrouvez avec une boucle infinie (qui ne s'arrête jamais).



# La boucle REPEAT

La boucle `REPEAT` travaille en quelque sorte de manière opposée à `WHILE`, puisqu'elle exécute des instructions de la boucle **jusqu'à ce que la condition donnée devienne vraie**.

**Exemple** : voici la même procédure écrite avec une boucle `REPEAT`.


```sql
DELIMITER |
CREATE PROCEDURE compter_jusque_repeat(IN p_nombre INT)
BEGIN
    DECLARE v_i INT DEFAULT 1;

    REPEAT
        SELECT v_i AS nombre; 

        SET v_i = v_i + 1;    -- À ne surtout pas oublier, sinon la condition restera vraie
    UNTIL v_i > p_nombre END REPEAT;
END |
DELIMITER ;

CALL compter_jusque_repeat(3);
```



[[attention]]
| Attention, comme la condition d'une boucle `REPEAT` est vérifiée après le bloc d'instructions de la boucle, **on passe au moins une fois dans la boucle**, même si la condition est tout de suite fausse !

**Test**


```sql
-- Condition fausse dès le départ, on ne rentre pas dans la boucle
CALL compter_jusque_while(0);   

-- Condition fausse dès le départ, on rentre quand même une fois dans la boucle
CALL compter_jusque_repeat(0);
```



# Donner un label à une boucle

Il est possible de donner un label (un nom) à une boucle, ou à un bloc d'instructions défini par `BEGIN... END`. Il suffit pour cela de faire précéder l'ouverture de la boucle/du bloc par ce label, suivi de ||:||.

La fermeture de la boucle/du bloc peut alors faire référence à ce label (mais ce n'est pas obligatoire).


[[attention]]
| Un label ne peut pas dépasser 16 caractères.

**Exemples**

```sql
-- Boucle WHILE
-- ------------
super_while: WHILE condition DO    -- La boucle a pour label "super_while"
    instructions
END WHILE super_while;             -- On ferme en donnant le label de la boucle (facultatif)

-- Boucle REPEAT
-- -------------                  
repeat_genial: REPEAT              -- La boucle s'appelle "repeat_genial"
    instructions
UNTIL condition END REPEAT;        -- Cette fois, on choisit de ne pas faire référence au label lors de la fermeture

-- Bloc d'instructions
-- -------------------
bloc_extra: BEGIN                  -- Le bloc a pour label "bloc_extra"
    instructions
END bloc_extra;
```



[[question]]
| Mais en quoi cela peut-il être utile ?

D'une part, cela peut permettre de **clarifier le code** lorsqu'il y a beaucoup de boucles et de blocs d'instructions imbriqués. D'autre part, il est nécessaire de donner un label aux boucles et aux blocs d'instructions pour lesquels on veut pouvoir **utiliser les instructions `ITERATE` et `LEAVE`**.


# Les instructions LEAVE et ITERATE


## LEAVE : quitter la boucle ou le bloc d'instructions

L'instruction `LEAVE` peut s'utiliser **dans une boucle ou un bloc d'instructions** et **déclenche la sortie immédiate de la structure dont le label est donné**.


```sql
LEAVE label_structure;
```


**Exemple** : cette procédure incrémente de 1, et affiche, un nombre entier passé en paramètre. Et cela, 4 fois maximum. Mais si l'on trouve un multiple de 10, la boucle s'arrête.


```sql
DELIMITER |
CREATE PROCEDURE test_leave1(IN p_nombre INT)
BEGIN
    DECLARE v_i INT DEFAULT 4;
    
    SELECT 'Avant la boucle WHILE';
    
    while1: WHILE v_i > 0 DO

        SET p_nombre = p_nombre + 1;        -- On incrémente le nombre de 1
        
        IF p_nombre%10 = 0 THEN              -- Si p_nombre est divisible par 10,
            SELECT 'Stop !' AS 'Multiple de 10';
            LEAVE while1;                   -- On quitte la boucle WHILE.
        END IF;
        
        SELECT p_nombre;                    -- On affiche p_nombre
        SET v_i = v_i - 1;                  -- Attention de ne pas l'oublier
    
    END WHILE while1;

    SELECT 'Après la boucle WHILE';
END|
DELIMITER ;

CALL test_leave1(3); -- La boucle s'exécutera 4 fois
```




+---------------------+
|Avant la boucle WHILE|
+=====================+
|Avant la boucle WHILE|
+---------------------+







+--------+
|p_nombre|
+========+
|4       |
+--------+







+--------+
|p_nombre|
+========+
|5       |
+--------+







+--------+
|p_nombre|
+========+
|6       |
+--------+







+--------+
|p_nombre|
+========+
|7       |
+--------+







+---------------------+
|Après la boucle WHILE|
+=====================+
|Après la boucle WHILE|
+---------------------+






```sql
CALL test_leave1(8); -- La boucle s'arrêtera dès qu'on atteint 10
```




+---------------------+
|Avant la boucle WHILE|
+=====================+
|Avant la boucle WHILE|
+---------------------+







+--------+
|p_nombre|
+========+
|9       |
+--------+







+--------------+
|Multiple de 10|
+==============+
|Stop !        |
+--------------+







+---------------------+
|Après la boucle WHILE|
+=====================+
|Après la boucle WHILE|
+---------------------+





Il est par conséquent possible d'utiliser `LEAVE` pour provoquer la fin de la procédure stockée.

**Exemple** : voici la même procédure. Cette fois-ci un multiple de 10 provoque l'arrêt de toute la procédure, pas seulement de la boucle `WHILE`.


```sql
DELIMITER |
CREATE PROCEDURE test_leave2(IN p_nombre INT)
corps_procedure: BEGIN                           -- On donne un label au bloc d'instructions principal
    DECLARE v_i INT DEFAULT 4;
    
    SELECT 'Avant la boucle WHILE';
    while1: WHILE v_i > 0 DO
        SET p_nombre = p_nombre + 1;             -- On incrémente le nombre de 1
        IF p_nombre%10 = 0 THEN                  -- Si p_nombre est divisible par 10,
            SELECT 'Stop !' AS 'Multiple de 10';
            LEAVE corps_procedure;               -- je quitte la procédure.
        END IF;
        
        SELECT p_nombre;                         -- On affiche p_nombre
        SET v_i = v_i - 1;                       -- Attention de ne pas l'oublier
    END WHILE while1;

    SELECT 'Après la boucle WHILE';
END|
DELIMITER ;

CALL test_leave2(8);
```


`'Après la boucle WHILE'` ne s'affiche plus lorsque l'instruction `LEAVE` est déclenchée, puisque l'on quitte la procédure stockée avant d'arriver à l'instruction `SELECT` qui suit la boucle `WHILE`.

En revanche, `LEAVE` ne permet pas de quitter directement une structure conditionnelle (`IF` ou `CASE`). Il n'est d'ailleurs pas non plus possible de donner un label à ces structures. Cette restriction est cependant aisément contournable en utilisant les blocs d'instructions.

**Exemple** : la procédure suivante affiche les nombres de 4 à 1, en précisant s'ils sont pairs. Sauf pour le nombre 2, pour lequel une instruction `LEAVE` empêche l'affichage habituel.


```sql
DELIMITER |
CREATE PROCEDURE test_leave3()
BEGIN
    DECLARE v_i INT DEFAULT 4;
    
    WHILE v_i > 0 DO
        
        IF v_i%2 = 0 THEN
            if_pair: BEGIN
                IF v_i = 2 THEN                   -- Si v_i vaut 2
                    LEAVE if_pair;                -- On quitte le bloc "if_pair", ce qui revient à quitter la structure IF v_i%2 = 0
                END IF;
                SELECT CONCAT(v_i, ' est pair') AS message;
            END if_pair; 
        ELSE    
            if_impair: BEGIN
                SELECT CONCAT(v_i, ' est impair') AS message;
            END if_impair;   
        END IF;

        SET v_i = v_i - 1;
    END WHILE;
END|
DELIMITER ;

CALL test_leave3();
```




+----------+
|message   |
+==========+
|4 est pair|
+----------+







+------------+
|message     |
+============+
|3 est impair|
+------------+







+------------+
|message     |
+============+
|1 est impair|
+------------+





`'2 est pair'` n'est pas affiché, puisqu'on a quitté le `IF` avant cet affichage.


## ITERATE : déclencher une nouvelle itération de la boucle

Cette instruction ne peut être utilisée que dans une boucle. Lorsqu'elle est exécutée, une **nouvelle itération de la boucle commence**. Toutes les instructions suivant `ITERATE` dans la boucle sont ignorées.

**Exemple** : la procédure suivante affiche les nombres de 1 à 3, avec un message avant le `IF` et après le `IF`. Sauf pour le nombre 2, qui relance une itération de la boucle dans le `IF`.


```sql
DELIMITER |
CREATE PROCEDURE test_iterate()
BEGIN
    DECLARE v_i INT DEFAULT 0;

    boucle_while: WHILE v_i < 3 DO
        SET v_i = v_i + 1;
        SELECT v_i, 'Avant IF' AS message;
 
        IF v_i = 2 THEN
            ITERATE boucle_while;
        END IF;
       
        SELECT v_i, 'Après IF' AS message;  -- Ne sera pas exécuté pour v_i = 2
    END WHILE;
END |
DELIMITER ;

CALL test_iterate();
```




+---+--------+
|v_i|message |
+===+========+
|1  |Avant IF|
+---+--------+







+---+--------+
|v_i|message |
+===+========+
|1  |Après IF|
+---+--------+







+---+--------+
|v_i|message |
+===+========+
|2  |Avant IF|
+---+--------+







+---+--------+
|v_i|message |
+===+========+
|3  |Avant IF|
+---+--------+







+---+--------+
|v_i|message |
+===+========+
|3  |Après IF|
+---+--------+






[[attention]]
| Attention à ne pas faire de boucle infinie avec `ITERATE`, on oublie facilement que cette instruction empêche l'exécution de toutes les instructions qui la suivent dans la boucle. Si j'avais mis par exemple `SET v_i = v_i + 1;` après `ITERATE` et non avant, la boucle serait restée coincée à `v_i = 2`.



# La boucle LOOP

On a gardé la boucle `LOOP` pour la fin, parce qu'elle est un peu particulière. En effet, voici sa syntaxe :


```sql
[label:] LOOP
    instructions
END LOOP [label]
```


Vous voyez bien : il n'est question de condition nulle part. En fait, une boucle `LOOP` doit intégrer dans ses instructions un élément qui va la faire s'arrêter : typiquement une instruction `LEAVE`. Sinon, c'est une boucle infinie.

**Exemple** : à nouveau une procédure qui affiche les nombres entiers de 1 à _p_nombre_.


```sql
DELIMITER |
CREATE PROCEDURE compter_jusque_loop(IN p_nombre INT)
BEGIN
    DECLARE v_i INT DEFAULT 1;

    boucle_loop: LOOP
        SELECT v_i AS nombre; 

        SET v_i = v_i + 1;

        IF v_i > p_nombre THEN
            LEAVE boucle_loop;
        END IF;    
    END LOOP;
END |
DELIMITER ;

CALL compter_jusque_loop(3);
```