Il arrive régulièrement qu'un traitement soit susceptible de générer une erreur SQL. Prenons la procédure suivante, qui enregistre une adoption :


```sql
DELIMITER |
CREATE PROCEDURE ajouter_adoption(IN p_client_id INT, IN p_animal_id INT, IN p_date DATE, IN p_paye TINYINT)
BEGIN
    DECLARE v_prix DECIMAL(7,2);
    
    SELECT COALESCE(Race.prix, Espece.prix) INTO v_prix
    FROM Animal
    INNER JOIN Espece ON Espece.id = Animal.espece_id
    LEFT JOIN Race ON Race.id = Animal.race_id
    WHERE Animal.id = p_animal_id;
    
    INSERT INTO Adoption (animal_id, client_id, date_reservation, date_adoption, prix, paye)
    VALUES (p_animal_id, p_client_id, CURRENT_DATE(), p_date, v_prix, p_paye);
    
  
    SELECT 'Adoption correctement ajoutée' AS message;
END|
DELIMITER ;
```


Plusieurs erreurs sont susceptibles de se déclencher selon les paramètres passés à cette procédure.

**Exemple 1** : le client n'existe pas.


```sql
SET @date_adoption = CURRENT_DATE() + INTERVAL 7 DAY;

CALL ajouter_adoption(18, 6, @date_adoption, 1);
```


```console
ERROR 1452 (23000): Cannot add or update a child row: a foreign key constraint fails (`elevage`.`Adoption`, CONSTRAINT `Adoption_ibfk_1` FOREIGN KEY (`client_id`) REFERENCES `Client` (`id`))
```


**Exemple 2** : l'animal a déjà été adopté.


```sql
CALL ajouter_adoption(12, 21, @date_adoption, 1);
```


```console
ERROR 1062 (23000): Duplicate entry '21' for key 'ind_uni_animal_id'
```


**Exemple 3** : l'animal n'existe pas, _v_prix_ est donc `NULL`.


```sql
CALL ajouter_adoption(12, 102, @date_adoption, 1);
```


```console
ERROR 1048 (23000): Column 'prix' cannot be null
```


Pour empêcher ces erreurs intempestives, deux solutions : 


- vérifier chaque paramètre pouvant poser problème (_p_animal_id_ et _p_client_id_ ne sont pas `NULL` et correspondent à quelque chose dans les tables _Animal_ et _Client_, _p_animal_id_ ne correspond pas à un animal déjà adopté, etc.) ;
- utiliser un gestionnaire d'erreur : c'est ce que nous allons apprendre à faire ici.



# Création d'un gestionnaire d'erreur

Voici la syntaxe à utiliser pour créer un gestionnaire d'erreur :


```sql
DECLARE { EXIT | CONTINUE } HANDLER FOR { numero_erreur | { SQLSTATE identifiant_erreur } | condition } 
    instruction ou bloc d'instructions
```



- Un gestionnaire d'erreur définit une instruction (une seule !), ou un bloc d'instructions (`BEGIN ... END;`), qui va être **exécuté en cas d'erreur** correspondant au gestionnaire.
- Tous les gestionnaires d'erreur doivent être déclarés au même endroit : **après la déclaration des variables locales, mais avant les instructions de la procédure**.
- Un gestionnaire peut, soit provoquer l'**arrêt de la procédure** (`EXIT`), soit **faire reprendre la procédure** après avoir géré l'erreur (`CONTINUE`).
- On peut identifier le type d'erreur que le gestionnaire va reconnaître de trois manières différentes : **un numéro d'erreur, un identifiant, ou une `CONDITION`**.
- Un gestionnaire étant défini grâce au mot-clé `DECLARE`, comme les variables locales, il a exactement **la même portée** que celles-ci.


**Exemples** : ces deux procédures enregistrent une adoption en gérant les erreurs, l'une arrêtant la procédure, l'autre relançant celle-ci :


```sql
DELIMITER |
CREATE PROCEDURE ajouter_adoption_exit(IN p_client_id INT, IN p_animal_id INT, IN p_date DATE, IN p_paye TINYINT)
BEGIN
    DECLARE v_prix DECIMAL(7,2);
    DECLARE EXIT HANDLER FOR SQLSTATE '23000' 
        BEGIN
            SELECT 'Une erreur est survenue...';
            SELECT 'Arrêt prématuré de la procédure';
        END;

    SELECT 'Début procédure';

    SELECT COALESCE(Race.prix, Espece.prix) INTO v_prix
    FROM Animal
    INNER JOIN Espece ON Espece.id = Animal.espece_id
    LEFT JOIN Race ON Race.id = Animal.race_id
    WHERE Animal.id = p_animal_id;
    
    INSERT INTO Adoption (animal_id, client_id, date_reservation, date_adoption, prix, paye)
    VALUES (p_animal_id, p_client_id, CURRENT_DATE(), p_date, v_prix, p_paye);

    SELECT 'Fin procédure' AS message;
END|

CREATE PROCEDURE ajouter_adoption_continue(IN p_client_id INT, IN p_animal_id INT, IN p_date DATE, IN p_paye TINYINT)
BEGIN
    DECLARE v_prix DECIMAL(7,2);
    DECLARE CONTINUE HANDLER FOR SQLSTATE '23000' SELECT 'Une erreur est survenue...';

    SELECT 'Début procédure';

    SELECT COALESCE(Race.prix, Espece.prix) INTO v_prix
    FROM Animal
    INNER JOIN Espece ON Espece.id = Animal.espece_id
    LEFT JOIN Race ON Race.id = Animal.race_id
    WHERE Animal.id = p_animal_id;
    
    INSERT INTO Adoption (animal_id, client_id, date_reservation, date_adoption, prix, paye)
    VALUES (p_animal_id, p_client_id, CURRENT_DATE(), p_date, v_prix, p_paye);
  
    SELECT 'Fin procédure';
END|
DELIMITER ;

SET @date_adoption = CURRENT_DATE() + INTERVAL 7 DAY;

CALL ajouter_adoption_exit(18, 6, @date_adoption, 1);
CALL ajouter_adoption_continue(18, 6, @date_adoption, 1);
```


Les instructions définies par le gestionnaire sont bien exécutées, mais `'Fin procédure'` n'est affiché que dans le cas de _ajouter_adoption_continue()_, qui fait reprendre la procédure une fois l'erreur gérée. La procédure _ajouter_adoption_exit()_ utilise un bloc d'instructions et peut donc exécuter plusieurs instructions.


# Définition de l'erreur gérée


## Identifiant ou numéro MySQL de l'erreur

Voici la déclaration du gestionnaire dans la procédure _ajouter_adoption_continue()_ :


```sql
DECLARE CONTINUE HANDLER FOR SQLSTATE '23000' SELECT 'Une erreur est survenue...';
```

Et voici une des erreurs qui peut être interceptée par ce gestionnaire :


```console
ERROR 1062 (23000): Duplicate entry '21' for key 'ind_uni_animal_id'
```

Le message d'erreur est constitué de trois éléments importants :


- 1062 : le **numéro d'erreur MySQL** (un nombre entier) ;
- 23000 : l'**identifiant de l'état SQL** (une chaîne de 5 caractères) ;
- Duplicate entry '21' for key 'ind_uni_animal_id' : un message donnant le détail de l'erreur.


**Identifiant de l'état SQL**

Dans la procédure _ajouter_adoption_continue()_, c'est l'identifiant de l'état SQL (`'23000'`) qui a été utilisé.
Il s'agit d'une chaîne de 5 caractères, renvoyée par le serveur au client pour **informer de la réussite ou de l'échec d'une instruction**. Un identifiant commençant par `'00'` par exemple, signifie que l'instruction a réussi. `'23000'` est l'identifiant renvoyé lorsqu'une erreur concernant une contrainte (`NOT NULL`, unicité, clé primaire ou secondaire,…) a été déclenchée.
Pour utiliser cet identifiant dans un gestionnaire d'erreur, il faut le faire précéder de `SQLSTATE`.

**Numéro d'erreur MySQL**

Pour utiliser le numéro d'erreur SQL, par contre, il suffit de l'indiquer, comme un nombre entier :


```sql
DECLARE CONTINUE HANDLER FOR 1062 SELECT 'Une erreur est survenue...';
```

Ce sont des codes qui, contrairement aux identifiants `SQLSTATE`, sont **propres à MySQL**. Ils sont aussi en général **plus précis**. L'identifiant SQL '23000' par exemple, correspond à une dizaine de codes d'erreur MySQL différents.

**Quelques exemples de codes souvent rencontrés**




+----------+----------+-------------------------------------------------------------+
|Code MySQL|`SQLSTATE`|Description                                                  |
+==========+==========+=============================================================+
|1048      |23000     |La colonne _x_ ne peut pas être `NULL`                       |
+----------+----------+-------------------------------------------------------------+
|1169      |23000     |Violation de contrainte d'unicité                            |
+----------+----------+-------------------------------------------------------------+
|1216      |23000     |Violation de clé secondaire : insertion ou modification      |
|          |          |impossible (table avec la clé secondaire)                    |
+----------+----------+-------------------------------------------------------------+
|1217      |23000     |Violation de clé secondaire : suppression ou modification    |
|          |          |impossible (table avec la **référence** de la clé secondaire)|
+----------+----------+-------------------------------------------------------------+
|1172      |42000     |Plusieurs lignes de résultats alors qu'on ne peut en avoir   |
|          |          |qu'une seule                                                 |
+----------+----------+-------------------------------------------------------------+
|1242      |21000     |La sous-requête retourne plusieurs lignes de résultats       |
|          |          |alors qu'on ne peut en avoir qu'une seule                    |
+----------+----------+-------------------------------------------------------------+





Pour une liste plus complète, il suffit d'aller sur la [documentation officielle](http://dev.mysql.com/doc/refman/5.5/en/error-messages-server.html).

Pour finir, notez qu'il est tout à fait possible d'intercepter des avertissements avec un gestionnaire d'erreur, qui sont également représentés par un identifiant SQL et un code d'erreur MySQL. Un avertissement, contrairement à une erreur, ne fait pas échouer l'instruction par défaut, mais en l'interceptant dans une requête stockée avec un gestionnaire, vous pouvez décider du comportement à adopter suite à cet avertissement.

Vous aurez par exemple un avertissement si vous insérez un `DATETIME` dans une colonne `DATE`, puisque la donnée sera tronquée pour correspondre au type de la colonne (l'heure sera supprimée pour ne garder que la date) : code Mysql `1265`, `SQLSTATE '01000'`.


## Utilisation d'une CONDITION

Avec un numéro d'erreur MySQL et un identifiant d'état SQL, il existe une troisième manière d'identifier les erreurs reconnues par un gestionnaire : une `CONDITION`.
Une `CONDITION` est en fait simplement un nom donné à un numéro d'erreur MySQL ou à un identifiant d'état SQL. Cela vous permet de travailler avec des erreurs plus claires.

Voici la syntaxe à utiliser pour nommer une erreur. Il s'agit à nouveau d'un `DECLARE`. Les déclarations de `CONDITION` doivent se trouver avant les déclarations de gestionnaires.


```sql
DECLARE nom_erreur CONDITION FOR { SQLSTATE identifiant_SQL | numero_erreur_MySQL };
```


**Exemple** : réécrivons la procédure _ajouter_adoption_exit()_ en nommant l'erreur :


```sql
DROP PROCEDURE ajouter_adoption_exit;
DELIMITER |
CREATE PROCEDURE ajouter_adoption_exit(IN p_client_id INT, IN p_animal_id INT, IN p_date DATE, IN p_paye TINYINT)
BEGIN
    DECLARE v_prix DECIMAL(7,2);

    DECLARE violation_contrainte CONDITION FOR SQLSTATE '23000';   -- On nomme l'erreur dont l'identifiant est 23000 "violation_contrainte"

    DECLARE EXIT HANDLER FOR violation_contrainte                  -- Le gestionnaire sert donc à intercepter
        BEGIN                                                      --   les erreurs de type "violation_contrainte"
            SELECT 'Une erreur est survenue...';
            SELECT 'Arrêt prématuré de la procédure';
        END;

    SELECT 'Début procédure';

    SELECT COALESCE(Race.prix, Espece.prix) INTO v_prix
    FROM Animal
    INNER JOIN Espece ON Espece.id = Animal.espece_id
    LEFT JOIN Race ON Race.id = Animal.race_id
    WHERE Animal.id = p_animal_id;
    
    INSERT INTO Adoption (animal_id, client_id, date_reservation, date_adoption, prix, paye)
    VALUES (p_animal_id, p_client_id, CURRENT_DATE(), p_date, v_prix, p_paye);

    SELECT 'Fin procédure';
END|
DELIMITER ;
```



## Conditions prédéfinies

Il existe trois conditions prédéfinies dans MySQL :


- `SQLWARNING` : tous les identifiants SQL commençant par '01', c'est-à-dire les avertissements et les notes ;
- `NOT FOUND` : tous les identifiants SQL commençant par '02', et que nous verrons plus en détail avec les curseurs ;
- `SQLEXCEPTION` : tous les identifiants SQL ne commençant ni par '00', ni par '01', ni par '02', donc les erreurs.



**Exemple** : réécriture de la procédure _ajouter_adoption_exit()_, de façon à ce que le gestionnaire intercepte toutes les erreurs SQL.


```sql
DROP PROCEDURE ajouter_adoption_exit;
DELIMITER |
CREATE PROCEDURE ajouter_adoption_exit(IN p_client_id INT, IN p_animal_id INT, IN p_date DATE, IN p_paye TINYINT)
BEGIN
    DECLARE v_prix DECIMAL(7,2);

    DECLARE EXIT HANDLER FOR SQLEXCEPTION              
        BEGIN
            SELECT 'Une erreur est survenue...';
            SELECT 'Arrêt prématuré de la procédure';
        END;

    SELECT 'Début procédure';

    SELECT COALESCE(Race.prix, Espece.prix) INTO v_prix
    FROM Animal
    INNER JOIN Espece ON Espece.id = Animal.espece_id
    LEFT JOIN Race ON Race.id = Animal.race_id
    WHERE Animal.id = p_animal_id;
    
    INSERT INTO Adoption (animal_id, client_id, date_reservation, date_adoption, prix, paye)
    VALUES (p_animal_id, p_client_id, CURRENT_DATE(), p_date, v_prix, p_paye);

    SELECT 'Fin procédure';
END|
DELIMITER ;
```



# Déclarer plusieurs gestionnaires, gérer plusieurs erreurs par gestionnaire

Un gestionnaire peut reconnaître plusieurs types d'erreurs différents. Par ailleurs, il est possible de déclarer plusieurs gestionnaires dans un même bloc d'instructions.

**Exemple** : toujours avec la procédure _ajouter_adoption_exit()_. On peut l'écrire en détaillant différentes erreurs possibles, puis en ajoutant un gestionnaire général qui reconnaîtra les `SQLEXCEPTION` et les `SQLWARNING`, pour tous les cas qu'on ne traite pas dans les autres gestionnaires. Ce qui donne :


```sql
DROP PROCEDURE ajouter_adoption_exit;
DELIMITER |
CREATE PROCEDURE ajouter_adoption_exit(IN p_client_id INT, IN p_animal_id INT, IN p_date DATE, IN p_paye TINYINT)
BEGIN
    DECLARE v_prix DECIMAL(7,2);

    DECLARE violation_cle_etrangere CONDITION FOR 1452;            -- Déclaration des CONDITIONS
    DECLARE violation_unicite CONDITION FOR 1062;
    
    DECLARE EXIT HANDLER FOR violation_cle_etrangere               -- Déclaration du gestionnaire pour     
        BEGIN                                                      -- les erreurs de clés étrangères
            SELECT 'Erreur : violation de clé étrangère.';
        END;
    DECLARE EXIT HANDLER FOR violation_unicite                     -- Déclaration du gestionnaire pour
        BEGIN                                                      -- les erreurs d'index unique
            SELECT 'Erreur : violation de contrainte d''unicité.';
        END;
    DECLARE EXIT HANDLER FOR SQLEXCEPTION, SQLWARNING              -- Déclaration du gestionnaire pour
        BEGIN                                                      -- toutes les autres erreurs ou avertissements
            SELECT 'Une erreur est survenue...';
        END;

    SELECT 'Début procédure';

    SELECT COALESCE(Race.prix, Espece.prix) INTO v_prix
    FROM Animal
    INNER JOIN Espece ON Espece.id = Animal.espece_id
    LEFT JOIN Race ON Race.id = Animal.race_id
    WHERE Animal.id = p_animal_id;
    
    INSERT INTO Adoption (animal_id, client_id, date_reservation, date_adoption, prix, paye)
    VALUES (p_animal_id, p_client_id, CURRENT_DATE(), p_date, v_prix, p_paye);

    SELECT 'Fin procédure';
END|
DELIMITER ;

SET @date_adoption = CURRENT_DATE() + INTERVAL 7 DAY;

CALL ajouter_adoption_exit(12, 3, @date_adoption, 1);        -- Violation unicité (animal 3 est déjà adopté)
CALL ajouter_adoption_exit(133, 6, @date_adoption, 1);       -- Violation clé étrangère (client 133 n'existe pas)
CALL ajouter_adoption_exit(NULL, 6, @date_adoption, 1);      -- Violation de contrainte NOT NULL
```

Cette procédure montre également que lorsque plusieurs gestionnaires d'erreur peuvent correspondre à l'erreur déclenchée (ou à l'avertissement), c'est le plus précis qui est utilisé. C'est la raison pour laquelle une violation de clé étrangère déclenche le gestionnaire `FOR violation_cle_etrangere` (numéro MySQL 1062), et non le gestionnaire `FOR SQLEXCEPTION`.