Vous avez maintenant vu les principales structures qu'il est possible d'utiliser dans un bloc d'instructions. Nous allons ici combiner ces structures avec des objets ou notions vues précédemment.

La puissance du langage SQL (et de tous les langages informatiques) réside dans le fait qu'on peut **combiner différentes notions pour réaliser des traitements complexes**. Voici quelques exemples de telles combinaisons. 
Notez que ces exemples utilisent tous des procédures stockées, mais la plupart sont adaptables à d'autres objets, comme les triggers, les fonctions stockées ou les événements.


# Utiliser des variables utilisateur dans un bloc d'instructions

En plus des variables locales, il est tout à fait possible d'utiliser des variables utilisateur dans un bloc d'instructions. Mais n'oubliez pas qu'**une variable utilisateur est définie pour toute la session, pas uniquement le bloc**, même si elle est créée à l'intérieur de celui-ci.

**Exemple** : procédure stockée utilisant une variable utilisateur.


```sql
DELIMITER |
CREATE PROCEDURE test_vu()
BEGIN
    SET @var = 15;
END|
DELIMITER ;

SELECT @var;    -- @var n'existe pas encore, on ne l'a pas définie
CALL test_vu(); -- On exécute la procédure
SELECT @var;    -- @var vaut maintenant 15, même en dehors de la procédure, puisqu'elle est définie partout dans la session
```

Voyant cela, on pourrait être tenté d'utiliser des variables utilisateur à la place des paramètres `OUT` et `INOUT` des procédures stockées. 

Cependant, il convient d'être extrêmement prudent lorsque l'on utilise des variables utilisateur dans un bloc d'instructions. 
Si l'on reste par exemple dans le contexte des procédures stockées, un des intérêts de celles-ci est d'avoir une interface entre la base de données et l'utilisateur. L'utilisateur n'est donc pas nécessairement conscient des variables utilisateur qui sont définies ou modifiées dans la procédure. Il pourrait donc définir des variables utilisateur, puis exécuter une procédure et constater que certaines de ses variables ont été écrasées par la procédure.

Ainsi dans le code suivant :


```sql
SET @var = 'Bonjour';
CALL test_vu();
SELECT @var;           -- Donne 15 !
```


Un utilisateur inconscient du fait que _test_vu()_ modifie `@var` pourrait bien y perdre des cheveux !

Un raisonnement similaire peut être tenu pour les autres objets utilisant les blocs d'instructions.
Évitez donc les variables utilisateur dans les blocs quand c'est possible (et nous allons bientôt voir un cas où ce n'est pas possible).


# Utiliser une procédure dans un bloc

`CALL nom_procedure();` est une instruction. On peut donc parfaitement exécuter une procédure dans un bloc d'instructions.

**Exemple** : la procédure _surface_cercle()_ calcule la surface d'un cercle à partir de son rayon. Elle exécute pour cela la procédure _carre()_, qui élève un nombre au carré.
Pour rappel, on calcule la surface d'un cercle avec la formule suivante : $S = \pi\times r^2$


```sql
DELIMITER |
CREATE PROCEDURE carre(INOUT p_nb FLOAT) SET p_nb = p_nb * p_nb|

CREATE PROCEDURE surface_cercle(IN p_rayon FLOAT, OUT p_surface FLOAT)
BEGIN
    CALL carre(p_rayon);

    SET p_surface = p_rayon * PI();
END|
DELIMITER ;

CALL surface_cercle(1, @surface);   -- Donne environ pi (3,14...)
SELECT @surface;
CALL surface_cercle(2, @surface);   -- Donne environ 12,57...
SELECT @surface;
```



[[information]]
| Un `FLOAT` stockant **une valeur approchée**, il est tout à fait possible (voire probable) que vous obteniez un résultat différent de celui donné par une calculette.



# Transactions et gestion d'erreurs

Un usage classique et utile des gestionnaires d'erreur est l'annulation des transactions en cas d'erreur.

**Exemple** : la procédure suivante prend en paramètre l'_id_ d'un client, et de deux animaux que le client veut adopter :


```sql
DELIMITER |
CREATE PROCEDURE adoption_deux_ou_rien(p_client_id INT, p_animal_id_1 INT, p_animal_id_2 INT)
BEGIN
    DECLARE v_prix DECIMAL(7,2);

    DECLARE EXIT HANDLER FOR SQLEXCEPTION ROLLBACK;  -- Gestionnaire qui annule la transaction et termine la procédure

    START TRANSACTION;

    SELECT COALESCE(Race.prix, Espece.prix) INTO v_prix
    FROM Animal
    INNER JOIN Espece ON Espece.id = Animal.espece_id
    LEFT JOIN Race ON Race.id = Animal.race_id
    WHERE Animal.id = p_animal_id_1;
    
    INSERT INTO Adoption (animal_id, client_id, date_reservation, date_adoption, prix, paye)
    VALUES (p_animal_id_1, p_client_id, CURRENT_DATE(), CURRENT_DATE(), v_prix, TRUE);

    SELECT 'Adoption animal 1 réussie' AS message;

    SELECT COALESCE(Race.prix, Espece.prix) INTO v_prix
    FROM Animal
    INNER JOIN Espece ON Espece.id = Animal.espece_id
    LEFT JOIN Race ON Race.id = Animal.race_id
    WHERE Animal.id = p_animal_id_2;
    
    INSERT INTO Adoption (animal_id, client_id, date_reservation, date_adoption, prix, paye)
    VALUES (p_animal_id_2, p_client_id, CURRENT_DATE(), CURRENT_DATE(), v_prix, TRUE);
    
    SELECT 'Adoption animal 2 réussie' AS message;
    
    COMMIT;
END|
DELIMITER ;

CALL adoption_deux_ou_rien(2, 43, 55);  -- L'animal 55 a déjà été adopté
```

La procédure s'interrompt, puisque la seconde insertion échoue. On n'exécute donc pas le second `SELECT`. Ici, grâce à la transaction et au `ROLLBACK` du gestionnaire, la première insertion a été annulée.


[[attention]]
| On ne peut pas utiliser de transactions dans un trigger.



# Préparer une requête dans un bloc d'instructions

Pour finir, on peut créer et exécuter une requête préparée dans un bloc d'instructions. Ceci permet de créer des requêtes dynamiques, puisqu'on prépare une requête à partir d'une chaîne de caractères.

**Exemple** : la procédure suivante ajoute la ou les clauses que l'on veut à une simple requête `SELECT` :


```sql
DELIMITER |
CREATE PROCEDURE select_race_dynamique(p_clause VARCHAR(255))
BEGIN
    SET @sql = CONCAT('SELECT nom, description FROM Race ', p_clause);
    
    PREPARE requete FROM @sql;
    EXECUTE requete;
END|
DELIMITER ;

CALL select_race_dynamique('WHERE espece_id = 2');  -- Affichera les races de chats
CALL select_race_dynamique('ORDER BY nom LIMIT 2'); -- Affichera les deux premières races par ordre alphabétique de leur nom
```

Il va sans dire que ce genre de construction dynamique de requêtes peut poser d'**énormes problèmes de sécurité** si l'on ne prend pas de précaution.
Par ailleurs, il n'est pas possible de construire une requête préparée à partir d'une variable locale. Il est donc nécessaire d'utiliser une variable utilisateur.


[[attention]]
| L'utilisation des requêtes préparées n'est pas permise dans un trigger.