# Syntaxe
 
Pour créer un trigger, on utilise la commande suivante :


```sql
CREATE TRIGGER nom_trigger moment_trigger evenement_trigger
ON nom_table FOR EACH ROW
corps_trigger
```



- `CREATE TRIGGER nom_trigger` : les triggers ont donc un nom.
- `moment_trigger evenement_trigger` : servent à définir quand et comment le trigger est déclenché.
- `ON nom_table` : c'est là qu'on définit à quelle table le trigger est attaché.
- `FOR EACH ROW` : signifie littéralement "pour chaque ligne", sous-entendu "pour chaque ligne insérée/supprimée/modifiée" selon ce qui a déclenché le trigger.
- `corps_trigger` : c'est le contenu du trigger. Comme pour les procédures stockées, il peut s'agir soit d'une seule instruction, soit d'un bloc d'instructions.



## Événement déclencheur

Trois événements différents peuvent déclencher l'exécution des instructions d'un trigger.


- L'insertion de lignes (`INSERT`) dans la table attachée au trigger.
- La modification de lignes (`UPDATE`) de cette table.
- La suppression de lignes (`DELETE`) de la table.


Un trigger est soit déclenché par `INSERT`, soit par `UPDATE`, soit par `DELETE`. Il ne peut pas être déclenché par deux événements différents. On peut par contre créer plusieurs triggers par table pour couvrir chaque événement.


## Avant ou après

Lorsqu'un trigger est déclenché, ses instructions peuvent être exécutées à deux moments différents. Soit juste avant que l'événement déclencheur n'ait lieu (`BEFORE`), soit juste après (`AFTER`).

Donc, si vous avez un trigger `BEFORE UPDATE` sur la table _A_, l'exécution d'une requête `UPDATE` sur cette table va d'abord déclencher l'exécution des instructions du trigger, ensuite seulement les lignes de la table seront modifiées.


## Exemple

Pour créer un trigger sur la table _Animal_, déclenché par une insertion, et s'exécutant après ladite insertion, on utilisera la syntaxe suivante :


```sql
CREATE TRIGGER after_insert_animal AFTER INSERT
ON Animal FOR EACH ROW
corps_trigger;
```



# Règle et convention

Il ne peut exister qu'un seul trigger par combinaison `moment_trigger/evenement_trigger` par table. Donc un seul trigger `BEFORE UPDATE` par table, un seul `AFTER DELETE`, etc.
Étant donné qu'il existe deux possibilités pour le moment d'exécution, et trois pour l'événement déclencheur, on a donc un **maximum de six triggers par table**.

Cette règle étant établie, il existe une convention quant à la manière de nommer ses triggers, que je vous encourage à suivre : _nom_trigger_ = _moment_evenement_table_. Donc le trigger `BEFORE UPDATE ON Animal` aura pour nom : _before_update_animal_.


# OLD et NEW
	
Dans le corps du trigger, MySQL met à disposition deux mots-clés : `OLD` et `NEW`.	


- `OLD` : représente les valeurs des colonnes de la ligne traitée **avant qu'elle ne soit modifiée** par l'événement déclencheur. Ces valeurs peuvent être **lues, mais pas modifiées**.
- `NEW` : représente les valeurs des colonnes de la ligne traitée **après qu'elle a été modifiée** par l'événement déclencheur. Ces valeurs peuvent être **lues et modifiées**.



Il n'y a que dans le cas d'un trigger `UPDATE` que `OLD` et `NEW` coexistent. Lors d'une insertion, `OLD` n'existe pas, puisque la ligne n'existe pas avant l'événement déclencheur ; dans le cas d'une suppression, c'est `NEW` qui n'existe pas, puisque la ligne n'existera plus après l'événement déclencheur.

**Premier exemple** : l'insertion d'une ligne.

Exécutons la commande suivante :


```sql
INSERT INTO Adoption (client_id, animal_id, date_reservation, prix, paye) 
VALUES (12, 15, NOW(), 200.00, FALSE);
```


Pendant le traitement de cette ligne par le trigger correspondant,


- `NEW.client_id` vaudra 12 ;
- `NEW.animal_id` vaudra 15 ;
- `NEW.date_reservation` vaudra `NOW()` ;
- `NEW.date_adoption` vaudra `NULL` ;
- `NEW.prix` vaudra 200.00 ;
- `NEW.paye` vaudra `FALSE` (0).


Les valeurs de `OLD` ne seront pas définies.
Dans le cas d'une suppression, on aura exactement l'inverse.

**Second exemple** : la modification d'une ligne.
On modifie la ligne que l'on vient d'insérer en exécutant la commande suivante :


```sql
UPDATE Adoption 
SET paye = TRUE
WHERE client_id = 12 AND animal_id = 15;
```

Pendant le traitement de cette ligne par le trigger correspondant,


- `NEW.paye` vaudra `TRUE`, tandis que `OLD.paye` vaudra `FALSE`.
- Par contre les valeurs respectives de `NEW.animal_id, NEW.client_id, NEW.date_reservation`, `NEW.date_adoption` et `NEW.prix` seront les mêmes que `OLD.animal_id, OLD.client_id, OLD.date_reservation`, `OLD.date_adoption` et `OLD.prix`, puisque ces colonnes ne sont pas modifiées par la requête.



[[attention]]
| Dans le cas d'une insertion ou d'une modification, si un trigger peut potentiellement changer la valeur de `NEW.colonne`, il doit être exécuté avant l'événement (`BEFORE`). Sinon, la ligne aura déjà été insérée ou modifiée, et la modification de `NEW.colonne` n'aura plus aucune influence sur celle-ci.



# Erreur déclenchée pendant un trigger



- Si un trigger `BEFORE` génère une erreur (non interceptée par un gestionnaire d'erreur), la requête ayant déclenché le trigger ne sera pas exécutée. Si l'événement devait également déclencher un trigger `AFTER`, il ne sera bien sûr pas non plus exécuté.
- Si un trigger `AFTER` génère une erreur, la requête ayant déclenché le trigger échouera.
- Dans le cas d'une table transactionnelle, si une erreur est déclenchée, un `ROLLBACK` sera fait. Dans le cas d'une table non-transactionnelle, tous les changements qui auraient été faits par le (ou les) trigger(s) avant le déclenchement de l'erreur persisteront.