# Contraintes et vérification des données


## Vérification du sexe des animaux

Dans notre table _Animal_ se trouve la colonne _sexe_. Cette colonne accepte tout caractère, ou `NULL`. Or, seuls les caractères "M" et "F" ont du sens. Nous allons donc créer deux triggers, un pour l'insertion, l'autre pour la modification, qui vont empêcher qu'on donne un autre caractère que "M" ou "F" pour _sexe_.

Ces deux triggers devront se déclencher avant l'insertion et la modification. On aura donc :


```sql
-- Trigger déclenché par l'insertion
DELIMITER |
CREATE TRIGGER before_insert_animal BEFORE INSERT
ON Animal FOR EACH ROW
BEGIN
    -- Instructions
END |

-- Trigger déclenché par la modification
CREATE TRIGGER before_update_animal BEFORE UPDATE
ON Animal FOR EACH ROW
BEGIN
    -- Instructions
END |
DELIMITER ;
```


Il ne reste plus qu'à écrire le code du trigger, qui sera similaire pour les deux triggers. Et comme ce corps contiendra des instructions, il ne faut pas oublier de changer le délimiteur.

Le corps consistera en une simple structure conditionnelle, et définira un comportement à adopter si le sexe donné ne vaut ni "M", ni "F", ni `NULL`.

[[question]]
| Quel comportement adopter en cas de valeur erronée ?
 
Deux possibilités :


- on modifie la valeur du sexe, en le mettant à `NULL` par exemple ;
- on provoque une erreur, ce qui empêchera l'insertion/la modification.


Commençons par le plus simple : mettre le sexe à `NULL`.


```sql
DELIMITER |
CREATE TRIGGER before_update_animal BEFORE UPDATE
ON Animal FOR EACH ROW
BEGIN
    IF NEW.sexe IS NOT NULL   -- le sexe n'est ni NULL
    AND NEW.sexe != 'M'       -- ni "M"
    AND NEW.sexe != 'F'       -- ni "F"
      THEN
        SET NEW.sexe = NULL;
    END IF;
END |
DELIMITER ;
```


**Test** :


```sql
UPDATE Animal
SET sexe = 'A'
WHERE id = 20;  -- l'animal 20 est Balou, un mâle

SELECT id, sexe, date_naissance, nom 
FROM Animal 
WHERE id = 20;
```




+--+----+-------------------+-----+
|id|sexe|date_naissance     |nom  |
+==+====+===================+=====+
|20|NULL|2007-04-24 12:45:00|Balou|
+--+----+-------------------+-----+





Le sexe est bien `NULL`, le trigger a fonctionné.

Pour le second trigger, déclenché par l'insertion de lignes, on va implémenter le second comportement : on va déclencher une erreur, ce qui empêchera l'insertion, et affichera l'erreur.


[[question]]
| Mais comment déclencher une erreur ?

Contrairement à certains SGBD, MySQL ne dispose pas d'une commande permettant de déclencher une erreur personnalisée. La seule solution est donc de faire une requête dont on sait qu'elle va générer une erreur.

**Exemple** :


```sql
SELECT 1, 2 INTO @a;
```


```console
ERROR 1222 (21000): The used SELECT statements have a different number of columns
```


Cependant, il serait quand même intéressant d'avoir un message d'erreur qui soit un peu explicite. Voici une manière d'obtenir un tel message : on crée une table _Erreur_, ayant deux colonnes, _id_ et _erreur_. La colonne _id_ est clé primaire, et _erreur_ contient un texte court décrivant l'erreur. Un index `UNIQUE` est ajouté sur cette dernière colonne. On insère ensuite une ligne correspondant à l'erreur qu'on veut utiliser dans le trigger.
Ensuite dans le corps du trigger, en cas de valeur erronée, on refait la même insertion. Cela déclenche une erreur de contrainte d'unicité, laquelle affiche le texte qu'on a essayé d'insérer dans _Erreur_.


```sql
-- Création de la table Erreur
CREATE TABLE Erreur (
    id TINYINT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
    erreur VARCHAR(255) UNIQUE);

-- Insertion de l'erreur qui nous intéresse
INSERT INTO Erreur (erreur) VALUES ('Erreur : sexe doit valoir "M", "F" ou NULL.');

-- Création du trigger
DELIMITER |
CREATE TRIGGER before_insert_animal BEFORE INSERT
ON Animal FOR EACH ROW
BEGIN
    IF NEW.sexe IS NOT NULL   -- le sexe n'est ni NULL
    AND NEW.sexe != 'M'       -- ni "M"
    AND NEW.sexe != 'F'       -- ni "F"
      THEN
        INSERT INTO Erreur (erreur) VALUES ('Erreur : sexe doit valoir "M", "F" ou NULL.');
    END IF;
END |
DELIMITER ;
```


**Test** :


```sql
INSERT INTO Animal (nom, sexe, date_naissance, espece_id)
VALUES ('Babar', 'A', '2011-08-04 12:34', 3);
```


```console
ERROR 1062 (23000): Duplicate entry 'Erreur : sexe doit valoir "M", "F" ou NULL.' for key 'erreur'
```


Et voilà, ce n'est pas parfait, mais au moins le message d'erreur permet de cerner d'où vient le problème. Et Babar n'a pas été inséré.


## Vérification du booléen dans _Adoption_

Il est important de savoir si un client a payé ou non pour les animaux qu'il veut adopter. Il faut donc vérifier la valeur de ce qu'on insère dans la colonne _paye_, et refuser toute insertion/modification donnant une valeur différente de `TRUE` (1) ou `FALSE` (0).
Les deux triggers à créer sont très similaires à ce que l'on a fait pour la colonne _sexe_ d'_Animal_. Essayez donc de construire les requêtes vous-mêmes.


```sql
INSERT INTO Erreur (erreur) VALUES ('Erreur : paye doit valoir TRUE (1) ou FALSE (0).');

DELIMITER |
CREATE TRIGGER before_insert_adoption BEFORE INSERT
ON Adoption FOR EACH ROW
BEGIN
    IF NEW.paye != TRUE       -- ni TRUE
    AND NEW.paye != FALSE     -- ni FALSE
      THEN
        INSERT INTO Erreur (erreur) VALUES ('Erreur : paye doit valoir TRUE (1) ou FALSE (0).');
    END IF;
END |

CREATE TRIGGER before_update_adoption BEFORE UPDATE
ON Adoption FOR EACH ROW
BEGIN
    IF NEW.paye != TRUE       -- ni TRUE
    AND NEW.paye != FALSE     -- ni FALSE
      THEN
        INSERT INTO Erreur (erreur) VALUES ('Erreur : paye doit valoir TRUE (1) ou FALSE (0).');
    END IF;
END |
DELIMITER ;
```


**Test** :


```sql
UPDATE Adoption 
SET paye = 3
WHERE client_id = 9;
```


```console
ERROR 1062 (23000): Duplicate entry 'Erreur : paye doit valoir TRUE (1) ou FALSE (0)' for key 'erreur'
```



## Vérification de la date d'adoption

Il reste une petite chose à vérifier, et ce sera tout pour les vérifications de données : la date d'adoption !
En effet, celle-ci doit être postérieure ou égale à la date de réservation. Un client ne peut pas emporter chez lui un animal avant même d'avoir prévenu qu'il voulait l'adopter.
À nouveau, essayez de faire le trigger vous-mêmes. Pour rappel, il ne peut exister qu'un seul trigger `BEFORE UPDATE` et un seul `BEFORE INSERT` pour chaque table.


```sql
INSERT INTO Erreur (erreur) VALUES ('Erreur : date_adoption doit être >= à date_reservation.');

DELIMITER |
DROP TRIGGER before_insert_adoption|
CREATE TRIGGER before_insert_adoption BEFORE INSERT
ON Adoption FOR EACH ROW
BEGIN
    IF NEW.paye != TRUE                                     -- On remet la vérification sur paye
    AND NEW.paye != FALSE     
      THEN
        INSERT INTO Erreur (erreur) VALUES ('Erreur : paye doit valoir TRUE (1) ou FALSE (0).');

    ELSEIF NEW.date_adoption < NEW.date_reservation THEN    -- Adoption avant réservation
        INSERT INTO Erreur (erreur) VALUES ('Erreur : date_adoption doit être >= à date_reservation.');
    END IF;
END |

DROP TRIGGER before_update_adoption|
CREATE TRIGGER before_update_adoption BEFORE UPDATE
ON Adoption FOR EACH ROW
BEGIN
    IF NEW.paye != TRUE                                     -- On remet la vérification sur paye
    AND NEW.paye != FALSE     
      THEN
        INSERT INTO Erreur (erreur) VALUES ('Erreur : paye doit valoir TRUE (1) ou FALSE (0).');

    ELSEIF NEW.date_adoption < NEW.date_reservation THEN    -- Adoption avant réservation
        INSERT INTO Erreur (erreur) VALUES ('Erreur : date_adoption doit être >= à date_reservation.');
    END IF;
END |
DELIMITER ;
```


On aurait pu faire un second `IF` au lieu d'un `ELSEIF`, mais de toute façon, le trigger ne pourra déclencher qu'une erreur à la fois.

**Test** :


```sql
INSERT INTO Adoption (animal_id, client_id, date_reservation, date_adoption, prix, paye)
VALUES (10, 10, NOW(), NOW() - INTERVAL 2 DAY, 200.00, 0);

INSERT INTO Adoption (animal_id, client_id, date_reservation, date_adoption, prix, paye)
VALUES (10, 10, NOW(), NOW(), 200.00, 4);
```


```console
ERROR 1062 (23000): Duplicate entry 'Erreur : date_adoption doit être >= à date_reservation.' for key 'erreur'

ERROR 1062 (23000): Duplicate entry 'Erreur : paye doit valoir TRUE (1) ou FALSE (0).' for key 'erreur'
```

Les deux vérifications fonctionnent !


# Mise à jour d'informations dépendant d'autres données

Pour l'instant, lorsque l'on a besoin de savoir quels animaux restent disponibles pour l'adoption, il faut faire une requête avec sous-requête.


```sql
SELECT id, nom, sexe, date_naissance, commentaires
FROM Animal
WHERE NOT EXISTS (
    SELECT * 
    FROM Adoption 
    WHERE Animal.id = Adoption.animal_id
);
```


Mais une telle requête n'est pas particulièrement performante, et elle est relativement peu facile à lire. Les triggers peuvent nous permettre de stocker automatiquement une donnée permettant de savoir immédiatement si un animal est disponible ou non.

Pour cela, il suffit d'ajouter une colonne _disponible_ à la table _Animal_, qui vaudra `FALSE` ou `TRUE`, et qui sera mise à jour grâce à trois triggers sur la table _Adoption_.


- À l'insertion d'une nouvelle adoption, il faut retirer l'animal adopté des animaux disponibles ;
- en cas de suppression, il faut faire le contraire ;
- en cas de modification d'une adoption, si l'animal adopté change, il faut remettre l'ancien parmi les animaux disponibles et retirer le nouveau.



```sql
-- Ajout de la colonne disponible
ALTER TABLE Animal ADD COLUMN disponible BOOLEAN DEFAULT TRUE;  -- À l'insertion, un animal est forcément disponible

-- Remplissage de la colonne
UPDATE Animal
SET disponible = FALSE
WHERE EXISTS (
    SELECT * 
    FROM Adoption 
    WHERE Animal.id = Adoption.animal_id
);

-- Création des trois triggers
DELIMITER |
CREATE TRIGGER after_insert_adoption AFTER INSERT
ON Adoption FOR EACH ROW
BEGIN
    UPDATE Animal
    SET disponible = FALSE
    WHERE id = NEW.animal_id;
END |

CREATE TRIGGER after_delete_adoption AFTER DELETE
ON Adoption FOR EACH ROW
BEGIN
    UPDATE Animal
    SET disponible = TRUE
    WHERE id = OLD.animal_id;
END |

CREATE TRIGGER after_update_adoption AFTER UPDATE
ON Adoption FOR EACH ROW
BEGIN
    IF OLD.animal_id <> NEW.animal_id THEN
        UPDATE Animal
        SET disponible = TRUE
        WHERE id = OLD.animal_id;

        UPDATE Animal
        SET disponible = FALSE
        WHERE id = NEW.animal_id;
    END IF;
END |
DELIMITER ;
```


**Test** :


```sql
SELECT animal_id, nom, sexe, disponible, client_id
FROM Animal
INNER JOIN Adoption ON Adoption.animal_id = Animal.id
WHERE client_id = 9;
```




+---------+-------+----+----------+---------+
|animal_id|nom    |sexe|disponible|client_id|
+=========+=======+====+==========+=========+
|33       |Caribou|M   |0         |9        |
+---------+-------+----+----------+---------+
|54       |Bubulle|M   |0         |9        |
+---------+-------+----+----------+---------+
|55       |Relou  |M   |0         |9        |
+---------+-------+----+----------+---------+





```sql
DELETE FROM Adoption                                                -- 54 doit redevenir disponible
WHERE animal_id = 54;

UPDATE Adoption
SET animal_id = 38, prix = 985.00                                   -- 38 doit devenir indisponible
WHERE animal_id = 33;                                               -- et 33 redevenir disponible

INSERT INTO Adoption (client_id, animal_id, date_reservation, prix, paye)
VALUES (9, 59, NOW(), 700.00, FALSE);                                       -- 59 doit devenir indisponible

SELECT Animal.id AS animal_id, nom, sexe, disponible, client_id
FROM Animal
LEFT JOIN Adoption ON Animal.id = Adoption.animal_id
WHERE Animal.id IN (33, 54, 55, 38, 59);
```




+---------+-------+----+----------+---------+
|animal_id|nom    |sexe|disponible|client_id|
+=========+=======+====+==========+=========+
|33       |Caribou|M   |1         |NULL     |
+---------+-------+----+----------+---------+
|38       |Boule  |F   |0         |9        |
+---------+-------+----+----------+---------+
|54       |Bubulle|M   |1         |NULL     |
+---------+-------+----+----------+---------+
|55       |Relou  |M   |0         |9        |
+---------+-------+----+----------+---------+
|59       |Bavard |M   |0         |9        |
+---------+-------+----+----------+---------+





Désormais, pour savoir quels animaux sont disponibles, il suffira de faire la requête suivante :


```sql
SELECT *
FROM Animal
WHERE disponible = TRUE;

-- Ou même

SELECT *
FROM Animal
WHERE disponible;
```



# Historisation

Voici deux exemples de systèmes d'historisation :


- l'un très basique, gardant simplement trace de l'insertion (date et utilisateur) et de la dernière modification (date et utilisateur), et se faisant directement dans la table concernée ;
- l'autre plus complet, qui garde une copie de chaque version antérieure des lignes dans une table dédiée, ainsi qu'une copie de la dernière version en cas de suppression.



## Historisation basique

On va utiliser cette historisation pour la table _Race_. Libre à vous d'adapter ou de créer les triggers d'autres tables pour les historiser également de cette manière.

On ajoute donc quatre colonnes à la table. Ces colonnes seront toujours remplies automatiquement par les triggers.


```sql
-- On modifie la table Race
ALTER TABLE Race ADD COLUMN date_insertion DATETIME,                 -- date d'insertion
                 ADD COLUMN utilisateur_insertion VARCHAR(20),       -- utilisateur ayant inséré la ligne
                 ADD COLUMN date_modification DATETIME,              -- date de dernière modification
                 ADD COLUMN utilisateur_modification VARCHAR(20);    -- utilisateur ayant fait la dernière modification

-- On remplit les colonnes
UPDATE Race 
SET date_insertion = NOW() - INTERVAL 1 DAY, 
    utilisateur_insertion = 'Test', 
    date_modification = NOW()- INTERVAL 1 DAY, 
    utilisateur_modification = 'Test';
```


J'ai mis artificiellement les dates d'insertion et de dernière modification à la veille d'aujourd'hui, et les utilisateurs pour l'insertion et la modification à "Test", afin d'avoir des données intéressantes lors des tests. Idéalement, ce type d'historisation doit bien sûr être mis en place dès la création de la table.

Occupons-nous maintenant des triggers. Il en faut sur l'insertion et sur la modification.


```sql
DELIMITER |
CREATE TRIGGER before_insert_race BEFORE INSERT
ON Race FOR EACH ROW
BEGIN
    SET NEW.date_insertion = NOW();
    SET NEW.utilisateur_insertion = CURRENT_USER();
    SET NEW.date_modification = NOW();
    SET NEW.utilisateur_modification = CURRENT_USER();
END |

CREATE TRIGGER before_update_race BEFORE UPDATE
ON Race FOR EACH ROW
BEGIN
    SET NEW.date_modification = NOW();
    SET NEW.utilisateur_modification = CURRENT_USER();
END |
DELIMITER ;
```


Les triggers sont très simples : ils mettent simplement à jour les colonnes d'historisation nécessaires ; ils doivent donc nécessairement être `BEFORE`.

**Test** :


```sql
INSERT INTO Race (nom, description, espece_id, prix)
VALUES ('Yorkshire terrier', 'Chien de petite taille au pelage long et soyeux de couleur bleu et feu.', 1, 700.00);

UPDATE Race 
SET prix = 630.00 
WHERE nom = 'Rottweiller' AND espece_id = 1;

SELECT nom, DATE(date_insertion) AS date_ins, utilisateur_insertion AS utilisateur_ins, DATE(date_modification) AS date_mod, utilisateur_modification AS utilisateur_mod 
FROM Race 
WHERE espece_id = 1;
```




+-------------------+----------+---------------+----------+---------------+
|nom                |date_ins  |utilisateur_ins|date_mod  |utilisateur_mod|
+===================+==========+===============+==========+===============+
|Berger allemand    |2012-05-02|Test           |2012-05-02|Test           |
+-------------------+----------+---------------+----------+---------------+
|Berger blanc suisse|2012-05-02|Test           |2012-05-02|Test           |
+-------------------+----------+---------------+----------+---------------+
|Rottweiller        |2012-05-02|Test           |2012-05-03|sdz@localhost  |
+-------------------+----------+---------------+----------+---------------+
|Yorkshire terrier  |2012-05-03|sdz@localhost  |2012-05-03|sdz@localhost  |
+-------------------+----------+---------------+----------+---------------+






## Historisation complète

Nous allons mettre en place un système d'historisation complet pour la table _Animal_. Celle-ci ne change pas et contiendra la dernière version des données. Par contre, on va ajouter une table _Animal_histo_, qui contiendra les versions antérieures (quand il y en a) des données d'_Animal_.


```sql
CREATE TABLE Animal_histo (
  id SMALLINT(6) UNSIGNED NOT NULL,                -- Colonnes historisées
  sexe CHAR(1),
  date_naissance DATETIME NOT NULL,
  nom VARCHAR(30),
  commentaires TEXT,
  espece_id SMALLINT(6) UNSIGNED NOT NULL,
  race_id SMALLINT(6) UNSIGNED DEFAULT NULL,
  mere_id SMALLINT(6) UNSIGNED DEFAULT NULL,
  pere_id SMALLINT(6) UNSIGNED DEFAULT NULL,
  disponible BOOLEAN DEFAULT TRUE,

  date_histo DATETIME NOT NULL,                   -- Colonnes techniques
  utilisateur_histo VARCHAR(20) NOT NULL,
  evenement_histo CHAR(6) NOT NULL,
  PRIMARY KEY (id, date_histo)
) ENGINE=InnoDB;
```


Les colonnes _date_histo_ et _utilisateur_histo_ contiendront bien sûr la date à laquelle la ligne a été historisée, et l'utilisateur qui a provoqué cette historisation. Quant à la colonne _evenement_histo_, elle contiendra l'événement qui a déclenché le trigger (soit "DELETE", soit "UPDATE"). La clé primaire de cette table est le couple (_id_, _date_histo_).

Voici les triggers nécessaires. Cette fois, ils pourraient être soit `BEFORE`, soit `AFTER`. Cependant, aucun traitement ne concerne les nouvelles valeurs de la ligne modifiée (ni, a fortiori, de la ligne supprimée). Par conséquent, autant utiliser `AFTER`, cela évitera d'exécuter les instructions du trigger en cas d'erreur lors de la requête déclenchant celui-ci.


```sql
DELIMITER |
CREATE TRIGGER after_update_animal AFTER UPDATE
ON Animal FOR EACH ROW
BEGIN
    INSERT INTO Animal_histo (
        id, 
        sexe, 
        date_naissance, 
        nom, 
        commentaires, 
        espece_id, 
        race_id, 
        mere_id, 
        pere_id, 
        disponible,

        date_histo, 
        utilisateur_histo, 
        evenement_histo)
    VALUES (
        OLD.id,
        OLD.sexe,
        OLD.date_naissance,
        OLD.nom,
        OLD.commentaires,
        OLD.espece_id,
        OLD.race_id,
        OLD.mere_id,
        OLD.pere_id,
        OLD.disponible,

        NOW(),
        CURRENT_USER(),
        'UPDATE');
END |

CREATE TRIGGER after_delete_animal AFTER DELETE
ON Animal FOR EACH ROW
BEGIN
    INSERT INTO Animal_histo (
        id, 
        sexe, 
        date_naissance, 
        nom, 
        commentaires, 
        espece_id, 
        race_id, 
        mere_id, 
        pere_id, 
        disponible,

        date_histo, 
        utilisateur_histo, 
        evenement_histo)
    VALUES (
        OLD.id,
        OLD.sexe,
        OLD.date_naissance,
        OLD.nom,
        OLD.commentaires,
        OLD.espece_id,
        OLD.race_id,
        OLD.mere_id,
        OLD.pere_id,
        OLD.disponible,

        NOW(),
        CURRENT_USER(),
        'DELETE');
END |
DELIMITER ;
```


Cette fois, ce sont les valeurs avant modification/suppression qui nous intéressent, d'où l'utilisation de `OLD`.

**Test** :


```sql
UPDATE Animal
SET commentaires = 'Petit pour son âge'
WHERE id = 10;

DELETE FROM Animal
WHERE id = 47;

SELECT id, sexe, date_naissance, nom, commentaires, espece_id
FROM Animal 
WHERE id IN (10, 47);

SELECT id, nom, date_histo, utilisateur_histo, evenement_histo 
FROM Animal_histo;
```




+--+----+-------------------+----+------------------+---------+
|id|sexe|date_naissance     |nom |commentaires      |espece_id|
+==+====+===================+====+==================+=========+
|10|M   |2010-07-21 15:41:00|Bobo|Petit pour son âge|1        |
+--+----+-------------------+----+------------------+---------+







+--+-------+-------------------+-----------------+---------------+
|id|nom    |date_histo         |utilisateur_histo|evenement_histo|
+==+=======+===================+=================+===============+
|10|Bobo   |2012-05-03 21:51:12|sdz@localhost    |UPDATE         |
+--+-------+-------------------+-----------------+---------------+
|47|Scroupy|2012-05-03 21:51:12|sdz@localhost    |DELETE         |
+--+-------+-------------------+-----------------+---------------+






## Quelques remarques sur l'historisation

Les deux systèmes d'historisation montrés dans ce cours ne sont que deux possibilités parmi des dizaines. Si vous pensez avoir besoin d'un système de ce type, prenez le temps de réfléchir, et de vous renseigner sur les diverses possibilités qui s'offrent à vous.
Dans certains systèmes, on combine les deux historisations que j'ai présentées. 
Parfois, on ne conserve pas les lignes supprimées dans la table d'historisation, mais on utilise plutôt un système d'archive, séparé de l'historisation.
Au-delà du modèle d'historisation que vous choisirez, les détails sont également modifiables. Voulez-vous garder toutes les versions des données, ou les garder seulement pour une certaine période de temps ? Voulez-vous enregistrer l'utilisateur SQL ou plutôt des utilisateurs créés pour votre application, découplés des utilisateurs SQL ?
Ne restez pas bloqués sur les exemples montrés dans ce cours (que ce soit pour l'historisation ou le reste), le monde est vaste !

*[SGBD]: Système de Gestion de Base de Données