Les restrictions sur les triggers sont malheureusement trop importantes pour qu'on puisse se permettre de ne pas les mentionner. On peut espérer qu'une partie de ces restrictions soit levée dans une prochaine version de MySQL, mais en attendant, il est nécessaire d'avoir celles-ci en tête.
Voici donc les principales.


## Commandes interdites

**Il est impossible de travailler avec des transactions à l'intérieur d'un trigger.** 
Cette restriction est nécessaire, puisque la requête ayant provoqué l'exécution du trigger pourrait très bien se trouver elle-même à l'intérieur d'une transaction. Auquel cas, toute commande `START TRANSACTION`, `COMMIT` ou `ROLLBACK` interagirait avec cette transaction, de manière intempestive.

**Les requêtes préparées ne peuvent pas non plus être utilisées.**

Enfin, on ne peut pas appeler n'importe quelle procédure à partir d'un trigger. 


- Les procédures appelées par un trigger **ne peuvent pas envoyer d'informations au client MySQL**. Par exemple, elles ne peuvent pas exécuter un simple `SELECT`, qui produit un affichage dans le client (un `SELECT...INTO` par contre est permis). Elles peuvent toutefois renvoyer des informations au trigger grâce à des paramètres `OUT` ou `INOUT`.
- Les procédures appelées ne peuvent utiliser ni les transactions (`START TRANSACTION`, `COMMIT` ou `ROLLBACK`) ni les requêtes préparées. C'est-à-dire qu'**elles doivent respecter les restrictions des triggers**.




## Tables utilisées par la requête

Comme mentionné auparavant, il est **impossible de modifier les données d'une table utilisée par la requête ayant déclenché le trigger** à l'intérieur de celui-ci.

Cette restriction est importante, et peut remettre en question l'utilisation de certains triggers. 

**Exemple** : le trigger `AFTER INSERT ON Adoption` modifie les données de la table _Animal_. Si l'on exécute la requête suivante, cela posera problème.


```sql
INSERT INTO Adoption (animal_id, client_id, date_reservation, prix, paye)
SELECT Animal.id, 4, NOW(), COALESCE(Race.prix, Espece.prix), FALSE 
FROM Animal
INNER JOIN Espece ON Espece.id = Animal.espece_id
LEFT JOIN Race ON Race.id = Animal.race_id
WHERE Animal.nom = 'Boucan' AND Animal.espece_id = 2;
```


```console
ERROR 1442 (HY000): Can't update table 'animal' in stored function/trigger because it is already used by statement which invoked this stored function/trigger.
```

Le trigger échoue puisque la table _Animal_ est utilisée par la requête `INSERT` qui le déclenche. L'insertion elle-même est donc finalement annulée.


## Clés étrangères

**Une suppression ou modification de données déclenchée par une clé étrangère ne provoquera pas l'exécution du trigger correspondant.**
Par exemple, la colonne _Animal.race_id_ possède une clé étrangère, qui référence la colonne _Race.id_. Cette clé étrangère a été définie avec l'option `ON DELETE SET NULL`. Donc en cas de suppression d'une race, tous les animaux de cette race seront modifiés, et leur _race_id_ changée en `NULL`. Il s'agit donc d'une modification de données. Mais comme cette modification a été déclenchée par une contrainte de clé étrangère, les éventuels triggers `BEFORE UPDATE` et `AFTER UPDATE` de la table _Animal_ ne seront pas déclenchés.

En cas d'utilisation de triggers sur des tables présentant des clés étrangères avec ces options, il vaut donc mieux supprimer celles-ci et déplacer ce comportement dans des triggers. 
Une autre solution est de ne pas utiliser les triggers sur les tables concernées. Vous pouvez alors remplacer les triggers par l'utilisation de procédures stockées et/ou de transactions.


[[question]]
| Qu'avons-nous comme clés étrangères dans nos tables ?



- _Race_ : `CONSTRAINT fk_race_espece_id FOREIGN KEY (espece_id) REFERENCES Espece (id) ON DELETE CASCADE;`
- _Animal_ : `CONSTRAINT fk_race_id FOREIGN KEY (race_id) REFERENCES Race (id) ON DELETE SET NULL;`
- _Animal_ : `CONSTRAINT fk_espece_id FOREIGN KEY (espece_id) REFERENCES Espece (id);`
- _Animal_ : `CONSTRAINT fk_mere_id FOREIGN KEY (mere_id) REFERENCES Animal (id) ON DELETE SET NULL;`
- _Animal_ : `CONSTRAINT fk_pere_id FOREIGN KEY (pere_id) REFERENCES Animal (id) ON DELETE SET NULL;`


Quatre d'entre elles pourraient donc poser problème. Quatre, sur cinq ! Ce n'est donc pas anodin comme restriction !

On va donc modifier nos clés étrangères pour qu'elles reprennent leur comportement par défaut. Il faudra ensuite créer (ou recréer) quelques triggers pour reproduire le comportement que l'on avait défini.
À ceci près que la restriction sur la modification des données d'une table utilisée par l'événement déclencheur fait qu'on ne pourra pas reproduire certains comportements. On ne pourra pas mettre à `NULL` les colonnes _pere_id_ et _mere_id_ de la table _Animal_ en cas de suppression de l'animal de référence.

Voici les commandes :


```sql
-- On supprime les clés
ALTER TABLE Race DROP FOREIGN KEY fk_race_espece_id;
ALTER TABLE Animal DROP FOREIGN KEY fk_race_id,
                   DROP FOREIGN KEY fk_mere_id,
                   DROP FOREIGN KEY fk_pere_id;

-- On les recrée sans option
ALTER TABLE Race ADD CONSTRAINT fk_race_espece_id FOREIGN KEY (espece_id) REFERENCES Espece (id);
ALTER TABLE Animal ADD CONSTRAINT fk_race_id FOREIGN KEY (race_id) REFERENCES Race (id),
                   ADD CONSTRAINT fk_mere_id FOREIGN KEY (mere_id) REFERENCES Animal (id),
                   ADD CONSTRAINT fk_pere_id FOREIGN KEY (pere_id) REFERENCES Animal (id);

-- Trigger sur Race
DELIMITER |
CREATE TRIGGER before_delete_race BEFORE DELETE
ON Race FOR EACH ROW
BEGIN
    UPDATE Animal 
    SET race_id = NULL
    WHERE race_id = OLD.id;
END|

-- Trigger sur Espece
CREATE TRIGGER before_delete_espece BEFORE DELETE
ON Espece FOR EACH ROW
BEGIN
    DELETE FROM Race
    WHERE espece_id = OLD.id;
END |
DELIMITER ;
```