# Le principe
 
Pour notre élevage, la requête suivante est très utile.


```sql
SELECT Animal.id, Animal.sexe, Animal.date_naissance, Animal.nom, Animal.commentaires, 
       Animal.espece_id, Animal.race_id, Animal.mere_id, Animal.pere_id, Animal.disponible,
       Espece.nom_courant AS espece_nom, Race.nom AS race_nom
FROM Animal
INNER JOIN Espece ON Animal.espece_id = Espece.id
LEFT JOIN Race ON Animal.race_id = Race.id;
```


Avec ou sans clause `WHERE`, il arrive régulièrement qu'on veuille trouver des renseignements sur nos animaux, y compris leur race et leur espèce (et, seul, l'_id_ contenu dans _Animal_ n'est pas une information très explicite). Il serait donc bien pratique de pouvoir stocker cette requête plutôt que de devoir la retaper en entier à chaque fois.

C'est très exactement le principe d'une vue : **on stocke une requête `SELECT` en lui donnant un nom**, et on peut ensuite appeler directement la vue par son nom. 

Quelques remarques importantes :


- Il s'agit bien d'objets de la base de données, **stockés de manière durable**, comme le sont les tables ou les procédures stockées.
- C'est donc bien différent des requêtes préparées, qui ne sont définies que le temps d'une session, et qui ont un tout autre but.
- Ce qui est stocké est la **requête**, et non pas les résultats de celle-ci. **On ne gagne absolument rien en terme de performance** en utilisant une vue plutôt qu'en faisant une requête directement sur les tables.



# Création

Pour créer une vue, on utilise tout simplement la commande `CREATE VIEW`, dont voici la syntaxe :


```sql
CREATE [OR REPLACE] VIEW nom_vue
AS requete_select;
```


La clause `OR REPLACE` est facultative. Si elle est fournie, la vue _nom_vue_ sera soit créée si elle n'existe pas, soit remplacée si elle existait déjà. Si `OR REPLACE` est omise et qu'une vue portant le même nom a été précédemment définie, cela déclenchera une erreur.

Donc, si l'on reprend la requête précédente, voici comment créer une vue pour stocker celle-ci :


```sql
CREATE VIEW V_Animal_details
AS SELECT Animal.id, Animal.sexe, Animal.date_naissance, Animal.nom, Animal.commentaires, 
       Animal.espece_id, Animal.race_id, Animal.mere_id, Animal.pere_id, Animal.disponible,
       Espece.nom_courant AS espece_nom, Race.nom AS race_nom
FROM Animal
INNER JOIN Espece ON Animal.espece_id = Espece.id
LEFT JOIN Race ON Animal.race_id = Race.id;
```


Dorénavant, plus besoin de retaper cette requête, il suffit de travailler à partir de la vue, comme s'il s'agissait d'une table :


```sql
SELECT * FROM V_Animal_details;
```



[[attention]]
| J'ai préfixé le nom de la vue par "V_". Il s'agit d'une convention que je vous conseille fortement de respecter. Cela permet de savoir au premier coup d’œil si l'on travaille avec une vraie table, ou avec une vue.


D'ailleurs, si l'on demande la liste des tables de la base de données, on peut voir que la vue est reprise (bien qu'il ne s'agit pas d'une table, mais d'une requête `SELECT` stockée, j'insiste).


```sql
SHOW TABLES;
```




+-----------------+
|Tables_in_elevage|
+=================+
|Adoption         |
+-----------------+
|Animal           |
+-----------------+
|Animal_histo     |
+-----------------+
|Client           |
+-----------------+
|Erreur           |
+-----------------+
|Espece           |
+-----------------+
|Race             |
+-----------------+
|V_Animal_details |
+-----------------+






# Les colonnes de la vue

Comme pour les tables, on peut utiliser la commande `DESCRIBE` pour voir les différentes colonnes de notre vue.


```sql
DESCRIBE V_Animal_details;
```




+--------------+--------------------+----+---+-------+-----+
|Field         |Type                |Null|Key|Default|Extra|
+==============+====================+====+===+=======+=====+
|id            |smallint(6) unsigned|NO  |   |0      |     |
+--------------+--------------------+----+---+-------+-----+
|sexe          |char(1)             |YES |   |NULL   |     |
+--------------+--------------------+----+---+-------+-----+
|date_naissance|datetime            |NO  |   |NULL   |     |
+--------------+--------------------+----+---+-------+-----+
|nom           |varchar(30)         |YES |   |NULL   |     |
+--------------+--------------------+----+---+-------+-----+
|commentaires  |text                |YES |   |NULL   |     |
+--------------+--------------------+----+---+-------+-----+
|espece_id     |smallint(6) unsigned|NO  |   |NULL   |     |
+--------------+--------------------+----+---+-------+-----+
|race_id       |smallint(6) unsigned|YES |   |NULL   |     |
+--------------+--------------------+----+---+-------+-----+
|mere_id       |smallint(6) unsigned|YES |   |NULL   |     |
+--------------+--------------------+----+---+-------+-----+
|pere_id       |smallint(6) unsigned|YES |   |NULL   |     |
+--------------+--------------------+----+---+-------+-----+
|disponible    |tinyint(1)          |YES |   |1      |     |
+--------------+--------------------+----+---+-------+-----+
|espece_nom    |varchar(40)         |NO  |   |NULL   |     |
+--------------+--------------------+----+---+-------+-----+
|race_nom      |varchar(40)         |YES |   |NULL   |     |
+--------------+--------------------+----+---+-------+-----+





Comme on pouvait s'y attendre, les noms de colonnes ont été déterminés par la clause `SELECT` de la requête définissant la vue. S'il n'y avait pas d'alias pour la colonne, c'est simplement le même nom que dans la table d'origine (_date_naissance_ par exemple), et si un alias a été donné, il est utilisé (_espece_nom_ par exemple).


## Lister les colonnes dans CREATE VIEW

Il existe une autre possibilité que les alias dans la clause `SELECT` pour nommer les colonnes d'une vue. Il suffit de lister les colonnes juste après le nom de la vue, lors de la création de celle-ci. La commande suivante est l'équivalent de la précédente pour la création de _V_Animal_details_, mais cette fois sans alias dans la requête `SELECT`.


```sql
CREATE VIEW V_Animal_details  (id, sexe, date_naissance, nom, commentaires, espece_id, race_id, mere_id, pere_id, disponible, espece_nom, race_nom)
AS SELECT Animal.id, Animal.sexe, Animal.date_naissance, Animal.nom, Animal.commentaires, 
       Animal.espece_id, Animal.race_id, Animal.mere_id, Animal.pere_id, Animal.disponible,
       Espece.nom_courant, Race.nom
FROM Animal
INNER JOIN Espece ON Animal.espece_id = Espece.id
LEFT JOIN Race ON Animal.race_id = Race.id;
```



[[attention]]
| Il faut toujours vérifier que les colonnes sélectionnées sont dans le bon ordre par rapport à la liste des noms, car la correspondance se fait uniquement sur la base de la position. MySQL ne lèvera pas le petit doigt si l'on nomme "espece" une colonne contenant le sexe.



## Doublons interdits

Comme pour une table, il est impossible que deux colonnes ayant le même nom cohabitent dans une même vue.


```sql
CREATE VIEW V_test 
AS SELECT Animal.id, Espece.id 
FROM Animal
INNER JOIN Espece ON Animal.espece_id = Espece.id;
```



```console
ERROR 1060 (42S21): Duplicate column name 'id'
```


Pour pouvoir créer cette vue, il est nécessaire de renommer une des deux colonnes dans la vue (soit avec un alias, soit en listant les colonnes juste après `CREATE VIEW nom_vue`).


# Requête SELECT stockée dans la vue


[[question]]
| Que peut-on mettre dans la requête `SELECT` qui sert à définir la vue ?
 
La requête définissant une vue peut être n'importe quelle requête `SELECT`, à quelques exceptions près.


- Il n'est pas possible d'utiliser une requête `SELECT` dont la clause `FROM` contient une sous-requête `SELECT`.
- La requête ne peut pas faire référence à des variables utilisateur, des variables système, ni même des variables locales (dans le cas d'une vue définie par une procédure stockée).
- Toutes les tables (ou vues) mentionnées dans la requête doivent exister (au moment de la création du moins).


En dehors de ça, carte blanche ! La requête peut contenir une clause `WHERE`, une clause `GROUP BY`, des fonctions (scalaires ou d’agrégation), des opérations mathématiques, une autre vue, des jointures, etc.

**Exemple 1** : une vue pour les chiens.

L'employé de l'élevage préposé aux chiens peut créer une vue sur ceux-ci, afin de ne pas devoir ajouter une clause `WHERE espece_id = 1` à chacune de ses requêtes.


```sql
CREATE VIEW V_Chien
AS SELECT id, sexe, date_naissance, nom, commentaires, espece_id, race_id, mere_id, pere_id, disponible
FROM Animal
WHERE espece_id = 1;
```


**Exemple 2** : combien de chats possédons-nous ?

Il est tout à fait possible de créer une vue définie par une requête qui compte le nombre d'animaux de chaque espèce que l'on possède.


```sql
CREATE OR REPLACE VIEW V_Nombre_espece
AS SELECT Espece.id, COUNT(Animal.id) AS nb
FROM Espece
LEFT JOIN Animal ON Animal.espece_id = Espece.id
GROUP BY Espece.id;
```


**Exemple 3** : vue sur une vue.

La vue _V_Chien_ sélectionne les chiens. Créons la vue _V_Chien_race_ qui sélectionne les chiens dont on connaît la race. On peut bien sûr créer cette vue à partir de la table _Animal_, mais on peut aussi repartir simplement de la vue _V_Chien_.


```sql
CREATE OR REPLACE VIEW V_Chien_race
AS SELECT id, sexe, date_naissance, nom, commentaires, espece_id, race_id, mere_id, pere_id, disponible
FROM V_Chien
WHERE race_id IS NOT NULL;
```


**Exemple 4** : expression dans une vue.

Voici un exemple de vue définie par une requête contenant une expression, qui sélectionne les espèces, avec leur prix en dollars.


```sql
CREATE VIEW V_Espece_dollars
AS SELECT id, nom_courant, nom_latin, description, ROUND(prix*1.31564, 2) AS prix_dollars
FROM Espece;
```



## La requête SELECT est "figée"


La requête `SELECT` définissant une vue est figée : les changements de structure faits par la suite sur la ou les tables sous-jacentes n'influent pas sur la vue.

Par exemple, si l'on crée une vue _V_Client_ toute simple.


```sql
CREATE VIEW V_Client
AS SELECT * 
FROM Client;
```


Cette vue sélectionne les huit colonnes de _Client_ (_id, nom, prenom, adresse, code_postal, ville, pays, email_). Une fois cette vue créée, elle est figée, et l'ajout d'une colonne dans la table _Client_ **ne changera pas** les résultats d'une requête sur _V_Client_. Cette vue ne sélectionnera jamais que _id, nom, prenom, adresse, code_postal, ville, pays_ et _email_, malgré le caractère * dans la clause `SELECT`. Pour que _V_Client_ sélectionne la nouvelle colonne de _Client_, il faudrait recréer la vue pour l'inclure.

**Exemple** : ajout d'une colonne _date_naissance_ à la table _Client_.


```sql
ALTER TABLE Client ADD COLUMN date_naissance DATE;

DESCRIBE V_Client;
```




+-----------+--------------------+----+---+-------+-----+
|Field      |Type                |Null|Key|Default|Extra|
+===========+====================+====+===+=======+=====+
|id         |smallint(5) unsigned|NO  |   |0      |     |
+-----------+--------------------+----+---+-------+-----+
|nom        |varchar(100)        |NO  |   |NULL   |     |
+-----------+--------------------+----+---+-------+-----+
|prenom     |varchar(60)         |NO  |   |NULL   |     |
+-----------+--------------------+----+---+-------+-----+
|adresse    |varchar(200)        |YES |   |NULL   |     |
+-----------+--------------------+----+---+-------+-----+
|code_postal|varchar(6)          |YES |   |NULL   |     |
+-----------+--------------------+----+---+-------+-----+
|ville      |varchar(60)         |YES |   |NULL   |     |
+-----------+--------------------+----+---+-------+-----+
|pays       |varchar(60)         |YES |   |NULL   |     |
+-----------+--------------------+----+---+-------+-----+
|email      |varbinary(100)      |YES |   |NULL   |     |
+-----------+--------------------+----+---+-------+-----+






## Tri des données directement dans la vue

Si l'on met un `ORDER BY` dans la requête définissant une vue, celui-ci prendra effet, sauf si l'on fait une requête sur la vue avec un `ORDER BY`. Dans ce cas, c'est ce dernier qui prime, et l'`ORDER BY` originel est ignoré.

**Exemple**


```sql
CREATE OR REPLACE VIEW V_Race
AS SELECT Race.id, nom, Espece.nom_courant AS espece
FROM Race
INNER JOIN Espece ON Espece.id = Race.espece_id
ORDER BY nom;

SELECT * 
FROM V_Race;     -- Sélection sans ORDER BY, on prend l'ORDER BY de la définition

SELECT * 
FROM V_Race
ORDER BY espece; -- Sélection avec ORDER BY, c'est celui-là qui sera pris en compte
```


La première requête donnera bien les races par ordre alphabétique, les races de chiens et les races de chats sont mélangées. Par contre, dans la deuxième, on a d'abord toutes les races de chats, puis toutes les races de chiens.


## Comportement d'autres clauses de SELECT

En ce qui concerne notamment les clauses `LIMIT` et `DISTINCT` (et son opposé, `ALL`), le comportement est indéfini. Dans le cas où l'on fait une requête avec un `LIMIT` sur une vue dont la requête possède aussi un `LIMIT`, on ne sait pas quelle clause, de la définition ou de la sélection, sera appliquée. Par conséquent, il est déconseillé d'inclure ces clauses dans la définition d'une vue.