Lors de la création d'une vue, on peut définir quel algorithme sera utilisé par MySQL lors d'une sélection sur celle-ci.


```sql
CREATE [OR REPLACE]
    [ALGORITHM = {UNDEFINED | MERGE | TEMPTABLE}]
    VIEW nom_vue
    AS requete_select
```


Il s'agit d'une **clause non standard**, donc valable uniquement pour MySQL. Deux algorithmes différents peuvent être utilisés : `MERGE` et `TEMPTABLE`.

Les algorithmes interviennent au moment où l'on **sélectionne des données de la vue**, et pas directement à la création de celle-ci.


# MERGE

Si l'algorithme `MERGE` (fusion en anglais) a été choisi, lorsque l'on sélectionne des données de la vue, MySQL va **fusionner la requête `SELECT` qui définit la vue avec les clauses de sélections**. Faire une sélection sur une vue qui utilise cet algorithme revient donc à faire une requête directement sur les tables sous-jacentes.

Comme ce n'est pas nécessairement très clair, voici deux exemples.

**Exemple 1**

On a créé la vue _V_Animal_details_ à partir de la requête suivante :


```sql
SELECT Animal.id, Animal.sexe, Animal.date_naissance, Animal.nom, Animal.commentaires, 
       Animal.espece_id, Animal.race_id, Animal.mere_id, Animal.pere_id, Animal.disponible,
       Espece.nom_courant AS espece_nom, Race.nom AS race_nom
FROM Animal
INNER JOIN Espece ON Animal.espece_id = Espece.id
LEFT JOIN Race ON Animal.race_id = Race.id;
```


Ensuite, on fait une sélection sur cette vue :


```sql
SELECT * 
FROM V_Animal_details
WHERE MONTH(date_naissance) = 6;
```


Si c'est l'algorithme `MERGE` qui est utilisé, MySQL va fusionner :



- la requête définissant la vue : 

```sql
SELECT Animal.id, Animal.sexe, Animal.date_naissance, Animal.nom, Animal.commentaires, 
       Animal.espece_id, Animal.race_id, Animal.mere_id, Animal.pere_id, Animal.disponible,
       Espece.nom_courant AS espece_nom, Race.nom AS race_nom
FROM Animal
INNER JOIN Espece ON Animal.espece_id = Espece.id
LEFT JOIN Race ON Animal.race_id = Race.id;
```

- les clauses de la requête de sélection : 

```sql
WHERE MONTH(date_naissance) = 6;
```




Au final, faire cette requête sur la vue provoquera l'exécution de la requête suivante :


```sql
SELECT Animal.id, Animal.sexe, Animal.date_naissance, Animal.nom, Animal.commentaires, 
       Animal.espece_id, Animal.race_id, Animal.mere_id, Animal.pere_id, Animal.disponible,
       Espece.nom_courant AS espece_nom, Race.nom AS race_nom
FROM Animal
INNER JOIN Espece ON Animal.espece_id = Espece.id
LEFT JOIN Race ON Animal.race_id = Race.id
WHERE MONTH(date_naissance) = 6;
```


**Exemple 2**

Si l'on exécute la requête suivante sur la vue _V_Chien_, et que l'algorithme `MERGE` est utilisé :


```sql
SELECT nom, date_naissance 
FROM V_Chien
WHERE pere_id IS NOT NULL;
```


MySQL va fusionner les deux requêtes, et exécuter ceci :


```sql
SELECT nom, date_naissance
FROM Animal
WHERE espece_id = 1
AND pere_id IS NOT NULL;
```



# TEMPTABLE

L'algorithme `TEMPTABLE`, par contre, **crée une table temporaire** contenant les résultats de la requête définissant la vue puis, par la suite, exécute la requête de sélection sur cette table temporaire.

Donc, si l'on exécute la requête suivante sur la vue _V_Chien_ :


```sql
SELECT nom, date_naissance 
FROM V_Chien
WHERE pere_id IS NOT NULL;
```


Avec l'algorithme `TEMPTABLE`, la requête définissant la vue va être exécutée et ses résultats stockés dans une table temporaire.


```sql
SELECT id, sexe, date_naissance, nom, commentaires, espece_id, race_id, mere_id, pere_id, disponible
FROM Animal
WHERE espece_id = 1;
```


Ensuite, sur cette table temporaire, va être exécutée la requête finale :


```sql
SELECT nom, date_naissance 
FROM table_temporaire 
WHERE pere_id IS NOT NULL;
```



# Algorithme par défaut et conditions

Il existe une troisième option possible pour la clause `ALGORITHM` dans la requête de création des vues : `UNDEFINED`.

Par défaut, si on ne précise pas d'algorithme pour la vue, c'est l'option `UNDEFINED` qui est utilisée. Cette option laisse MySQL décider lui-même de l'algorithme qu'il appliquera.
Si c'est possible, `MERGE` sera utilisé, car cet algorithme est plus performant que `TEMPTABLE`. Cependant, toutes les vues ne peuvent pas utiliser l'algorithme `MERGE`. En effet, une vue utilisant un ou plusieurs des éléments suivants **ne pourra pas** utiliser `MERGE` :


- `DISTINCT` ;
- `LIMIT` ;
- une fonction d'agrégation (`SUM()`, `COUNT()`, `MAX()`, etc.) ;
- `GROUP BY` ;
- `HAVING` ;
- `UNION` ;
- une sous-requête dans la clause `SELECT`.