On a tendance à penser que les vues ne servent que pour la sélection de données. En réalité, il est possible de modifier, insérer et supprimer des données par l'intermédiaire d'une vue.
Les requêtes sont les mêmes que pour insérer, modifier et supprimer des données à partir d'une table (si ce n'est qu'on met le nom de la vue au lieu du nom de la table bien sûr).

Cependant, pour qu'une vue ne soit pas en "lecture seule", elle doit répondre à une série de conditions. 


# Conditions pour qu'une vue permette de modifier des données (requêtes UPDATE)


## Jointures

Il est possible de modifier des données à partir d'une vue définie avec une jointure, à condition de ne modifier qu'**une seule table**.

**Exemple**


```sql
-- Modifie Animal
UPDATE V_Animal_details
SET commentaires = 'Rhume chronique'
WHERE id = 21;

-- Modifie Race
UPDATE V_Animal_details 
SET race_nom = 'Maine Coon'
WHERE race_nom = 'Maine coon';

-- Erreur
UPDATE V_Animal_details 
SET commentaires = 'Vilain oiseau', espece_nom = 'Perroquet pas beau' -- commentaires vient de Animal, et espece_nom vient de Espece
WHERE espece_id = 4;
```


Les deux premières modifications ne posent aucun problème, mais la troisième échouera, car elle modifie des colonnes appartenant à deux tables différentes.


```console
ERROR 1393 (HY000): Can not modify more than one base table through a join view 'elevage.v_animal_details'
```



## Algorithme

La vue doit utiliser l'algorithme `MERGE` (que l'algorithme ait été spécifié à la création, ou choisi par MySQL si l'algorithme n'a pas été défini). Par conséquent, les mêmes conditions que pour utiliser l'algorithme `MERGE` s'appliquent. Les éléments suivants sont ainsi interdits dans une vue si l'on veut pouvoir modifier les données à partir de celle-ci :


- `DISTINCT` ;
- `LIMIT` ;
- une fonction d’agrégation (`SUM()`, `COUNT()`, `MAX()`, etc.) ;
- `GROUP BY` ;
- `HAVING` ;
- `UNION` ;
- une sous-requête dans la clause `SELECT`.


**Exemple** : la vue _V_Nombre_espece_, qui utilise une fonction d'agrégation, ne permet pas de modifier les données.


```sql
UPDATE V_Nombre_espece
SET nb = 6
WHERE id = 4;
```



```console
1288 (HY000): The target table V_Nombre_espece of the UPDATE is not updatable
```



## Autres conditions



- On ne peut pas modifier les données à partir d'une vue qui est elle-même définie à partir d'une vue qui ne permet pas la modification des données.
- Ce n'est pas non plus possible à partir d'une vue dont la clause `WHERE` contient une sous-requête faisant référence à une des tables de la clause `FROM`.




# Conditions pour qu'une vue permette d'insérer des données (requêtes INSERT)

On peut insérer des données dans une vue si celle-ci respecte **les mêmes conditions que pour la modification de données**, ainsi que les conditions supplémentaires suivantes.


## Valeurs par défaut

Toutes les colonnes n'ayant pas de valeur par défaut (et ne pouvant pas être `NULL`) de la table dans laquelle on veut faire l'insertion doivent être référencées par la vue.

**Exemple**


```sql
INSERT INTO V_Animal_stagiaire (nom, sexe, date_naissance, espece_id, race_id)
VALUES ('Rocco', 'M', '2012-03-12', 1, 9);
```


Ceci fonctionne. La colonne _commentaires_ n'est pas référencée par _V_Animal_stagiaire_ mais peut être `NULL` (qui est donc sa valeur par défaut).


```sql
CREATE VIEW V_Animal_mini
AS SELECT id, nom, sexe, espece_id
FROM Animal;

INSERT INTO V_Animal_mini(nom, sexe, espece_id)
VALUES ('Toxi', 'F', 1);
```


Par contre, l'insertion dans _V_Animal_mini_ échoue puisque _date_naissance_ n'est pas référencée, ne peut pas être `NULL`, et n'a pas de valeur par défaut.


```console
ERROR 1423 (HY000): Field of view 'elevage.v_animal_mini' underlying table doesn't have a default value
```



## Jointures

Les vues avec jointure peuvent supporter l'insertion si :


- il n'y a que des jointures internes ;
- l'insertion se fait sur une seule table (comme pour la modification).


**Exemple**


```sql
INSERT INTO V_Animal_details (espece_nom, espece_nom_latin)
VALUES ('Perruche terrestre', 'Pezoporus wallicus');
```


Il y a une jointure externe dans _V_Animal_details_, donc l'insertion ne fonctionnera pas.


```console
ERROR 1471 (HY000): The target table V_Animal_details of the INSERT is not insertable-into
```


Par contre, si l'on crée une table _V_Animal_espece_, avec uniquement une jointure interne, il n'y a aucun problème.


```sql
CREATE OR REPLACE VIEW V_Animal_espece
AS SELECT Animal.id, Animal.sexe, Animal.date_naissance, Animal.nom, Animal.commentaires, 
       Animal.espece_id, Animal.race_id, Animal.mere_id, Animal.pere_id, Animal.disponible,
       Espece.nom_courant AS espece_nom, Espece.nom_latin AS espece_nom_latin
FROM Animal
INNER JOIN Espece ON Espece.id = Animal.espece_id;

INSERT INTO V_Animal_espece (espece_nom, espece_nom_latin)
VALUES ('Perruche terrestre', 'Pezoporus wallicus');
```



## Expressions

Les colonnes de la vue doivent être de simples références à des colonnes, et non pas des expressions.

**Exemple**

Dans la vue _V_Espece_dollars_, la colonne _prix_dollars_ correspond à _ROUND(prix*1.30813, 2)_. Il n'est donc pas possible d'insérer des données à partir de cette vue.


```sql
INSERT INTO V_Espece_dollars (nom_courant, nom_latin, prix_dollars)
VALUES ('Perruche terrestre', 'Pezoporus wallicus', 30);
```



```console
ERROR 1471 (HY000): The target table V_Espece_dollars of the INSERT is not insertable-into
```



## Colonnes dupliquées

La même colonne ne peut pas être référencée deux fois dans la vue.

**Exemple**

Si l'on crée une vue avec deux fois la même colonne référencée, il est possible de modifier des données à partir de celle-ci, mais pas d'en insérer.


```sql
CREATE VIEW V_Espece_2noms
AS SELECT id, nom_courant, nom_latin, description, prix, nom_courant AS nom2 -- nom_courant est référencé deux fois
FROM Espece;

-- Modification, pas de problème
UPDATE V_Espece_2noms
SET description= 'Joli oiseau aux plumes majoritairement vert brillant', prix = 20.00
WHERE nom_courant = 'Perruche terrestre';

-- Insertion, impossible
INSERT INTO V_Espece_2noms (nom_courant, nom_latin, prix)
VALUES ('Perruche turquoisine', 'Neophema pulchella', 40);
```



```console
ERROR 1471 (HY000): The target table V_Espece_2noms of the INSERT is not insertable-into
```



# Conditions pour qu'une vue permette de supprimer des données (requêtes DELETE)

Il est possible de supprimer des données à partir d'une vue si et seulement si :


- il est possible de modifier des données à partir de cette vue ;
- cette vue est "mono-table" (une seule table sous-jacente).



# Option de la vue pour la modification des données

Lors de la création d'une vue, on peut ajouter une option : `WITH [LOCAL | CASCADED] CHECK OPTION`.


```sql
CREATE [OR REPLACE]
    [ALGORITHM = {UNDEFINED | MERGE | TEMPTABLE}]
    VIEW nom_vue [(liste_colonnes)]
    AS requete_select
    [WITH [CASCADED | LOCAL] CHECK OPTION]
```


Lorsque cette option est spécifiée, les modifications et insertions ne sont acceptées que si les données répondent aux conditions de la vue (c'est-à-dire aux conditions données par la clause `WHERE` de la requête définissant la vue). 

**Exemples**

Si la vue _V_Animal_stagiaire_ (qui, pour rappel, sélectionne les chats uniquement) est définie avec `WITH CHECK OPTION`, on ne peut pas modifier l'_espece_id_ à partir de cette vue.


```sql
CREATE OR REPLACE VIEW V_Animal_stagiaire
AS SELECT id, nom, sexe, date_naissance, espece_id, race_id, mere_id, pere_id, disponible
FROM Animal
WHERE espece_id = 2
WITH CHECK OPTION;
```


En effet, cette vue est définie avec la condition `WHERE espece_id = 2`. Les modifications faites sur les données de cette vue doivent respecter cette condition.


```sql
UPDATE V_Animal_stagiaire 
SET espece_id = 1
WHERE nom = 'Cracotte';
```



```console
ERROR 1369 (HY000): CHECK OPTION failed 'elevage.v_animal_stagiaire'
```


De même, l'insertion d'un animal dont l'_espece_id_ n'est pas 2 sera refusée aussi.


```sql
INSERT INTO V_Animal_stagiaire (sexe, date_naissance, espece_id, nom)
VALUES ('F', '2011-09-21 15:14:00', 2, 'Bambi');       -- c'est un chat, pas de problème

INSERT INTO V_Animal_stagiaire (sexe, date_naissance, espece_id, nom)
VALUES ('M', '2011-03-11 05:54:00', 6, 'Tiroli');       -- c'est une perruche, impossible
```


La première insertion ne pose aucun problème, mais la seconde échoue.


## LOCAL ou CASCADED



- `LOCAL` : seules les conditions de la vue même sont vérifiées.
- `CASCADED` : les conditions des vues sous-jacentes éventuelles sont également vérifiées. C'est l'option par défaut.


La vue _V_Chien_race_, par exemple, est définie à partir de la vue _V_Chien_. Si on ajoute `WITH LOCAL CHECK OPTION` à _V_Chien_race_, les modifications et insertions dans cette vue devront vérifier uniquement les conditions de la vue elle-même, c'est-à-dire `race_id IS NOT NULL`. Si par contre, c'est `WITH CASCADED CHECK OPTION` qui est choisi, alors les modifications et insertions devront toujours vérifier les conditions de _V_Chien_race_, mais également celles de _V_Chien_ (`espece_id = 1`).

**Exemple 1** : `LOCAL`


```sql
CREATE OR REPLACE VIEW V_Chien_race
AS SELECT id, sexe, date_naissance, nom, commentaires, espece_id, race_id, mere_id, pere_id, disponible 
FROM V_Chien
WHERE race_id IS NOT NULL
WITH LOCAL CHECK OPTION;

-- Modification --
-- ------------ --
UPDATE V_Chien_race 
SET race_id = NULL             -- Ne respecte pas la condition de V_Chien_race 
WHERE nom = 'Zambo';           -- => Impossible

UPDATE V_Chien_race
SET espece_id = 2, race_id = 4 -- Ne respecte pas la condition de V_Chien 
WHERE nom = 'Java';           -- => possible puisque LOCAL CHECK OPTION

-- Insertion --
-- --------- --
INSERT INTO V_Chien_race (sexe, date_naissance, nom, commentaires, espece_id, race_id)   -- Respecte toutes les conditions
VALUES ('M', '2012-02-28 03:05:00', 'Pumba', 'Prématuré, à surveiller', 1, 9);           -- => Pas de problème

INSERT INTO V_Chien_race (sexe, date_naissance, nom, commentaires, espece_id, race_id)   -- La race n'est pas NULL, mais c'est un chat
VALUES ('M', '2011-05-24 23:51:00', 'Lion', NULL, 2, 5);                                 -- => pas de problème puisque LOCAL

INSERT INTO V_Chien_race (sexe, date_naissance, nom, commentaires, espece_id, race_id)   -- La colonne race_id est NULL
VALUES ('F', '2010-04-28 13:01:00', 'Mouchou', NULL, 1, NULL);                           -- => impossible
```


La première modification et la dernière insertion échouent donc avec l'erreur suivante :


```console
ERROR 1369 (HY000): CHECK OPTION failed 'elevage.v_chien_race'
```


**Exemple 2** : `CASCADED`


```sql
CREATE OR REPLACE VIEW V_Chien_race
AS SELECT id, sexe, date_naissance, nom, commentaires, espece_id, race_id, mere_id, pere_id, disponible
FROM V_Chien
WHERE race_id IS NOT NULL
WITH CASCADED CHECK OPTION;

UPDATE V_Chien_race 
SET race_id = NULL              -- Ne respecte pas la condition de V_Chien_race 
WHERE nom = 'Zambo';            -- => impossible

UPDATE V_Chien_race
SET espece_id = 2, race_id = 4  -- Ne respecte pas la condition de V_Chien 
WHERE nom = 'Fila';             -- => impossible aussi puisque CASCADED
```


Cette fois, les deux modifications échouent.