On l'a dit, une table temporaire est une table qui n'existe que dans la session qui l'a créée. En dehors de ça, c'est une table presque normale. On peut exécuter sur ces tables toutes les opérations que l'on exécute sur une table classique : insérer des données, les modifier, les supprimer, et bien sûr les sélectionner.


# Création, modification, suppression d'une table temporaire

Pour créer, modifier et supprimer une table temporaire, on utilise les mêmes commandes que pour les tables normales.


## Création

Pour créer une table temporaire, on peut utiliser tout simplement `CREATE TABLE` en ajoutant `TEMPORARY`, pour préciser qu'il s'agit d'une table temporaire.

**Exemple** : création d'une table temporaire _TMP_Animal_.


```sql
CREATE TEMPORARY TABLE TMP_Animal (
    id INT UNSIGNED PRIMARY KEY,
    nom VARCHAR(30),
    espece_id INT UNSIGNED,
    sexe CHAR(1)
);

DESCRIBE TMP_Animal;
```




+---------+----------------+----+---+-------+-----+
|Field    |Type            |Null|Key|Default|Extra|
+=========+================+====+===+=======+=====+
|id       |int(10) unsigned|NO  |PRI|NULL   |     |
+---------+----------------+----+---+-------+-----+
|nom      |varchar(30)     |YES |   |NULL   |     |
+---------+----------------+----+---+-------+-----+
|espece_id|int(10) unsigned|YES |   |NULL   |     |
+---------+----------------+----+---+-------+-----+
|sexe     |char(1)         |YES |   |NULL   |     |
+---------+----------------+----+---+-------+-----+





Visiblement, la table a bien été créée. Par contre, si l'on tente de vérifier cela en utilisant `SHOW TABLES` plutôt que `DESCRIBE TMP_Animal`, on ne trouvera pas la nouvelle table temporaire dans la liste des tables. Et pour cause, seules les tables permanentes (et les vues) sont listées par cette commande.


## Modification

Pour la modification, nul besoin du mot-clé `TEMPORARY`, on utilise directement `ALTER TABLE`, comme s'il s'agissait d'une table normale.

**Exemple** : ajout d'une colonne à _TMP_Animal_.


```sql
ALTER TABLE TMP_Animal
ADD COLUMN date_naissance DATETIME;
```



## Suppression

En ce qui concerne la suppression, on a le choix d'ajouter `TEMPORARY`, ou non.
Si `TEMPORARY` est précisé, la table mentionnée ne sera supprimée que s'il s'agit bien d'une table temporaire. Sans ce mot-clé, on pourrait supprimer par erreur une table non temporaire, en cas de confusion des noms des tables par exemple.

**Exemple** : suppression de _TMP_Animal_, qui n'aura pas fait long feu.


```sql
DROP TEMPORARY TABLE TMP_Animal;
```



# Utilisation des tables temporaires

Une table temporaire s'utilise comme une table normale. Les commandes d'insertion, modification et suppression de données sont exactement les mêmes.

Notez que l'immense majorité des tables temporaires étant créées pour stocker des données venant d'autres tables, on utilise souvent `INSERT INTO ... SELECT` pour l'insertion des données.


# Cache-cache table

Une table temporaire existe donc uniquement pour la session qui la crée. Par conséquent, deux sessions différentes peuvent parfaitement utiliser chacune une table temporaire ayant le même nom, mais ni la même structure ni les mêmes données.

Il est également possible de créer une table temporaire ayant le même nom qu'une table normale. Toutes les autres sessions éventuelles continueront à travailler sur la table normale comme si de rien n'était.


[[question]]
| Et la session qui crée cette fameuse table ?

En cas de conflit de noms entre une table temporaire et une table permanente, la table temporaire va masquer la table permanente.
Donc, tant que cette table temporaire existera (c'est-à-dire jusqu'à ce qu'on la supprime explicitement, ou jusqu'à déconnexion de la session), toutes les requêtes faites en utilisant son nom seront exécutées sur la table temporaire, et non sur la table permanente de même nom.

**Exemple** : création d'une table temporaire nommée _Animal_.

On commence par sélectionner les perroquets de la table _Animal_ (la table normale, puisqu'on n'a pas encore créé la table temporaire). On obtient quatre résultats.


```sql
SELECT id, sexe, nom, commentaires, espece_id 
FROM Animal
WHERE espece_id = 4;
```




+--+----+--------+---------------------+---------+
|id|sexe|nom     |commentaires         |espece_id|
+==+====+========+=====================+=========+
|57|M   |Safran  |Coco veut un gâteau !|4        |
+--+----+--------+---------------------+---------+
|58|M   |Gingko  |Coco veut un gâteau !|4        |
+--+----+--------+---------------------+---------+
|59|M   |Bavard  |Coco veut un gâteau !|4        |
+--+----+--------+---------------------+---------+
|60|F   |Parlotte|Coco veut un gâteau !|4        |
+--+----+--------+---------------------+---------+





On crée ensuite la table temporaire, et on y insère un animal avec _espece_id = 4_. On refait la même requête de sélection, et on obtient uniquement l'animal que l'on vient d'insérer, avec la structure de la table temporaire.


```sql
CREATE TEMPORARY TABLE Animal (
    id INT UNSIGNED PRIMARY KEY,
    nom VARCHAR(30),
    espece_id INT UNSIGNED,
    sexe CHAR(1)
);

INSERT INTO Animal
VALUES (1, 'Tempo', 4, 'M');

SELECT * 
FROM Animal
WHERE espece_id = 4;
```




+--+-----+---------+----+
|id|nom  |espece_id|sexe|
+==+=====+=========+====+
|1 |Tempo|4        |M   |
+--+-----+---------+----+





Pour terminer, suppression de la table temporaire. La même requête de sélection nous affiche à nouveau les quatre perroquets de départ.


```sql
DROP TEMPORARY TABLE Animal;

SELECT id, sexe, nom, commentaires, espece_id 
FROM Animal
WHERE espece_id = 4;
```




+--+----+--------+---------------------+---------+
|id|sexe|nom     |commentaires         |espece_id|
+==+====+========+=====================+=========+
|57|M   |Safran  |Coco veut un gâteau !|4        |
+--+----+--------+---------------------+---------+
|58|M   |Gingko  |Coco veut un gâteau !|4        |
+--+----+--------+---------------------+---------+
|59|M   |Bavard  |Coco veut un gâteau !|4        |
+--+----+--------+---------------------+---------+
|60|F   |Parlotte|Coco veut un gâteau !|4        |
+--+----+--------+---------------------+---------+





Dans ce cas de figure, il vaut mieux utiliser le mot-clé `TEMPORARY` dans la requête de suppression de la table, afin d'être sûr qu'on détruit bien la table temporaire et non la table permanente.


[[attention]]
| Gardez bien en tête le fait que les tables temporaires sont détruites dès que la connexion prend fin. Dans de nombreuses applications, une nouvelle connexion est créée à chaque nouvelle action. Par exemple, pour un site internet codé en PHP, on crée une nouvelle connexion (donc une nouvelle session) à chaque rechargement de page.



# Restrictions des tables temporaires

Deux restrictions importantes à noter lorsque l'on travaille avec des tables temporaires :


- on ne peut pas mettre de clé étrangère sur une table temporaire ;
- on ne peut pas faire référence à une table temporaire deux fois dans la même requête.


Quelques explications sur la deuxième restriction : lorsque l'on parle de "faire référence deux fois à la table", il s'agit d'aller **chercher des données** deux fois dans la table. Le nom de la table temporaire peut apparaître plusieurs fois dans la requête, par exemple pour préfixer le nom des colonnes.

**Exemples** : on recrée une table _TMP_Animal_, un peu plus complexe cette fois, qu'on remplit avec les chats de la table _Animal_.


```sql
CREATE TEMPORARY TABLE TMP_Animal (
    id INT UNSIGNED PRIMARY KEY,
    nom VARCHAR(30),
    espece_id INT UNSIGNED,
    sexe CHAR(1),
    mere_id INT UNSIGNED,
    pere_id INT UNSIGNED
);

INSERT INTO TMP_Animal
SELECT id, nom, espece_id, sexe, mere_id, pere_id 
FROM Animal
WHERE espece_id = 2;
```


**1.** Référence n'est pas occurrence


```sql
SELECT TMP_Animal.nom, TMP_Animal.sexe
FROM TMP_Animal
WHERE nom LIKE 'B%';
```




+--------+----+
|nom     |sexe|
+========+====+
|Bagherra|M   |
+--------+----+
|Boucan  |M   |
+--------+----+
|Boule   |F   |
+--------+----+
|Bilba   |F   |
+--------+----+
|Bambi   |F   |
+--------+----+





Ici, _TMP_Animal_ apparaît trois fois dans la requête. Cela ne pose aucun problème, puisque la table n'est utilisée qu'une fois pour en extraire des données : dans le `FROM`.

**2.** Auto-jointure

On essaye d'afficher le nom des chats, ainsi que le nom de leur père quand on le connaît.


```sql
SELECT TMP_Animal.nom, TMP_Pere.nom AS pere 
FROM TMP_Animal
INNER JOIN TMP_Animal AS TMP_Pere 
    ON TMP_Animal.pere_id = TMP_Pere.id;
```


```console
ERROR 1137 (HY000): Can't reopen table: 'TMP_Animal'
```


Cette fois, cela ne fonctionne pas : on extrait en effet des données de _TMP_Animal_, et de _TMP_Animal_ `AS` _TMP_Pere_.

**3.** Sous-requête

On affiche le nom des animaux référencés comme pères.


```sql
SELECT nom
FROM TMP_Animal
WHERE id IN (SELECT pere_id FROM TMP_Animal);
```


```console
ERROR 1137 (HY000): Can't reopen table: 'TMP_Animal'
```


À nouveau, on extrait des données de _TMP_Animal_ à deux endroits différents : dans la requête et dans la sous-requête. Cela ne peut pas fonctionner.


# Interaction avec les transactions

Un comportement assez intéressant lié aux tables temporaires concerne les transactions. 
On a vu que les commandes de création d'objets en tout genre provoquaient la validation implicite de la transaction dans laquelle la commande était faite. Ainsi, `CREATE TABLE` provoque une validation implicite.

Mais ce n'est pas le cas pour les tables temporaires : `CREATE TEMPORARY TABLE` et `DROP TEMPORARY TABLE` ne valident pas les transactions.

Cependant ces deux commandes ne peuvent pas être annulées par un `ROLLBACK`.

**Exemple**


```sql
START TRANSACTION;

INSERT INTO Espece (nom_courant, nom_latin)
VALUES ('Gerbille de Mongolie', 'Meriones unguiculatus');

CREATE TEMPORARY TABLE TMP_Test (id INT);

ROLLBACK;

SELECT id, nom_courant, nom_latin, prix FROM Espece;

SELECT * FROM TMP_Test;
```




+--+------------------+---------------------+------+
|id|nom_courant       |nom_latin            |prix  |
+==+==================+=====================+======+
|1 |Chien             |Canis canis          |200.00|
+--+------------------+---------------------+------+
|2 |Chat              |Felis silvestris     |150.00|
+--+------------------+---------------------+------+
|3 |Tortue d'Hermann  |Testudo hermanni     |140.00|
+--+------------------+---------------------+------+
|4 |Perroquet amazone |Alipiopsitta xanthops|700.00|
+--+------------------+---------------------+------+
|5 |Rat brun          |Rattus norvegicus    |10.00 |
+--+------------------+---------------------+------+
|6 |Perruche terrestre|Pezoporus wallicus   |20.00 |
+--+------------------+---------------------+------+





```console
Empty set (0.00 sec)
```


Il n'y a pas de gerbille dans nos espèces, puisque l'insertion a été annulée. Par contre, la création de la table _TMP_Test_, bien que faite à l'intérieur d'une transaction annulée, a bel et bien été exécutée. On n'obtient en effet aucune erreur lorsque l'on fait une sélection sur cette table (mais bien un résultat vide, puisqu'il n'y a aucune donnée dans cette table).