Les deux méthodes de création de tables présentées ici sont valables pour créer **des tables normales ou des tables temporaires**. Elles se basent toutes les deux sur des tables et/ou des données préexistantes. 
Le choix de présenter ces méthodes dans le cadre des tables temporaires s'explique par le fait qu'en général, celles-ci sont créées pour stocker des données provenant d'autres tables, et par conséquent, sont souvent créées en utilisant une de ces deux méthodes.


# Créer une table à partir de la structure d'une autre

Il est possible de créer la copie exacte d'une table, en utilisant la commande suivante :


```sql
CREATE [TEMPORARY] TABLE nouvelle_table
LIKE ancienne_table;
```


Les types de données, les index, les contraintes, les valeurs par défaut, le moteur de stockage, toutes les caractéristiques de _ancienne_table_ seront reproduites dans _nouvelle_table_. Par contre, les données ne seront pas copiées : _nouvelle_table_ sera vide.

**Exemple** : reproduction de la table _Espece_.


```sql
CREATE TABLE Espece_copy
LIKE Espece;

DESCRIBE Espece;

DESCRIBE Espece_copy;
```


Les deux commandes `DESCRIBE` renvoient exactement la même chose :




+-----------+---------------------+----+---+-------+--------------+
|Field      |Type                 |Null|Key|Default|Extra         |
+===========+=====================+====+===+=======+==============+
|id         |smallint(6) unsigned |NO  |PRI|NULL   |auto_increment|
+-----------+---------------------+----+---+-------+--------------+
|nom_courant|varchar(40)          |NO  |   |NULL   |              |
+-----------+---------------------+----+---+-------+--------------+
|nom_latin  |varchar(40)          |NO  |UNI|NULL   |              |
+-----------+---------------------+----+---+-------+--------------+
|description|text                 |YES |   |NULL   |              |
+-----------+---------------------+----+---+-------+--------------+
|prix       |decimal(7,2) unsigned|YES |   |NULL   |              |
+-----------+---------------------+----+---+-------+--------------+





Ensuite, pour remplir la nouvelle table avec tout ou partie des données de la table d'origine, il suffit d'utiliser la commande `INSERT INTO ... SELECT`.

**Exemple**


```sql
INSERT INTO Espece_copy
SELECT * FROM Espece
WHERE prix < 100;

SELECT id, nom_courant, prix 
FROM Espece_copy;
```




+--+------------------+-----+
|id|nom_courant       |prix |
+==+==================+=====+
|5 |Rat brun          |10.00|
+--+------------------+-----+
|6 |Perruche terrestre|20.00|
+--+------------------+-----+






## Tables temporaires

Si l'on crée une table temporaire avec cette commande, tous les attributs de la table d'origine seront conservés, sauf les clés étrangères, puisqu'on ne peut avoir de clé étrangère dans une table temporaire.

**Exemple** : copie de la table _Animal_


```sql
CREATE TEMPORARY TABLE Animal_copy
LIKE Animal;

INSERT INTO Animal (nom, sexe, date_naissance, espece_id)
VALUES ('Mutant', 'M', NOW(), 12);

INSERT INTO Animal_copy (nom, sexe, date_naissance, espece_id)
VALUES ('Mutant', 'M', NOW(), 12);
```


Aucune espèce n'a 12 pour _id_, l'insertion dans _Animal_ échoue donc à cause de la clé étrangère. Par contre, dans la table temporaire _Animal_copy_, l'insertion réussit.


[[attention]]
| Si on crée une table sur la base d'une table temporaire, la nouvelle table n'est pas temporaire par défaut. Pour qu'elle le soit, il faut obligatoirement le préciser à la création avec `CREATE TEMPORARY TABLE`.



# Créer une table à partir de données sélectionnées

Cette seconde méthode ne se base pas sur la structure d'une table, mais sur les données récupérées par une requête `SELECT` pour construire une table, et y insérer des données. On fait donc ici d'une pierre deux coups : création de la table et insertion de données dans celle-ci.


```sql
CREATE [TEMPORARY] TABLE nouvelle_table
SELECT ...
```


Le type des colonnes de la nouvelle table sera déduit des colonnes sélectionnées. Par contre, la plupart des caractéristiques des colonnes sélectionnées seront perdues :


- les index ;
- les clés (primaires ou étrangères) ;
- l'auto-incrémentation.


Les valeurs par défaut et les contraintes `NOT NULL` seront par contre conservées.

**Exemple** : suppression puis recréation d'une table temporaire _Animal_copy_ contenant tous les rats de la table _Animal_. 


```sql
DROP TABLE Animal_copy;

CREATE TEMPORARY TABLE Animal_copy
SELECT *
FROM Animal
WHERE espece_id = 5;

DESCRIBE Animal;

DESCRIBE Animal_copy;
```




+--------------+--------------------+----+---+-------+--------------+
|Field         |Type                |Null|Key|Default|Extra         |
+==============+====================+====+===+=======+==============+
|id            |smallint(6) unsigned|NO  |PRI|NULL   |auto_increment|
+--------------+--------------------+----+---+-------+--------------+
|sexe          |char(1)             |YES |   |NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|date_naissance|datetime            |NO  |   |NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|nom           |varchar(30)         |YES |MUL|NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|commentaires  |text                |YES |   |NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|espece_id     |smallint(6) unsigned|NO  |MUL|NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|race_id       |smallint(6) unsigned|YES |MUL|NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|mere_id       |smallint(6) unsigned|YES |MUL|NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|pere_id       |smallint(6) unsigned|YES |MUL|NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|disponible    |tinyint(1)          |YES |   |1      |              |
+--------------+--------------------+----+---+-------+--------------+







+--------------+--------------------+----+---+-------+-----+
|Field         |Type                |Null|Key|Default|Extra|
+==============+====================+====+===+=======+=====+
|id            |smallint(6) unsigned|NO  |   |0      |     |
+--------------+--------------------+----+---+-------+-----+
|sexe          |char(1)             |YES |   |NULL   |     |
+--------------+--------------------+----+---+-------+-----+
|date_naissance|datetime            |NO  |   |NULL   |     |
+--------------+--------------------+----+---+-------+-----+
|nom           |varchar(30)         |YES |   |NULL   |     |
+--------------+--------------------+----+---+-------+-----+
|commentaires  |text                |YES |   |NULL   |     |
+--------------+--------------------+----+---+-------+-----+
|espece_id     |smallint(6) unsigned|NO  |   |NULL   |     |
+--------------+--------------------+----+---+-------+-----+
|race_id       |smallint(6) unsigned|YES |   |NULL   |     |
+--------------+--------------------+----+---+-------+-----+
|mere_id       |smallint(6) unsigned|YES |   |NULL   |     |
+--------------+--------------------+----+---+-------+-----+
|pere_id       |smallint(6) unsigned|YES |   |NULL   |     |
+--------------+--------------------+----+---+-------+-----+
|disponible    |tinyint(1)          |YES |   |1      |     |
+--------------+--------------------+----+---+-------+-----+





Les types de colonnes et les contraintes `NOT NULL` sont les mêmes, mais les index ont disparu, ainsi que l'auto-incrémentation.


## Forcer le type des colonnes

On peut laisser MySQL déduire le type des colonnes du `SELECT`, mais il est également possible de préciser le type que l'on désire, en faisant attention à la compatibilité entre les types que l'on précise et les colonnes sélectionnées.
On peut également préciser les index désirés, les clés et l'éventuelle colonne à auto-incrémenter.

La syntaxe est alors similaire à un `CREATE [TEMPORARY] TABLE` classique, si ce n'est qu'on rajoute une requête de sélection à la suite.

**Exemple** : recréation d'_Animal_copy_, en modifiant quelques types et attributs.


```sql
DROP TABLE Animal_copy;

CREATE TEMPORARY TABLE Animal_copy (
    id INT UNSIGNED PRIMARY KEY AUTO_INCREMENT,
    sexe CHAR(1),
    date_naissance DATETIME,
    nom VARCHAR(100),
    commentaires TEXT,
    espece_id INT NOT NULL,
    race_id INT,
    mere_id INT,
    pere_id INT,
    disponible BOOLEAN DEFAULT TRUE,
    INDEX (nom(10))
) ENGINE=InnoDB
SELECT *
FROM Animal
WHERE espece_id = 5;

DESCRIBE Animal_copy;
```




+--------------+----------------+----+---+-------+--------------+
|Field         |Type            |Null|Key|Default|Extra         |
+==============+================+====+===+=======+==============+
|id            |int(10) unsigned|NO  |PRI|NULL   |auto_increment|
+--------------+----------------+----+---+-------+--------------+
|sexe          |char(1)         |YES |   |NULL   |              |
+--------------+----------------+----+---+-------+--------------+
|date_naissance|datetime        |YES |   |NULL   |              |
+--------------+----------------+----+---+-------+--------------+
|nom           |varchar(100)    |YES |MUL|NULL   |              |
+--------------+----------------+----+---+-------+--------------+
|commentaires  |text            |YES |   |NULL   |              |
+--------------+----------------+----+---+-------+--------------+
|espece_id     |int(11)         |NO  |   |NULL   |              |
+--------------+----------------+----+---+-------+--------------+
|race_id       |int(11)         |YES |   |NULL   |              |
+--------------+----------------+----+---+-------+--------------+
|mere_id       |int(11)         |YES |   |NULL   |              |
+--------------+----------------+----+---+-------+--------------+
|pere_id       |int(11)         |YES |   |NULL   |              |
+--------------+----------------+----+---+-------+--------------+
|disponible    |tinyint(1)      |YES |   |1      |              |
+--------------+----------------+----+---+-------+--------------+





Les colonnes _id_, _espece_id_, _race_id_, _mere_id_ et _pere_id_ ne sont plus des `SMALLINT` mais des `INT`, et seul _id_ est `UNSIGNED`. Par contre, on a bien un index sur _nom_, et une clé primaire auto-incrémentée avec _id_. On n'a pas précisé de contrainte `NOT NULL` sur _date_naissance_, donc il n'y en a pas, bien que cette contrainte existe dans la table d'origine.


## Nom des colonnes

Les noms des colonnes de la table créée correspondront aux noms des colonnes dans la requête `SELECT`. Il est bien sûr possible d'utiliser les alias dans celle-ci pour renommer une colonne.
Cela implique que lorsque vous forcez le type des colonnes comme on vient de le voir, il est impératif que vos noms de colonnes correspondent à ceux de la requête `SELECT`, car l'insertion des données dans leurs colonnes respectives se fera sur la base du nom, et pas sur la base de la position.

**Exemple** : on recrée _Animal_copy_, mais les noms des colonnes sont dans le désordre, et certains ne correspondent pas à la requête `SELECT`.


```sql
DROP TABLE Animal_copy;

CREATE TEMPORARY TABLE Animal_copy (
    id INT UNSIGNED PRIMARY KEY AUTO_INCREMENT,
    nom VARCHAR(100),                            -- Ordre différent de la requête SELECT
    sexe CHAR(1),
    espece_id INT NOT NULL,                      -- Ordre différent de la requête SELECT
    date_naissance DATETIME,
    commentaires TEXT,
    race_id INT,
    maman_id INT,                                -- Nom de colonne différent de la requête SELECT
    papa_id INT,                                 -- Nom de colonne différent de la requête SELECT
    disponible BOOLEAN DEFAULT TRUE,
    INDEX (nom(10))
) ENGINE=InnoDB
SELECT id, sexe, date_naissance, nom, commentaires, espece_id, race_id, mere_id, pere_id, disponible
FROM Animal
WHERE espece_id = 5;

DESCRIBE Animal_copy;
```




+--------------+--------------------+----+---+-------+--------------+
|Field         |Type                |Null|Key|Default|Extra         |
+==============+====================+====+===+=======+==============+
|maman_id      |int(11)             |YES |   |NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|papa_id       |int(11)             |YES |   |NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|id            |int(10) unsigned    |NO  |PRI|NULL   |auto_increment|
+--------------+--------------------+----+---+-------+--------------+
|sexe          |char(1)             |YES |   |NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|date_naissance|datetime            |YES |   |NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|nom           |varchar(100)        |YES |MUL|NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|commentaires  |text                |YES |   |NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|espece_id     |int(11)             |NO  |   |NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|race_id       |int(11)             |YES |   |NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|mere_id       |smallint(6) unsigned|YES |   |NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|pere_id       |smallint(6) unsigned|YES |   |NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|disponible    |tinyint(1)          |YES |   |1      |              |
+--------------+--------------------+----+---+-------+--------------+





Les colonnes _mere_id_ et _pere_id_ de la requête `SELECT` ne correspondaient à aucune colonne définie par la commande `CREATE TABLE`. Ces colonnes ont donc été créées dans la nouvelle table, mais leur type a été déduit à partir du `SELECT`. Quant aux colonnes _maman_id_ et _papa_id_, elles ont bien été créées bien qu'elles ne correspondent à rien dans la sélection. Elles ne contiendront simplement aucune donnée.

Voyons maintenant le contenu d'_Animal_copy_.


```sql
SELECT maman_id, papa_id, id, sexe, nom, espece_id, mere_id, pere_id 
FROM Animal_copy;
```




+--------+-------+--+----+----+---------+-------+-------+
|maman_id|papa_id|id|sexe|nom |espece_id|mere_id|pere_id|
+========+=======+==+====+====+=========+=======+=======+
|NULL    |NULL   |69|F   |Baba|5        |NULL   |NULL   |
+--------+-------+--+----+----+---------+-------+-------+
|NULL    |NULL   |70|M   |Bibo|5        |72     |73     |
+--------+-------+--+----+----+---------+-------+-------+
|NULL    |NULL   |72|F   |Momy|5        |NULL   |NULL   |
+--------+-------+--+----+----+---------+-------+-------+
|NULL    |NULL   |73|M   |Popi|5        |NULL   |NULL   |
+--------+-------+--+----+----+---------+-------+-------+
|NULL    |NULL   |75|F   |Mimi|5        |NULL   |NULL   |
+--------+-------+--+----+----+---------+-------+-------+





Les données sont dans les bonnes colonnes, et les colonnes _maman_id_ et _papa_id_ sont vides, contrairement à _mere_id_ et _pere_id_.

Une conséquence intéressante de ce comportement est qu'il est tout à fait possible de préciser le type et les attributs pour une partie des colonnes uniquement.

**Exemple** : Recréation d'_Animal_copy_. Cette fois on ne précise le type et les attributs que de la colonne _id_ et on ajoute un index sur _nom_.


```sql
DROP TABLE Animal_copy;

CREATE TEMPORARY TABLE Animal_copy (
    id INT UNSIGNED PRIMARY KEY AUTO_INCREMENT,
    INDEX (nom(10))
) ENGINE=InnoDB
SELECT *
FROM Animal
WHERE espece_id = 5;

DESCRIBE Animal_copy;
```




+--------------+--------------------+----+---+-------+--------------+
|Field         |Type                |Null|Key|Default|Extra         |
+==============+====================+====+===+=======+==============+
|id            |int(10) unsigned    |NO  |PRI|NULL   |auto_increment|
+--------------+--------------------+----+---+-------+--------------+
|sexe          |char(1)             |YES |   |NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|date_naissance|datetime            |NO  |   |NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|nom           |varchar(30)         |YES |MUL|NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|commentaires  |text                |YES |   |NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|espece_id     |smallint(6) unsigned|NO  |   |NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|race_id       |smallint(6) unsigned|YES |   |NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|mere_id       |smallint(6) unsigned|YES |   |NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|pere_id       |smallint(6) unsigned|YES |   |NULL   |              |
+--------------+--------------------+----+---+-------+--------------+
|disponible    |tinyint(1)          |YES |   |1      |              |
+--------------+--------------------+----+---+-------+--------------+






## Réunir les données de plusieurs tables

Puisque cette méthode de création de table se base sur une requête de sélection, cela signifie qu'on peut très bien créer une table sur la base d'une partie des colonnes d'une table existante.
Cela signifie également que l'on peut créer une table à partir de données provenant de plusieurs tables différentes.
Bien entendu, une table ne pouvant avoir deux colonnes de même nom, il ne faut pas oublier d'utiliser les alias pour renommer certaines colonnes en cas de noms dupliqués.

**Exemple 1** : création de _Client_mini_, qui correspond à une partie de la table _Client_.


```sql
CREATE TABLE Client_mini
SELECT nom, prenom, date_naissance
FROM Client;
```


**Exemple 2** : création de _Race_espece_, à partir des tables _Espece_ et _Race_.


```sql
CREATE TABLE Race_espece
SELECT Race.id, Race.nom, Espece.nom_courant AS espece, Espece.id AS espece_id
FROM Race
INNER JOIN Espece ON Espece.id = Race.espece_id;
```