Les tables temporaires n'existant que le temps d'une session, leur usage est limité à des situations précises.


# Gain de performance

Si, **dans une même session**, vous devez effectuer des calculs et/ou des requêtes plusieurs fois sur le même set de données, il peut être intéressant de stocker ce set de données dans une table temporaire, pour travailler sur cette table.
En effet, une fois vos données de travail isolées dans une table temporaire, les requêtes vous servant à sélectionner les données qui vous intéressent seront simplifiées, donc plus rapides.

Typiquement, imaginons qu'on désire faire une série de statistiques sur les adoptions de chiens.


- Quelles races sont les plus demandées ?
- Y a-t-il une période de l'année pendant laquelle les gens adoptent beaucoup ?
- Nos clients ayant adopté un chien sont-ils souvent dans la même région ?
- Les chiens adoptés sont-ils plutôt des mâles ou des femelles ?
- …


On va avoir besoin de données provenant des tables _Animal_, _Race_, _Client_ et _Adoption_, en faisant au minimum une jointure à chaque requête pour n'avoir que les données liées aux chiens. 
Il peut dès lors être intéressant de stocker toutes les données dont on aura besoin dans une table temporaire.


```sql
CREATE TEMPORARY TABLE TMP_Adoption_chien
SELECT Animal.id AS animal_id, Animal.nom AS animal_nom, Animal.date_naissance AS animal_naissance, Animal.sexe AS animal_sexe, Animal.commentaires AS animal_commentaires,
Race.id AS race_id, Race.nom AS race_nom, 
Client.id AS client_id, Client.nom AS client_nom, Client.prenom AS client_prenom, Client.adresse AS client_adresse, 
Client.code_postal AS client_code_postal, Client.ville AS client_ville, Client.pays AS client_pays, Client.date_naissance AS client_naissance,
Adoption.date_reservation AS adoption_reservation, Adoption.date_adoption AS adoption_adoption, Adoption.prix
FROM Animal
LEFT JOIN Race ON Animal.race_id = Race.id
INNER JOIN Adoption ON Animal.id = Adoption.animal_id
INNER JOIN Client ON Client.id = Adoption.client_id
WHERE Animal.espece_id = 1;
```


Toutes les requêtes pour obtenir les statistiques et informations demandées pourront donc se faire directement sur _TMP_Adoption_chien_, ce qui sera plus rapide que de refaire chaque fois une requête avec jointure(s).
Il peut bien sûr être intéressant d'ajouter des index sur les colonnes qui serviront souvent dans les requêtes.


# Tests

Il arrive qu'on veuille tester **l'effet des instructions, avant de les appliquer sur les données**. 
Pour cela, on peut par exemple utiliser les transactions, et annuler ou valider les requêtes selon que l'effet produit correspond à ce qui était attendu ou non.
Une autre solution est d'utiliser les tables temporaires. On crée des **tables temporaires contenant les données sur lesquelles on veut faire des tests**, on effectue les tests sur celles-ci, et une fois qu'on est satisfait du résultat, on applique les mêmes requêtes aux vraies tables. Cette solution permet de ne pas risquer de valider inopinément des requêtes de test. Sans oublier qu'il n'est pas toujours possible d'utiliser les transactions : si les tables sont de type MyISAM, les transactions ne sont pas disponibles. De même, les transactions sont inutilisables si les tests comprennent des requêtes provoquant une validation implicite des transactions.


[[information]]
| **Petite astuce** : pour tester une série d'instructions à partir d'un fichier (en utilisant la commande `SOURCE`), il est souvent plus simple de créer des tables temporaires **ayant le même nom** que les tables normales, qui seront ainsi masquées. Il ne sera alors pas nécessaire de modifier le nom des tables dans les requêtes à exécuter par le fichier selon qu'on est en phase de test ou non.




# Sets de résultats et procédures stockées

Lorsque l'on crée une procédure stockée, on peut utiliser les paramètres `OUT` et `INOUT` pour récupérer les résultats de la procédure. Mais ces paramètres ne peuvent contenir qu'un seul élément.


[[question]]
| Que faire si l'on veut récupérer plusieurs lignes de résultats à partir d'une procédure stockée ?

On peut avoir besoin de récupérer le résultat d'une requête `SELECT`, ou une liste de valeurs calculées dans la procédure. Ou encore, on peut vouloir passer un set de résultats d'une procédure stockée à une autre.

Une solution est d'utiliser une table temporaire pour stocker les résultats qu'on veut utiliser en dehors de la procédure.

**Exemple** : création par une procédure d'une table temporaire stockant les adoptions qui ne sont pas en ordre de paiement.


```sql
DELIMITER |
CREATE PROCEDURE table_adoption_non_payee()
BEGIN
    DROP TEMPORARY TABLE IF EXISTS Adoption_non_payee;

    CREATE TEMPORARY TABLE Adoption_non_payee
    SELECT Client.id AS client_id, Client.nom AS client_nom, Client.prenom AS client_prenom, Client.email AS client_email,
           Animal.nom AS animal_nom, Espece.nom_courant AS espece, Race.nom AS race, 
           Adoption.date_reservation, Adoption.date_adoption, Adoption.prix
    FROM Adoption
    INNER JOIN Client ON Client.id = Adoption.client_id
    INNER JOIN Animal ON Animal.id = Adoption.animal_id
    INNER JOIN Espece ON Espece.id = Animal.espece_id
    LEFT JOIN Race ON Race.id = Animal.race_id
    WHERE Adoption.paye = FALSE; 
END |
DELIMITER ;

CALL table_adoption_non_payee();

SELECT client_id, client_nom, client_prenom, animal_nom, prix 
FROM Adoption_non_payee;
```




+---------+-----------+-------------+----------+------+
|client_id|client_nom |client_prenom|animal_nom|prix  |
+=========+===========+=============+==========+======+
|9        |Corduro    |Anaelle      |Bavard    |700.00|
+---------+-----------+-------------+----------+------+
|10       |Faluche    |Eline        |Dana      |140.00|
+---------+-----------+-------------+----------+------+
|11       |Penni      |Carine       |Pipo      |630.00|
+---------+-----------+-------------+----------+------+
|12       |Broussaille|Virginie     |Mimi      |10.00 |
+---------+-----------+-------------+----------+------+