# Vues - rappels et performance

Une vue, c'est tout simplement **une requête `SELECT` à laquelle on donne un nom**. Lorsque l'on sélectionne des données à partir d'une vue, on exécute en réalité la requête SQL de la vue. Par conséquent, les vues ne permettent pas de gagner en performance.

En fait, dans certains cas, les requêtes sur des vues peuvent même être moins rapides que si l'on fait directement la requête sur la (ou les) table(s).

Les vues peuvent en effet utiliser deux algorithmes différents : 


- `MERGE` : les clauses de la requête sur la vue (`WHERE`, `ORDER BY`,…) sont fusionnées à la requête définissant la vue ;
- `TEMPTABLE` : une table temporaire est créée avec les résultats de la requête définissant la vue, et la requête de sélection sur la vue est ensuite exécutée sur cette table temporaire.


Avec l'algorithme `MERGE`, tout se passe comme si l'on exécutait la requête directement sur les tables contenant les données. On perd un petit peu de temps à fusionner les clauses des deux requêtes, mais c'est négligeable.

Par contre, avec `TEMPTABLE`, non seulement on exécute deux requêtes (une pour créer la vue temporaire, l'autre sur cette dernière), mais en plus **la table temporaire ne possède aucun index**, contrairement aux tables normales. Une recherche dans la table temporaire peut donc prendre plus de temps que la même recherche sur une table normale, pour peu que cette dernière possède des index. 

Enfin, n'oublions pas que MySQL nécessite l'utilisation de l'algorithme `TEMPTABLE` pour les vues contenant au moins un des éléments suivants :


- `DISTINCT` ;
- `LIMIT` ;
- une fonction d'agrégation (`SUM()`, `COUNT()`, `MAX()`, etc.) ;
- `GROUP BY` ;
- `HAVING` ;
- `UNION` ;
- une sous-requête dans la clause `SELECT`.



# Vues matérialisées

Il fallait donc quelque chose qui combine les avantages des vues et les avantages des tables normales : c'est ainsi que le concept des vues matérialisées est né !

Une vue matérialisée est **un objet qui permet de stocker le résultat d'une requête `SELECT`**. Là où une vue se contente de stocker la requête, la vue matérialisée va stocker directement les résultats (elle va donc les matérialiser), plutôt que la requête. Lorsque l'on fait une requête sur une vue matérialisée, on va donc chercher directement des données dans celle-ci, sans passer par les tables d'origine et/ou une table temporaire intermédiaire.


## "Vraies" vues matérialisées

Certains SGBD (Oracle par exemple) permettent de créer directement des vues matérialisées, avec des outils et des options dédiés. Il suffit alors de créer les vues matérialisées que l'on désire et de les paramétrer correctement, et tout se fait automatiquement.

Malheureusement, MySQL ne propose pas encore ce type d'objet. Il faut donc se débrouiller avec les outils disponibles.


## "Fausses" vues matérialisées pour MySQL

On veut donc une structure qui permet de stocker des données, provenant d'une requête `SELECT`.


[[question]]
| Qu'a-t-on comme structure qui permet de stocker les données ?

La réponse semble évidente : **une table** !

Pour créer une vue matérialisée avec MySQL, on utilise donc tout simplement une table, dans laquelle on stocke les résultats d'une requête `SELECT`.

**Exemple** : on veut matérialiser _V_Revenus_annee_espece_.


```sql
CREATE TABLE VM_Revenus_annee_espece
ENGINE = InnoDB
SELECT YEAR(date_reservation) AS annee, Espece.id AS espece_id, SUM(Adoption.prix) AS somme, COUNT(Adoption.animal_id) AS nb
FROM Adoption
INNER JOIN Animal ON Animal.id = Adoption.animal_id
INNER JOIN Espece ON Animal.espece_id = Espece.id
GROUP BY annee, Espece.id;
```



[[information]]
| Par facilité, on a utilisé ici la commande `CREATE TABLE ... SELECT`. Mais il est tout à fait possible de créer la table avec la commande habituelle, et de la remplir par la suite, avec un `INSERT INTO... SELECT` par exemple. Il s'agit d'une table tout à fait normale. Ce n'est que l'usage auquel elle est destinée qui en fait une vue matérialisée.


On a maintenant matérialisé la vue _V_Revenus_annee_espece_, avec _VM_Revenus_annee_espece_.


[[question]]
| Et c'est tout ?

Non bien sûr. C'est bien beau d'avoir créé une table, mais il va falloir aussi tenir cette vue matérialisée à jour.

*[SGBD]: Système de Gestion de Base de Données