On l'a dit, il ne suffit pas de créer une vue matérialisée à partir des données, il faut aussi la tenir à jour lorsque les données des tables d'origine changent.

Deux possibilités :


- une mise à jour sur demande ;
- une mise à jour automatique chaque fois qu'un changement est fait.



# Mise à jour sur demande

Si l'on veut pouvoir mettre la vue matérialisée à jour ponctuellement, on peut utiliser une procédure stockée. Le plus simple sera de supprimer les données de la vue matérialisée, puis d'insérer les données à jour grâce à la même requête de sélection ayant servi à créer/initialiser la vue matérialisée.

**Exemple** : procédure stockée permettant de mettre à jour la vue _VM_Revenus_annee_especes_.  


```sql
DELIMITER |
CREATE PROCEDURE maj_vm_revenus()
BEGIN
    TRUNCATE VM_Revenus_annee_espece;

    INSERT INTO VM_Revenus_annee_espece
    SELECT YEAR(date_reservation) AS annee, Espece.id AS espece_id, SUM(Adoption.prix) AS somme, COUNT(Adoption.animal_id) AS nb
    FROM Adoption
    INNER JOIN Animal ON Animal.id = Adoption.animal_id
    INNER JOIN Espece ON Animal.espece_id = Espece.id
    GROUP BY annee, Espece.id;
END |
DELIMITER ;
```


La commande `TRUNCATE nom_table`; a le même effet que `DELETE FROM nom_table;` (sans clause `WHERE`) ; elle supprime toutes les lignes de la table. Cependant, `TRUNCATE` est un peu différent de `DELETE FROM`.


- `TRUNCATE` ne supprime pas les lignes une à une : `TRUNCATE` supprime la table, puis la recrée (sans les données).
- Par conséquent, `TRUNCATE` ne traite pas les clés étrangères : on ne peut pas faire un `TRUNCATE` sur une table dont une colonne est référencée par une clé étrangère, sauf si celle-ci est dans la même table ; en outre les options `ON DELETE` ne sont pas traitées lorsque des données sont supprimées avec cette commande.
- `TRUNCATE` valide implicitement les transactions, et ne peut pas être annulé par un rollback.
- Pour toutes ces raisons, `TRUNCATE` est beaucoup plus rapide que `DELETE FROM` (en particulier si le nombre de ligne à supprimer est important).


Dans le cas de _VM_Revenus_annee_espece_, on peut sans problème utiliser `TRUNCATE`.

Pour mettre à jour la vue matérialisée, il suffit donc d'exécuter la procédure stockée.


```sql
CALL maj_vm_revenus();
```


Ce type de mise à jour est généralement utilisé pour des données qu'il n'est pas impératif d'avoir à jour en permanence. La plupart du temps, la procédure est appelée par un script qui tourne périodiquement (une fois par jour ou une fois par semaine par exemple).


# Mise à jour automatique

Si à l'inverse, on veut que les données soient toujours à jour par rapport aux derniers changements de la base de données, on utilisera plutôt les triggers pour mettre à jour la vue matérialisée.

Dans le cas de _VM_Revenus_annee_espece_, la vue matérialisée doit être mise à jour en cas de modification de la table _Adoption_.


- Une insertion provoquera la mise à jour de la ligne correspondant à l'année et à l'espèce de l'adoption insérée (majoration du total des prix et du nombre d'adoptions), ou insérera une nouvelle ligne si elle n'existe pas encore.
- Une suppression provoquera la mise à jour de la ligne correspondant à l'année et à l'espèce de l'adoption supprimée, ou la suppression de celle-ci s'il s'agissait de la seule adoption correspondante.
- Une modification sera un mix de la suppression et de l'insertion.


En ce qui concerne la colonne _espece_id_ de la vue matérialisée, il vaut mieux lui ajouter une clé étrangère, avec l'option `ON DELETE CASCADE`. En principe, _Espece.id_ ne devrait jamais être modifiée, mais en mettant cette clé étrangère, on s'assure que la correspondance entre les tables existera toujours.

On va également ajouter une clé primaire : (_annee, espece_id_), afin de simplifier nos triggers. Ça permettra d'utiliser la commande `INSERT INTO ... ON DUPLICATE KEY UPDATE`.


```sql
ALTER TABLE VM_Revenus_annee_espece
    ADD CONSTRAINT fk_vm_revenu_espece_id FOREIGN KEY (espece_id) REFERENCES Espece (id) ON DELETE CASCADE,
    ADD PRIMARY KEY (annee, espece_id);
```


Et voici donc les triggers nécessaires à la mise à jour de la vue matérialisée :


[[attention]]
| Ces trois triggers existaient déjà, et permettaient de mettre à jour la colonne _Animal.disponible_. Il faut donc d'abord les supprimer, puis les recréer en ajoutant la mise à jour de _VM_Revenus_annee_espece_.



```sql
DELIMITER |

DROP TRIGGER after_insert_adoption |
CREATE TRIGGER after_insert_adoption AFTER INSERT
ON Adoption FOR EACH ROW
BEGIN
    UPDATE Animal
    SET disponible = FALSE
    WHERE id = NEW.animal_id;

    INSERT INTO VM_Revenus_annee_espece (espece_id, annee, somme, nb)
    SELECT espece_id, YEAR(NEW.date_reservation), NEW.prix, 1
    FROM Animal
    WHERE id = NEW.animal_id
    ON DUPLICATE KEY UPDATE somme = somme + NEW.prix, nb = nb + 1;
END |

DROP TRIGGER after_update_adoption |
CREATE TRIGGER after_update_adoption AFTER UPDATE
ON Adoption FOR EACH ROW
BEGIN
    IF OLD.animal_id <> NEW.animal_id THEN
        UPDATE Animal
        SET disponible = TRUE
        WHERE id = OLD.animal_id;

        UPDATE Animal
        SET disponible = FALSE
        WHERE id = NEW.animal_id;
    END IF;

    INSERT INTO VM_Revenus_annee_espece (espece_id, annee, somme, nb)
    SELECT espece_id, YEAR(NEW.date_reservation), NEW.prix, 1
    FROM Animal
    WHERE id = NEW.animal_id
    ON DUPLICATE KEY UPDATE somme = somme + NEW.prix, nb = nb + 1;

    UPDATE VM_Revenus_annee_espece
    SET somme = somme - OLD.prix, nb = nb - 1
    WHERE annee = YEAR(OLD.date_reservation)
    AND espece_id = (SELECT espece_id FROM Animal WHERE id = OLD.animal_id);
 
    DELETE FROM VM_Revenus_annee_espece
    WHERE nb = 0;
END |

DROP TRIGGER after_delete_adoption |
CREATE TRIGGER after_delete_adoption AFTER DELETE
ON Adoption FOR EACH ROW
BEGIN
    UPDATE Animal
    SET disponible = TRUE
    WHERE id = OLD.animal_id;

    UPDATE VM_Revenus_annee_espece
    SET somme = somme - OLD.prix, nb = nb - 1
    WHERE annee = YEAR(OLD.date_reservation)
    AND espece_id = (SELECT espece_id FROM Animal WHERE id = OLD.animal_id);
 
    DELETE FROM VM_Revenus_annee_espece
    WHERE nb = 0;
END |

DELIMITER ;
```