# Tables _vs_ vue _vs_ vue matérialisée

On va tester les performances des vues et des vues matérialisées en créant trois procédures stockées différentes, répondant à la même question : quelle est l'année ayant rapporté le plus en termes d'adoption de chats ? 
Les trois procédures utiliseront des objets différents :


- l'une fera la requête directement sur les tables ;
- l'autre fera la requête sur la vue ;
- la dernière utilisera la vue matérialisée.


Nos tables contenant très peu de données, la requête sera répétée un millier de fois, afin que les temps d'exécution soient utilisables.


## Empêcher MySQL d'utiliser le cache

Lorsqu'on exécute la même requête plusieurs fois de suite, MySQL ne l'exécute en réalité qu'une seule fois, et stocke le résultat en cache. Toutes les fois suivantes, MySQL ressort simplement ce qu'il a en cache, ce qui ne prend quasiment rien comme temps. 
Pour faire des tests de performance, cela pose évidemment problème, puisque cela fausse le temps d'exécution. Heureusement, il existe une clause spéciale, qui permet d'empêcher MySQL de stocker le résultat de la requête en cache : `SQL_NO_CACHE`.


# Les trois procédures


## Sur les tables

On utilise `SELECT... INTO` afin de stocker les résultats dans des variables, plutôt que de les afficher mille fois.


```sql
DELIMITER |
CREATE PROCEDURE test_perf_table()
BEGIN
    DECLARE v_max INT DEFAULT 1000;                     
    DECLARE v_i INT DEFAULT 0;
    DECLARE v_nb INT;
    DECLARE v_somme DECIMAL(15,2);
    DECLARE v_annee CHAR(4);
 
    boucle: LOOP
        IF v_i = v_max THEN LEAVE boucle; END IF;         -- Condition d'arrêt de la boucle

        SELECT SQL_NO_CACHE YEAR(date_reservation) AS annee, 
                            SUM(Adoption.prix) AS somme,
                            COUNT(Adoption.animal_id) AS nb
               INTO v_annee, v_somme, v_nb
        FROM Adoption
        INNER JOIN Animal ON Animal.id = Adoption.animal_id
        INNER JOIN Espece ON Animal.espece_id = Espece.id
        WHERE Espece.id = 2
        GROUP BY annee
        ORDER BY somme DESC
        LIMIT 1;
 
        SET v_i = v_i + 1;
    END LOOP;
    
END |
DELIMITER ;
```



## Sur la vue


```sql
DELIMITER |
CREATE PROCEDURE test_perf_vue()
BEGIN
    DECLARE v_max INT DEFAULT 1000;                   
    DECLARE v_i INT DEFAULT 0;
    DECLARE v_nb INT;
    DECLARE v_somme DECIMAL(15,2);
    DECLARE v_annee CHAR(4);

    boucle: LOOP
        IF v_i = v_max THEN LEAVE boucle; END IF;    

        SELECT SQL_NO_CACHE annee, somme, nb         
               INTO v_annee, v_somme, v_nb                                          
        FROM V_Revenus_annee_espece
        WHERE espece_id = 2
        ORDER BY somme DESC
        LIMIT 1;
 
        SET v_i = v_i + 1;
    END LOOP;
    
END |
DELIMITER ;
```




## Sur la vue matérialisée


```sql
DELIMITER |
CREATE PROCEDURE test_perf_vm()
BEGIN
    DECLARE v_max INT DEFAULT 1000;                   
    DECLARE v_i INT DEFAULT 0;
    DECLARE v_nb INT;
    DECLARE v_somme DECIMAL(15,2);
    DECLARE v_annee CHAR(4);

    boucle: LOOP
        IF v_i = v_max THEN LEAVE boucle; END IF;    

        SELECT SQL_NO_CACHE annee, somme, nb         
               INTO v_annee, v_somme, v_nb                                          
        FROM VM_Revenus_annee_espece
        WHERE espece_id = 2
        ORDER BY somme DESC
        LIMIT 1;
 
        SET v_i = v_i + 1;
    END LOOP;
    
END |
DELIMITER ;
```



# Le test

Il n'y a plus maintenant qu'à exécuter les trois procédures pour voir leur temps d'exécution.


```sql
CALL test_perf_table();
CALL test_perf_vue();
CALL test_perf_vm();
```


```console
mysql> CALL test_perf_table();
Query OK, 1 row affected (0.27 sec)

mysql> CALL test_perf_vue();
Query OK, 1 row affected (0.29 sec)

mysql> CALL test_perf_vm();
Query OK, 1 row affected (0.06 sec)
```


Le temps d'exécution semble équivalent pour la requête faite sur les tables directement, et sur la vue. Par contre, on a un gros gain de performance avec la requête sur la vue matérialisée.

Voici les moyennes et écarts-types des temps d'exécution des trois procédures, pour 20 exécutions :




+--------------+----------+--------+--------------------+
|              |Sur tables|Sur vue |Sur vue matérialisée|
+==============+==========+========+====================+
|**Moyenne**   |0,266 s   |0,2915 s|0,058               |
+--------------+----------+--------+--------------------+
|**Écart-type**|0,00503   |0,00366 |0,00410             |
+--------------+----------+--------+--------------------+





La requête sur la vue est donc bien légèrement plus lente que la requête faite directement sur les tables. Mais le résultat le plus intéressant est bien celui obtenu avec la vue matérialisée : 5 fois plus rapide que la requête sur la vue !

Par ailleurs, on peut maintenant ajouter des clés et des index à _VM_Revenus_annee_especes_ pour accélérer encore les requêtes. Ici, le nombre réduit de lignes fait que l'effet ne sera pas perceptible, mais sur des tables plus fournies, ça vaut la peine !


```sql
ALTER TABLE VM_Revenus_annee_espece ADD INDEX (somme);
```



# Conclusion

Nos tables contiennent très peu de données, par rapport à la réalité. Les applications moyennes classiques exploitent des bases de données dont les tables peuvent compter des centaines de milliers, voire des millions de lignes. Le gain de performance permis par les vues matérialisées, et les index, sera encore plus visible sur de telles bases.

Cependant, **toutes les vues ne sont pas intéressantes à matérialiser**. De manière générale, une vue contenant des fonctions d'agrégation, des regroupements, et qui donc devra utiliser l'algorithme `TEMPTABLE`, est une bonne candidate.

Mais il faut également prendre en compte le coût de la mise à jour de la vue matérialisée. 
S'il s'agit d'une vue qui doit être constamment à jour, et qui utiliserait donc des triggers, il ne sera sans doute pas intéressant de la matérialiser si les données sur laquelle elle s'appuie sont souvent modifiées, car **le gain de performance pour les requêtes de sélection sur la vue ne compenserait pas  le coût des mises à jour de celle-ci**.

Par ailleurs, **certaines vues ne nécessitent pas qu'on leur consacre une table entière**. Ainsi, la vue _V_Nombre_espece_ peut être matérialisée simplement en ajoutant une colonne à la table _Espece_. Cette colonne peut alors être tenue à jour avec des triggers sur _Animal_ (comme on le fait avec la colonne _Animal.disponible_).

Comme souvent, il faut donc bien choisir au cas par cas, peser le pour et le contre et réfléchir à toutes les implications quant au choix des vues à matérialiser.