# Les privilèges ALL, USAGE et GRANT OPTION

Pour terminer avec les différents privilèges, nous allons parler de trois privilèges un peu particuliers.


## ALL

Le privilège `ALL` (ou `ALL PRIVILEGES`), comme son nom l'indique, représente tous les privilèges. Accorder le privilège `ALL` revient donc à accorder tous les droits à l'utilisateur. Il faut évidemment préciser le niveau auquel tous les droits sont accordés (on octroie tous les privilèges possibles sur une table, ou sur une base de données, etc.). 


[[information]]
| Un privilège fait exception  : `GRANT OPTION` n'est pas compris dans les privilèges représentés par `ALL`.


**Exemple** : on accorde tous les droits sur la table _Client_ à `'john'@'localhost'`.


```sql
GRANT ALL 
ON elevage.Client
TO 'john'@'localhost';
```


## USAGE

À l'inverse de `ALL`, le privilège `USAGE` signifie "aucun privilège".
À première vue, utiliser la commande `GRANT` pour n'accorder aucun privilège peut sembler un peu ridicule. En réalité, c'est extrêmement utile : `USAGE` permet en fait de modifier les caractéristiques d'un compte avec la commande `GRANT`, sans modifier les privilèges du compte.
`USAGE` est toujours utilisé comme un privilège global (donc `ON *.*`).

**Exemple** : modification du mot de passe de `'john'@'localhost'`. Ses privilèges ne changent pas.


```sql
GRANT USAGE
ON *.*
TO 'john'@'localhost' IDENTIFIED BY 'test2012usage';
```


Nous verrons plus tard les autres options de la commande `GRANT`, que l'on pourrait vouloir modifier en utilisant le privilège `USAGE`.


## GRANT OPTION

Nous voici donc au fameux privilège `GRANT OPTION`. Un utilisateur ayant ce privilège est autorisé à utiliser la commande `GRANT`, pour accorder des privilèges à d'autres utilisateurs.
Ce privilège n'est pas compris dans le privilège `ALL`. Par ailleurs, un utilisateur ne peut accorder que les privilèges qu'il possède lui-même.

On peut accorder `GRANT OPTION` de deux manières :


- comme un privilège normal, après le mot `GRANT` ;
- à la fin de la commande `GRANT`, avec la clause `WITH GRANT OPTION`.


**Exemple** : on accorde les privilèges `SELECT`, `UPDATE`, `INSERT`, `DELETE` et `GRANT OPTION` sur la base de données _elevage_ à `'joseph'@'localhost'`.


```sql
GRANT SELECT, UPDATE, INSERT, DELETE, GRANT OPTION
ON elevage.*
TO 'joseph'@'localhost' IDENTIFIED BY 'ploc4';

-- OU

GRANT SELECT, UPDATE, INSERT, DELETE
ON elevage.*
TO 'joseph'@'localhost' IDENTIFIED BY 'ploc4'
WITH GRANT OPTION;
```



[[attention]]
| Le privilège `ALL` doit s'utiliser tout seul. Il n'est donc pas possible d'accorder `GRANT OPTION` et `ALL` de la manière suivante : `GRANT ALL, GRANT OPTION…` Il est nécessaire dans ce cas d'utiliser `WITH GRANT OPTION`.



# Particularité des triggers, vues et procédures stockées

Les triggers, les vues et les procédures stockées (ainsi que les événements et les fonctions stockées, non couvertes par ce cours) ont un système spécial quant à la vérification des privilèges des utilisateurs.
En effet, ces objets sont créés dans le but d'être exécutés dans le futur, et l'utilisateur créant un tel objet pourrait bien être différent de l'utilisateur se servant de l'objet. Il y a donc deux types d'utilisateurs potentiels pour ces types d'objet : celui ayant défini l'objet, et celui utilisant l'objet. Quels privilèges faut-il vérifier lorsqu'une procédure est exécutée ? Lorsque la requête `SELECT` d'une vue est exécutée ?

Par défaut, ce sont les privilèges du définisseur (celui qui a défini l'objet) qui sont vérifiés. Ce qui veut dire qu'un utilisateur pourrait exécuter une procédure agissant sur des tables sur lesquelles il n'a lui-même aucun privilège. L'important étant les privilèges de l'utilisateur ayant défini la procédure. Bien entendu, un utilisateur doit toujours avoir le privilège `EXECUTE` pour exécuter une procédure stockée, les privilèges du définisseur concernent les instructions à l'intérieur de la procédure ou du trigger (ou de la requête `SELECT` pour une vue).

**Exemple** : avec l'utilisateur _sdz_, on définit une procédure faisant une requête `SELECT` sur la table _Adoption_. On exécute ensuite cette procédure avec l'utilisateur _john_, qui n'a aucun droit sur la table _Adoption_.

Utilisateur _sdz_ :

```sql
USE elevage;
DELIMITER |
CREATE PROCEDURE test_definer()
BEGIN
    SELECT * FROM Adoption;
END |
DELIMITER ;
```


Utilisateur _john_ :

```sql
USE elevage;
SELECT * FROM Adoption;
CALL test_definer();
```


L'utilisateur _john_ n'a aucun droit sur _Adoption_. La requête `SELECT` échoue donc avec le message d'erreur suivant :


```console
ERROR 1142 (42000): SELECT command denied to user 'john'@'localhost' for table 'adoption'
```


Par contre, _john_ a le droit d'exécuter les procédures de la base de données _elevage_. Il exécute donc sans problème _test_definer()_, qui lui affiche le contenu d'_Adoption_. Les privilèges vérifiés au niveau des instructions exécutées par la procédure sont en effet ceux de l'utilisateur _sdz_, ayant défini celle-ci.


## Préciser et modifier le définisseur

Les commandes de création des vues, triggers et procédures stockées permettent de préciser une clause `DEFINER`, dans laquelle on précise l'identifiant d'un utilisateur.
Par défaut, c'est l'utilisateur courant qui est utilisé.
Cependant il est possible de donner un autre utilisateur comme définisseur de l'objet, à condition d'avoir le privilège global `SUPER`. Sans ce privilège, on ne peut donner comme `DEFINER` que soi-même, soit avec `CURRENT_USER` ou `CURRENT_USER()`, soit avec l'identifiant (`'sdz'@'localhost'` par exemple).
La clause `DEFINER` se place après le mot-clé `CREATE`.

**Exemple** : définition de deux procédures stockées avec l'utilisateur _root_ (le seul ayant le privilège `SUPER` sur notre serveur), l'une avec _root_ pour `DEFINER` (`CURRENT_USER()`), l'autre avec _john_.


```sql
DELIMITER |
CREATE DEFINER = CURRENT_USER() PROCEDURE test_definer2()
BEGIN
    SELECT * FROM Race;
END |

CREATE DEFINER = 'john'@'localhost' PROCEDURE test_definer3()
BEGIN
    SELECT * FROM Race;
END |
DELIMITER ;
```


Si l'on exécute ces deux procédures avec l'utilisateur _john_, on obtient deux résultats différents.


```sql
CALL test_definer2();
CALL test_definer3();
```


La première procédure s'exécute sans problème. _john_ n'a aucun droit sur la table _Race_, mais le définisseur de _test_definer2()_ étant _root_, ce sont ses privilèges qui comptent. Par contre, la seconde échoue, car le définisseur de _test_definer3()_ a été initialisé à _john_.


## Modification du contexte

Il est possible, pour les vues et les procédures stockées, de changer la manière dont les privilèges sont vérifiés à l'exécution. On peut faire en sorte que ce soit les privilèges de l'utilisateur qui se sert de la vue ou de la procédure ("l'invocateur") qui soient vérifiés, et non plus les privilèges du définisseur.

Ce n'est pas possible pour les triggers, car ceux-ci ne sont pas exécutés directement par un utilisateur, mais par une action de l'utilisateur (insertion, modification, suppression dans la table sur laquelle le trigger est défini).

Pour changer le contexte de vérification des privilèges d'une vue ou d'une procédure, il faut utiliser la clause `SQL SECURITY {DEFINER | INVOKER}`.

**Syntaxe de création des vues et des procédures**, clauses `DEFINER` et `SQL SECURITY` comprises.


```sql
-- Vues
CREATE [OR REPLACE]
    [ALGORITHM = {UNDEFINED | MERGE | TEMPTABLE}]
    [DEFINER = { utilisateur | CURRENT_USER }]
    [SQL SECURITY { DEFINER | INVOKER }]
    VIEW nom_vue [(liste_colonnes)]
    AS requete_select
    [WITH [CASCADED | LOCAL] CHECK OPTION]

-- Procédures
CREATE
    [DEFINER = { utilisateur | CURRENT_USER }]
    PROCEDURE nom_procedure ([parametres_procedure])
    SQL SECURITY { DEFINER | INVOKER }
    corps_procedure
```


**Exemple** : création, par l'utilisateur _root_, de deux vues avec des contextes de vérification des privilèges différents.


```sql
CREATE DEFINER = CURRENT_USER
       SQL SECURITY DEFINER
       VIEW test_contexte1
AS SELECT * FROM Race;

CREATE DEFINER = CURRENT_USER
       SQL SECURITY INVOKER
       VIEW test_contexte2
AS SELECT * FROM Race;
```


Toujours avec l'utilisateur _root_, on autorise _john_ à faire des `SELECT` sur ces vues.


```sql
GRANT SELECT ON elevage.test_contexte1 TO 'john'@'localhost';
GRANT SELECT ON elevage.test_contexte2 TO 'john'@'localhost';
```

Utilisons maintenant ces vues avec l'utilisateur _john_, qui n'a toujours aucun droit sur _Race_. 


```sql
SELECT * FROM test_contexte1;
SELECT * FROM test_contexte2;
```


La première requête affiche bien la table _Race_. Par contre, la seconde échoue avec l'erreur suivante :


```console
ERROR 1356 (HY000): View 'elevage.test_contexte2' references invalid table(s) or column(s) or function(s) or definer/invoker of view lack rights to use them
```