On a vu comment obtenir des informations sur les objets de nos bases de données, voyons maintenant comment obtenir des **informations sur les requêtes que l'on exécute** sur nos bases de données.

Plus particulièrement, nous allons nous attarder sur la commande `EXPLAIN`, qui permet de décortiquer l'exécution d'une requête `SELECT`. Grâce à cette commande, il est possible de savoir quelles tables et quels index sont utilisés, et dans quel ordre.

L'utilisation de cette commande est extrêmement simple : il suffit d'ajouter `EXPLAIN` devant la requête `SELECT` que l'on désire examiner.

**Exemple**


```sql
EXPLAIN SELECT Animal.nom, Espece.nom_courant AS espece, Race.nom AS race
FROM Animal
INNER JOIN Espece ON Animal.espece_id = Espece.id
LEFT JOIN Race ON Animal.race_id = Race.id
WHERE Animal.id = 37;
```




+--+-----------+------+-----+--------------------+-------+-------+-----+----+-----+
|id|select_type|table |type |possible_keys       |key    |key_len|ref  |rows|Extra|
+==+===========+======+=====+====================+=======+=======+=====+====+=====+
|1 |SIMPLE     |Animal|const|PRIMARY,fk_espece_id|PRIMARY|2      |const|1   |     |
+--+-----------+------+-----+--------------------+-------+-------+-----+----+-----+
|1 |SIMPLE     |Espece|const|PRIMARY             |PRIMARY|2      |const|1   |     |
+--+-----------+------+-----+--------------------+-------+-------+-----+----+-----+
|1 |SIMPLE     |Race  |const|PRIMARY             |PRIMARY|2      |const|1   |     |
+--+-----------+------+-----+--------------------+-------+-------+-----+----+-----+






- **_id_** : cette colonne identifie la requête `SELECT` concernée par l'étape. Ici, il n'y en a qu'une, mais dans le cas d'une requête avec des sous-requêtes, ou avec un `UNION`, il peut y avoir plusieurs requêtes `SELECT` différentes.
- _**select_type**_ : le type de la requête `SELECT` concernée par l'étape.




+--------------------+----------------------------------------------------------------------------------------------------------------------------+
|Type                |Explication                                                                                                                 |
+====================+============================================================================================================================+
|SIMPLE              |Un simple `SELECT`, sans sous-requête ou `UNION`.                                                                           |
+--------------------+----------------------------------------------------------------------------------------------------------------------------+
|PRIMARY             |`SELECT` extérieur, c'est-à-dire le premier, le principal `SELECT` (en présence de sous-requêtes ou `UNION`).               |
+--------------------+----------------------------------------------------------------------------------------------------------------------------+
|UNION               |Seconde requête `SELECT` d'un `UNION` (et requêtes suivantes).                                                              |
+--------------------+----------------------------------------------------------------------------------------------------------------------------+
|DEPENDENT UNION     |Comme UNION, mais requête dépendant d'une requête `SELECT` externe (à cause d'une sous-requête corrélée par exemple).       |
+--------------------+----------------------------------------------------------------------------------------------------------------------------+
|UNION RESULT        |Résultat d'une `UNION`.                                                                                                     |
+--------------------+----------------------------------------------------------------------------------------------------------------------------+
|SUBQUERY            |Première requête `SELECT` d'une sous-requête.                                                                               |
+--------------------+----------------------------------------------------------------------------------------------------------------------------+
|DEPEDENT SUBQUERY   |Première requête `SELECT` d'une sous-requête, dépendant d'une requête externe.                                              |
+--------------------+----------------------------------------------------------------------------------------------------------------------------+
|DERIVED             |Table dérivée (résultat d'une sous-requête).                                                                                |
+--------------------+----------------------------------------------------------------------------------------------------------------------------+
|UNCACHEABLE SUBQUERY|Sous-requête dont le résultat ne peut être mis en cache et doit donc être réévalué pour chaque ligne de la requête `SELECT`.|
+--------------------+----------------------------------------------------------------------------------------------------------------------------+
|UNCACHEABLE UNION   |Deuxième (ou plus) requête `SELECT` dans une `UNION` faite dans une sous-requête qui ne peut être mise en cache.            |
+--------------------+----------------------------------------------------------------------------------------------------------------------------+



- **_table_** : le nom de la table sur laquelle l'étape est réalisée.
- **_type_** : le type de jointure utilisée par l'étape. Pour un détail des valeurs possibles, je vous renvoie à la documentation officielle. Les voici cependant classées du meilleur (du point de vue de la performance) au moins bon : _system, const, eq_ref, ref, fulltext, ref_or_null, index_merge, unique_subquery, index_subquery, range, index, ALL_.
- **_possible_keys_** : les index que MySQL a envisagé d'utiliser pour l'étape.
- **_key_** : l'index effectivement utilisé
- **_key_len_** : la taille de l'index utilisé (peut être inférieure à la longueur de l'index en cas d'utilisation d'un index par la gauche).
- **_ref_** : indique à quoi l'index sera comparé : une constante (_const_) ou une colonne.
- **_rows_** : le nombre **estimé** de lignes que MySQL devra parcourir pour terminer l'étape (plus ce nombre est petit, mieux c'est).
- **_Extra_** : donne des informations supplémentaires sur l'étape.


Dans le cas de notre requête `SELECT`, on a donc 3 étapes :


- La clé primaire d'_Animal_ est utilisée et permet de trouver tout de suite la ligne correspondant à notre recherche (`id = 37`). On ne doit donc parcourir qu'une seule ligne.
- En utilisant la valeur d'_Animal.espece_id_ trouvée à l'étape 1, on trouve la ligne correspondante dans la table _Espece_, à nouveau en utilisant la clé primaire.
- Même chose pour la table _Race_.



[[question]]
| Mais quel est l'intérêt de savoir cela ?

L'intérêt est de pouvoir **optimiser ses requêtes**. Soit en ajoutant un ou plusieurs index, soit en trouvant la manière optimale d'écrire la requête.


## Savoir sur quelle colonne ajouter un index

Lorsqu'une requête est un peu lente, il est souvent possible de l'accélérer en ajoutant un ou plusieurs index à des endroits stratégiques. En utilisant `EXPLAIN`, on peut découvrir facilement quelles étapes de la requête n'utilisent pas d'index, et donc sur quelles colonnes il peut être intéressant d'ajouter un index.

**Exemple**


```sql
EXPLAIN SELECT Animal.nom, Adoption.prix, Adoption.date_reservation
FROM Animal 
INNER JOIN Adoption ON Adoption.animal_id = Animal.id
WHERE date_reservation >= '2012-05-01' \G
```


```sql
*************************** 1. row ***************************
           id: 1
  select_type: SIMPLE
        table: Adoption
         type: ALL
possible_keys: ind_uni_animal_id
          key: NULL
      key_len: NULL
          ref: NULL
         rows: 24
        Extra: Using where
*************************** 2. row ***************************
           id: 1
  select_type: SIMPLE
        table: Animal
         type: eq_ref
possible_keys: PRIMARY
          key: PRIMARY
      key_len: 2
          ref: elevage7.Adoption.animal_id
         rows: 1
        Extra:
```


Pour la première étape, _key_, _key_len_ et _ref_ sont `NULL`. Aucun index n'est donc utilisé. type vaut ALL, ce qui est la valeur la moins intéressante. Enfin, _rows_ vaut 24, ce qui est le nombre de ligne dans la table _Adoption_. Toutes les lignes doivent être parcourues pour trouver les lignes correspondant à la clause `WHERE`.
La deuxième étape par contre utilise bien un index (pour faire la jointure avec _Animal_).

En ajoutant un index sur _Adoption.date_reservation_, on peut améliorer les performances de cette requête.


```sql
ALTER TABLE Adoption ADD INDEX ind_date_reservation (date_reservation);
```


La même commande `EXPLAIN` donnera désormais le résultat suivant :


```sql
*************************** 1. row ***************************
           id: 1
  select_type: SIMPLE
        table: Adoption
         type: range
possible_keys: ind_uni_animal_id,ind_date_reservation
          key: ind_date_reservation
      key_len: 3
          ref: NULL
         rows: 4
        Extra: Using where
*************************** 2. row ***************************
           id: 1
  select_type: SIMPLE
        table: Animal
         type: eq_ref
possible_keys: PRIMARY
          key: PRIMARY
      key_len: 2
          ref: elevage7.Adoption.animal_id
         rows: 1
        Extra:
```


La première étape utilise l'index, et ne doit donc plus parcourir toutes les lignes, mais seulement une partie (dont le nombre est estimé à 4).


[[information]]
| Attention de ne pas tomber dans l'excès en mettant des index partout. En général, on utilise `EXPLAIN` sur des requêtes lourdes, dont on sait qu'elles ralentissent l'application. Il n'est pas nécessaire d'optimiser la moindre petite requête `SELECT`.



## Comparer le plan d'exécution de plusieurs requêtes

Lorsque l'on fait une condition sur une colonne (dans une clause `WHERE` ou pour une condition de jointure), ce n'est pas parce qu'il existe un index sur celle-ci qu'il sera utilisé par la requête. En particulier, si la colonne est utilisée dans une expression, l'index ne sera pas utilisé, car la valeur de l'expression devra être calculée pour chaque ligne.
Selon la manière d'écrire une condition donc, l'utilisation des index sera possible, ou non. Lorsqu'on hésite entre plusieurs écritures, utiliser `EXPLAIN` peut permettre d'utiliser la requête la plus performante.

**Exemple**


```sql
EXPLAIN SELECT * 
FROM VM_Revenus_annee_espece
WHERE somme/2 > 1000 \G

EXPLAIN SELECT * 
FROM VM_Revenus_annee_espece
WHERE somme > 1000*2 \G
```


Ces deux requêtes `SELECT` produisent un résultat équivalent, mais la première empêche l'utilisation de l'index sur _somme_, contrairement à la deuxième.


```sql
*************************** 1. row ***************************
           id: 1
  select_type: SIMPLE
        table: VM_Revenus_annee_espece
         type: ALL
possible_keys: NULL
          key: NULL
      key_len: NULL
          ref: NULL
         rows: 16
        Extra: Using where


*************************** 1. row ***************************
           id: 1
  select_type: SIMPLE
        table: VM_Revenus_annee_espece
         type: range
possible_keys: somme
          key: somme
      key_len: 14
          ref: NULL
         rows: 2
        Extra: Using where
```