Nous avons vu qu'il était possible d'exploiter le résultat d'un `SELECT` dans un bloc d'instructions, en utilisant la commande `SELECT colonne(s) INTO variable(s)`, qui assigne les valeurs sélectionnées à des variables.
Cependant, `SELECT ... INTO` ne peut être utilisé que pour des requêtes qui ne ramènent qu'une seule ligne de résultats.

Les curseurs permettent de **parcourir un jeu de résultats d'une requête `SELECT`**, quel que soit le nombre de lignes récupérées, et d'en exploiter les valeurs.

Quatre étapes sont nécessaires pour utiliser un curseur.


- Déclaration du curseur : avec une instruction `DECLARE.`
- Ouverture du curseur : on exécute la requête `SELECT` du curseur et on stocke le résultat dans celui-ci.
- Parcours du curseur : on parcourt une à une les lignes.
- Fermeture du curseur.



# Syntaxe


## Déclaration du curseur

Comme toutes les instructions `DECLARE`, la déclaration d'un curseur doit se faire au début du bloc d'instructions pour lequel celui-ci est défini. Plus précisément, on déclare les curseurs **après les variables locales et les conditions**, mais **avant les gestionnaires d'erreur**.


```sql
DECLARE nom_curseur CURSOR FOR requete_select;
```


Un curseur est donc composé d'**un nom**, et d'**une requête `SELECT`**.

**Exemple** :


```sql
DECLARE curseur_client CURSOR 
    FOR SELECT * 
    FROM Client;
```



## Ouverture et fermeture du curseur

En déclarant le curseur, on a donc associé un nom et une requête `SELECT`. L'ouverture du curseur va provoquer l'exécution de la requête `SELECT`, ce qui va produire un jeu de résultats.

Une fois qu'on aura parcouru les résultats, il n'y aura plus qu'à fermer le curseur. Si on ne le fait pas explicitement, le curseur sera fermé à la fin du bloc d'instructions.


```sql
OPEN nom_curseur;
  -- Parcours du curseur et instructions diverses
CLOSE nom_curseur;
```



## Parcours du curseur

Une fois que le curseur a été ouvert et le jeu de résultats récupéré, le curseur place un pointeur sur la première ligne de résultats. Avec la commande `FETCH`, on récupère la ligne sur laquelle pointe le curseur, et on fait avancer le pointeur vers la ligne de résultats suivante.


```sql
FETCH nom_curseur INTO variable(s);
```

Bien entendu, comme pour `SELECT ... INTO`, il faut donner autant de variables dans la clause `INTO` qu'on a récupéré de colonnes dans la clause `SELECT` du curseur.

**Exemple** : la procédure suivante parcourt les deux premières lignes de la table _Client_ avec un curseur.


```sql
DELIMITER |
CREATE PROCEDURE parcours_deux_clients()
BEGIN
    DECLARE v_nom, v_prenom VARCHAR(100);

    DECLARE curs_clients CURSOR
        FOR SELECT nom, prenom                                 -- Le SELECT récupère deux colonnes
        FROM Client
        ORDER BY nom, prenom;                                  -- On trie les clients par ordre alphabétique

    OPEN curs_clients;                                         -- Ouverture du curseur

    FETCH curs_clients INTO v_nom, v_prenom;                   -- On récupère la première ligne et on assigne les valeurs récupérées à nos variables locales
    SELECT CONCAT(v_prenom, ' ', v_nom) AS 'Premier client';

    FETCH curs_clients INTO v_nom, v_prenom;                   -- On récupère la seconde ligne et on assigne les valeurs récupérées à nos variables locales
    SELECT CONCAT(v_prenom, ' ', v_nom) AS 'Second client';

    CLOSE curs_clients;                                         -- Fermeture du curseur
END|
DELIMITER ;

CALL parcours_deux_clients();
```




+------------------+
|Premier client    |
+==================+
|Maximilien Antoine|
+------------------+







+-------------+
|Second client|
+=============+
|Marie Boudur |
+-------------+






# Restrictions

`FETCH` est la seule commande permettant de récupérer une partie d'un jeu de résultats d'un curseur, et elle ne permet qu'une chose : récupérer la ligne de résultats suivante.
Il n'est pas possible de sauter une ou plusieurs lignes, ni d'aller rechercher une ligne précédente. **On ne peut que parcourir les lignes une à une**, de la première à la dernière.

Ensuite, il n'est pas possible de modifier une ligne directement à partir d'un curseur. Il s'agit d'une restriction particulière à MySQL. D'autres SGBD vous permettent des requêtes d'`UPDATE` directement sur les curseurs.

Avec Oracle, la requête suivante modifiera la dernière ligne récupérée avec `FETCH` :


```sql
UPDATE nom_table
   SET colonne = valeur
 WHERE CURRENT OF nom_curseur;
```


[[erreur]]
| Ce n'est, du moins actuellement, absolument **pas possible** avec MySQL !!!


Il vaut d'ailleurs mieux éviter tout `UPDATE` sur une table sur laquelle un curseur est ouvert, même sans faire de référence à celui-ci. Le résultat d'un tel `UPDATE` serait imprévisible.


[[information]]
| Ces restrictions sur les requêtes `UPDATE` sont bien entendu également valables pour les suppressions (`DELETE`).


# Parcourir intelligemment tous les résultats d'un curseur

Pour récupérer une ligne de résultats, on utilise donc `FETCH`. Dans la procédure _parcours_deux_clients()_, on voulait récupérer les deux premières lignes, on a donc utilisé deux `FETCH`.
Cependant, la plupart du temps, on ne veut pas seulement utiliser les deux premières lignes, mais toutes ! Or, sauf exception, on ne sait pas combien de lignes seront sélectionnées.

On veut donc parcourir une à une les lignes de résultats, et leur appliquer un traitement, sans savoir à l'avance combien de fois ce traitement devra être répété.
Pour cela, on utilise une boucle ! `WHILE`, `REPEAT` ou `LOOP`. Il n'y a plus qu'à trouver une condition pour arrêter la boucle une fois tous les résultats parcourus.


## Condition d'arrêt

Voyons ce qui se passe lorsque l'on fait un `FETCH` alors qu'il n'y a plus, ou pas, de résultats.
Voici une procédure qui sélectionne les clients selon une ville donnée en paramètre. Les lignes sont récupérées et affichées grâce au `FETCH`, placé dans une boucle `LOOP`. Je rappelle que cette boucle ne définit pas de condition d'arrêt : il est nécessaire d'ajouter une instruction `LEAVE` pour l'arrêter. Ici, pour tester, on ne mettra pas d'instruction `LEAVE`.


```sql
DELIMITER |
CREATE PROCEDURE test_condition(IN p_ville VARCHAR(100))
BEGIN
    DECLARE v_nom, v_prenom VARCHAR(100);

    DECLARE curs_clients CURSOR
        FOR SELECT nom, prenom
        FROM Client
        WHERE ville = p_ville;

    OPEN curs_clients;                                    

    LOOP                                                  
        FETCH curs_clients INTO v_nom, v_prenom;                   
        SELECT CONCAT(v_prenom, ' ', v_nom) AS 'Client';
    END LOOP;

    CLOSE curs_clients; 
END|
DELIMITER ;
```


Voyons donc ce que ça donne pour une ville dans laquelle quelques clients habitent.


```sql
CALL test_condition('Houtsiplou');
```




+-----------+
|Client     |
+===========+
|Jean Dupont|
+-----------+







+--------------------+
|Client              |
+====================+
|Virginie Broussaille|
+--------------------+





```console
ERROR 1329 (02000): No data - zero rows fetched, selected, or processed
```


Tentons ensuite l'expérience avec une ville qui ne donnera aucun résultat.


```sql
CALL test_condition('Bruxelles');
```


```console
ERROR 1329 (02000): No data - zero rows fetched, selected, or processed
```


On a la même erreur dans les deux cas, lorsqu'on essaye de faire un `FETCH` alors qu'il n'y a pas ou plus de ligne à récupérer. Or, vous vous souvenez peut-être d'une condition prédéfinie pour les gestionnaires d'erreur, `NOT FOUND`, qui représente les erreurs dont l'identifiant SQL commence par `'02'`, ce qui est le cas ici.

On va donc utiliser cette condition pour arrêter la boucle : on définit un gestionnaire pour la condition `NOT FOUND`, qui change la valeur d'une variable locale. Cette variable locale vaut 0 au départ, et passe à 1 quand le gestionnaire est déclenché (donc quand il n'y a plus de ligne). Il suffit alors d'ajouter une structure `IF` qui vérifie la valeur de la variable locale une fois le `FETCH` exécuté. Si elle vaut 1, on quitte la boucle.

**Exemple** : la procédure _test_condition2()_ ci-dessous fait la même chose que _test_condition()_, mais inclut le gestionnaire d'erreur et le `IF` nécessaires pour stopper la boucle dès que toutes les lignes sélectionnées ont été parcourues :


```sql
DELIMITER |
CREATE PROCEDURE test_condition2(IN p_ville VARCHAR(100))
BEGIN
    DECLARE v_nom, v_prenom VARCHAR(100);
    DECLARE fin TINYINT DEFAULT 0;                      -- Variable locale utilisée pour stopper la boucle

    DECLARE curs_clients CURSOR
        FOR SELECT nom, prenom
        FROM Client
        WHERE ville = p_ville;

    DECLARE CONTINUE HANDLER FOR NOT FOUND SET fin = 1; -- Gestionnaire d'erreur pour la condition NOT FOUND

    OPEN curs_clients;                                    

    loop_curseur: LOOP                                                
        FETCH curs_clients INTO v_nom, v_prenom;

        IF fin = 1 THEN                                 -- Structure IF pour quitter la boucle à la fin des résultats
            LEAVE loop_curseur;
        END IF;
                   
        SELECT CONCAT(v_prenom, ' ', v_nom) AS 'Client';
    END LOOP;

    CLOSE curs_clients; 
END|
DELIMITER ;

CALL test_condition2('Houtsiplou');
CALL test_condition2('Bruxelles');
```

Je vous laisse le soin, si vous le désirez, de réécrire cette procédure en utilisant une boucle `WHILE` ou une boucle `REPEAT`. C'est un excellent exercice ! :)

Notez que ce n'est pas la seule solution pour arrêter la boucle. On pourrait par exemple compter le nombre de lignes récupérées, grâce à la fonction `FOUND_ROWS()` (dont on a déjà parlé dans le [chapitre sur les fonctions](http://zestedesavoir.com/tutoriels/688/administrez-vos-bases-de-donnees-avec-mysql/950/fonctions-nombres-chaines-et-agregats/3943/rappels-et-introduction/#3-quelques-fonctions-generales)). Cependant, cette utilisation du gestionnaire d'erreur est la solution la plus utilisée, et je la trouve personnellement très élégante.


## Le cas des booléens chez MySQL

Un booléen, en informatique, est un type de donnée pouvant prendre deux états : vrai, ou faux.
MySQL ne propose pas ce type de données. Avec MySQL, on représente la valeur "vrai" par 1, et la valeur "faux" par 0.

**Démonstration** :


```sql
SELECT 1 = 1, 1 = 2;    -- 1 = 1 est vrai, bien sûr. Contrairement à 1 = 2 (si si !)
```




+-----+-----+
|1 = 1|1 = 2|
+=====+=====+
|1    |0    |
+-----+-----+





Par conséquent, pour représenter un booléen, on utilise en général un `TINYINT`, valant soit 0, soit 1. C'est ce que l'on a fait pour la colonne _paye_ de la table _Client_ par exemple.

Une autre conséquence, est que la structure `IF` que l'on utilise dans la procédure _test_condition2()_ peut être réécrite de la manière suivante :


```sql
IF fin THEN
    LEAVE loop_curseur;
END IF;
```

Et pour améliorer encore la lisibilité pour les données de ce genre, MySQL a créé plusieurs synonymes que l'on peut utiliser dans ces situations.


- `BOOL` et `BOOLEAN` sont synonymes de `TINYINT(1)`.
- `TRUE` est synonyme de 1.
- `FALSE` est synonyme de 0.


On peut donc réécrire la procédure _test_condition2()_ en utilisant ces synonymes.


```sql
DROP PROCEDURE test_condition2;
DELIMITER |
CREATE PROCEDURE test_condition2(IN p_ville VARCHAR(100))
BEGIN
    DECLARE v_nom, v_prenom VARCHAR(100);
    DECLARE fin BOOLEAN DEFAULT FALSE;                     -- On déclare fin comme un BOOLEAN, avec FALSE pour défaut

    DECLARE curs_clients CURSOR
        FOR SELECT nom, prenom
        FROM Client
        WHERE ville = p_ville;

    DECLARE CONTINUE HANDLER FOR NOT FOUND SET fin = TRUE; -- On utilise TRUE au lieu de 1

    OPEN curs_clients;                                    

    loop_curseur: LOOP                                                
        FETCH curs_clients INTO v_nom, v_prenom;

        IF fin THEN                                        -- Plus besoin de "= 1"
            LEAVE loop_curseur;
        END IF;
                   
        SELECT CONCAT(v_prenom, ' ', v_nom) AS 'Client';
    END LOOP;

    CLOSE curs_clients; 
END|
DELIMITER ;
```

*[SGBD]: Système de Gestion de Bases de Données