Au-delà de la légendaire paresse des informaticiens, bien contents de ne pas devoir retaper la même requête encore et encore, les vues sont utilisées pour différentes raisons, dont voici les principales.


# Clarification et facilitation des requêtes

Lorsqu'une requête implique un grand nombre de tables, ou nécessite des fonctions, des regroupements, etc., même une bonne indentation ne suffit pas toujours à rendre la requête claire. Qui plus est, plus la requête est longue et complexe, plus l'ajout de la moindre condition peut se révéler pénible. 
Avoir une vue pour des requêtes complexes permet de simplifier et de clarifier les requêtes. 

**Exemple** : on veut savoir quelles espèces rapportent le plus, année après année. Comme c'est une question importante pour le développement de l'élevage, et qu'elle reviendra souvent, on crée une vue, que l'on pourra interroger facilement.


```sql
CREATE OR REPLACE VIEW V_Revenus_annee_espece
AS SELECT YEAR(date_reservation) AS annee, Espece.id AS espece_id, SUM(Adoption.prix) AS somme, COUNT(Adoption.animal_id) AS nb
FROM Adoption
INNER JOIN Animal ON Animal.id = Adoption.animal_id
INNER JOIN Espece ON Animal.espece_id = Espece.id
GROUP BY annee, Espece.id;
```
 

Avec des requêtes toutes simples, on peut maintenant obtenir des informations résultant de requêtes complexes, par exemple :

**1.** Les revenus obtenus par année


```sql
SELECT annee, SUM(somme) AS total
FROM V_Revenus_annee_espece
GROUP BY annee;
```




+-----+-------+
|annee|total  |
+=====+=======+
|2007 |2315.00|
+-----+-------+
|2008 |3565.00|
+-----+-------+
|2009 |400.00 |
+-----+-------+
|2010 |340.00 |
+-----+-------+
|2011 |1755.00|
+-----+-------+
|2012 |3045.00|
+-----+-------+





**2.** Les revenus obtenus pour chaque espèce, toutes années confondues


```sql
SELECT Espece.nom_courant AS espece, SUM(somme) AS total
FROM V_Revenus_annee_espece
INNER JOIN Espece ON V_Revenus_annee_espece.espece_id = Espece.id
GROUP BY espece;
```




+-----------------+-------+
|espece           |total  |
+=================+=======+
|Chat             |6480.00|
+-----------------+-------+
|Chien            |2400.00|
+-----------------+-------+
|Perroquet amazone|2100.00|
+-----------------+-------+
|Rat brun         |20.00  |
+-----------------+-------+
|Tortue d'Hermann |420.00 |
+-----------------+-------+





**3.** Les revenus moyens générés par la vente d'un individu de l'espèce


```sql
SELECT Espece.nom_courant AS espece, SUM(somme)/SUM(nb) AS moyenne
FROM V_Revenus_annee_espece
INNER JOIN Espece ON V_Revenus_annee_espece.espece_id = Espece.id
GROUP BY espece;
```




+-----------------+----------+
|espece           |moyenne   |
+=================+==========+
|Chat             |720.000000|
+-----------------+----------+
|Chien            |342.857143|
+-----------------+----------+
|Perroquet amazone|700.000000|
+-----------------+----------+
|Rat brun         |10.000000 |
+-----------------+----------+
|Tortue d'Hermann |140.000000|
+-----------------+----------+






# Création d'une interface entre l'application et la base de données

Lorsque l'on a une base de données exploitée par une application (écrite dans un langage de programmation comme Java, ou PHP par exemple), c'est souvent dans cette application que sont construites les requêtes qui vont insérer, modifier, et sélectionner les données de la base.
Si pour une raison ou une autre (mauvaise conception de la base de données au départ, modèle de données qui s’étend fortement,…) la structure des tables de la base change, il faut réécrire également l'application pour prendre en compte les modifications nécessaires pour les requêtes.

Cependant, si l'on a utilisé des vues, on peut éviter de réécrire toutes ces requêtes, ou du moins limiter le nombre de requêtes à réécrire. Si les requêtes sont faites sur des vues, il suffit en effet de modifier la définition de ces vues pour qu'elles fonctionnent avec la nouvelle structure.

**Exemple**

On a créé une vue _V_Client_, qui permet de voir le contenu de notre table _Client_ (sauf la date de naissance, ajoutée après la définition de la vue).

Si un beau jour on décidait de stocker les adresses postales dans une table à part, il faudrait modifier la structure de notre base. Au lieu d'une seule table _Client_, on en aurait deux : _Client_(_id, nom, prenom, date_naissance, email, adresse_id_) et _Adresse_(_id, rue_numero, code_postal, ville, pays_).

Pour que l'application puisse continuer à sélectionner les personnes et leur adresse sans qu'on doive la modifier, il suffirait de changer la requête définissant la vue :


```sql
CREATE OR REPLACE VIEW V_Client     -- le OR REPLACE indispensable (ou on utilise ALTER VIEW)
AS SELECT Client.id, nom, prenom, rue_numero AS adresse, code_postal, ville, pays, email, date_naissance
FROM Client
LEFT JOIN Adresse ON Client.adresse_id = Adresse.id -- LEFT JOIN au cas où certains clients n'auraient pas d'adresse définie
```


Le changement de structure de la base de données serait ainsi transparent pour l'application (du moins en ce qui concerne la sélection des clients) !


# Restriction des données visibles par les utilisateurs

La gestion des utilisateurs et de leurs droits fera l'objet d'un prochain chapitre. Sachez simplement que pour chaque utilisateur, il est possible de définir des droits particuliers pour chaque table et chaque vue (entre autres choses). 
On peut par exemple autoriser l'utilisateur _A_ à faire des requêtes d'insertion, modification, suppression et sélection (le fameux CRUD) sur la table _T1_, mais n'autoriser l'utilisateur _B_ qu'à faire des sélections sur la table _T1_.
Et imaginons qu'il existe aussi un utilisateur _C_, auquel on veut donner l'autorisation de faire des requêtes de sélection sur la table _T1_, mais auquel il faudrait cacher certaines colonnes qui contiennent des données sensibles, ou certaines lignes auxquelles il ne devrait pas accéder. Il suffit de créer la vue _V_T1_, n'ayant accès qu'aux colonnes/lignes "publiques" de la table _T1_, et de donner à _C_ les droits sur la vue _V_T1_, mais pas sur la table _T1_.

**Exemple**

Le stagiaire travaillant dans notre élevage s'occupe exclusivement des chats, et ne doit pas avoir accès aux commentaires. On ne lui donne donc pas accès à la table _Animal_, mais à une vue _V_Animal_stagiaire_ créée de la manière suivante :


```sql
CREATE VIEW V_Animal_stagiaire
AS SELECT id, nom, sexe, date_naissance, espece_id, race_id, mere_id, pere_id, disponible
FROM Animal
WHERE espece_id = 2;
```

*[CRUD]: Create Read Update Delete